unit Main;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  StdCtrls, ExtCtrls, Forms, tsxTypes, tSXPluginHelper, Buttons, ComCtrls;

type
  TfrmMain = class(TtsxForm)
    Label1: TLabel;
    cbSelection: TComboBox;
    btnArray: TBitBtn;
    btnRemove: TBitBtn;
    pcArrayGenerator: TPageControl;
    tsSelected: TTabSheet;
    tsArray: TTabSheet;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    eZ: TEdit;
    eY: TEdit;
    eX: TEdit;
    eXSize: TEdit;
    eYSize: TEdit;
    eZSize: TEdit;
    Label4: TLabel;
    Label27: TLabel;
    Label11: TLabel;
    Label10: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label21: TLabel;
    Label20: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label8: TLabel;
    lEstimated: TLabel;
    eZCopies: TEdit;
    eYCopies: TEdit;
    eXCopies: TEdit;
    eXDist: TEdit;
    eYDist: TEdit;
    eZDist: TEdit;
    eZDelta: TEdit;
    eYDelta: TEdit;
    eXDelta: TEdit;
    eXRot: TEdit;
    eXRotDelta: TEdit;
    eYRot: TEdit;
    eYRotDelta: TEdit;
    eZRot: TEdit;
    eZRotDelta: TEdit;
    Label9: TLabel;
    Label24: TLabel;
    eXScale: TEdit;
    eYScale: TEdit;
    eZScale: TEdit;
    eXScaleDelta: TEdit;
    eYScaleDelta: TEdit;
    eZScaleDelta: TEdit;
    Label2: TLabel;
    cbCopies: TCheckBox;
    cbDistance: TCheckBox;
    cbDelta: TCheckBox;
    cbRot: TCheckBox;
    cbRotDelta: TCheckBox;
    cbScale: TCheckBox;
    cbScaleDelta: TCheckBox;
    Bevel3: TBevel;
    Label3: TLabel;
    eFaces: TEdit;
    Label25: TLabel;
    eVertices: TEdit;
    btnRender: TBitBtn;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure ReadSelectionInfo(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnArrayClick(Sender: TObject);
    procedure ValidateInteger(Sender: TObject);
    procedure ValidateFloat(Sender: TObject);
    procedure ValidateAngle(Sender: TObject);
    procedure RemoveLast(Sender: TObject);
    procedure ToggleLock(Sender: TObject);
    procedure btnRenderClick(Sender: TObject);
    procedure XValueChange(Sender: TObject);
  private
    FSelectedObject : tsxGNODE;
    FResultGroup : tsxGROUP;
    procedure RefreshObjectList;
    procedure ClearObjectList;
    procedure LockControls(Lock: Boolean; srcCtrl, LockCtrl1, LockCtrl2: TEdit);
    procedure RefreshDependents(Ctrl : TEdit);
    procedure UpdateCtrls(srcCtrl, LockCtrl1, LockCtrl2: TEdit);
  public
    Closing: boolean; //flag to keep track of whether we are closing the tsx, or if truespace is
    procedure SelectionCallback; //this is invoked when a new object is selected in trueSpace
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

uses tsxObjectClass, dialogs, Progress;

procedure SelectionCB; cdecl; //procedure which is set as the selection callback
begin
  //just pass through to form's selection callback method
  //(this is just a little trick to use a normal (non object method)
  // procedure as a callback, and have it interface with a form method)
  if Assigned(frmMain) and
    not frmMain.Closing then
      frmMain.SelectionCallback;
end;

///////////////////////////// Form Methods and Functions ///////////////////////////////
procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  //if the user closes the plugin manually, we ask trueSpace to deactivate it
  Closing := True;
  //tell truespace to deactivate this plugin
  tsxForceLeftClick(tSXPluginHost.tsxID);
  //clear the object list
  ClearObjectList;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  Closing := False;
  //Refresh the list of scene objects every time we display the plugin
  RefreshObjectList;
  //jump to the currently selected object
  SelectionCallback;
  ReadSelectionInfo(cbSelection);
  //enable selection callback
  tsxSelectionChangedCB(tSXPluginHost.tsxID, SelectionCB);
end;

procedure TfrmMain.RefreshObjectList;
var
  pNode : tsxGNODE;
  oNode : TtsxObjectClass;
begin
  //First, clear the existing object list
  ClearObjectList;
  //traverse the scene graph, and add an item for each top-level object
  with cbSelection.Items do
    begin
    //traverse the scene
    pNode := tsxSceneGetFirstNode;
    while (tsxIsGNode(pNode) = e_tsxTRUE) do
      begin
      //Add top-level GNodes to the selection list
      //Keep adding until we run out of nodes in scene.
      oNode := TtsxObjectClass.Create(pNode);
      cbSelection.Items.AddObject(oNode.Name, oNode);
      pNode := tsxGNodeGetNext(pNode);
      end;
    end;
end;

procedure TfrmMain.ClearObjectList;
//Since the selection combo box contains a list of objects, rather than just
//strings, we must free each of the objects from that list before we clear it,
//or else we will cause a memory leak.
var
  iIndex : integer;
begin
  for iIndex := 0 to (cbSelection.Items.Count - 1) do
    begin
    //free all allocated TtsxObjectClass objects from the list
    if Assigned(cbSelection.Items.Objects[iIndex]) then
      TtsxObjectClass(cbSelection.Items.Objects[iIndex]).Free;
    end;
  //Now it's safe to clear the actual list
  cbSelection.Items.Clear;
end;

procedure TfrmMain.ReadSelectionInfo(Sender: TObject);
var
  pNode : tsxGNODE;
begin
  if not Closing then
    begin
    //read the tsx object pointer from the combo box which was selected
    pNode := TtsxObjectClass(cbSelection.Items.Objects[cbSelection.ItemIndex]).ID;
    //Make sure the object is selected
    if (tsxIsSelectable(pNode) = e_tsxTRUE) then
      begin
      if (tsxIsSelected(pNode) = e_tsxFALSE ) then
        //need to select the object
        begin
        //disable callback to prevent trueSpace from recursing the selection callback
        tsxSelectionChangedCB(tSXPluginHost.tsxID, nil);
        //select the new object
        tsxSelectSobj(pNode, e_tsxSELECT, e_tsxFALSE);
        //Re-enable callback
        tsxSelectionChangedCB(tSXPluginHost.tsxID, SelectionCB);
        //Find out which object is currently selected
        SelectionCallback;
        end;
      FSelectedObject := pNode;
      btnArray.Enabled := True;
      end;
    end;
end;

procedure TfrmMain.SelectionCallback;
//this is called whenever the plugin is active, and another truespace tool is used to
//change the current selection. Also used anytime we need to to jump to the
//currently selected object
var
  iIndex : integer;
  pCurrSelect : tsxGNODE;
  oSelected : TtsxObjectClass;
  bbBounds : CtsxBBox3f;
begin
  try
    //Clear out all of the display fields
    eFaces.Text    := 'n/a';
    eVertices.Text := 'n/a';
    eX.Text        := 'n/a';
    eY.Text        := 'n/a';
    eZ.Text        := 'n/a';
    //Find the currently selected object (this will generate an access violation
    //if no objects are actually selected)
    pCurrSelect := tsxGetCurrentSelection;
    for iIndex := 0 to (cbSelection.Items.Count - 1) do
      if (TtsxObjectClass(cbSelection.Items.Objects[iIndex]).ID = pCurrSelect) then
        cbSelection.ItemIndex := iIndex;
    //Show a little information about the currently selected item
    oSelected := TtsxObjectClass(cbSelection.Items.Objects[cbSelection.ItemIndex]);
    eFaces.Text    := IntToStr(oSelected.Faces);
    eVertices.Text := IntToStr(oSelected.Vertices);
    eX.Text        := FloatToStr(oSelected.XPos);
    eY.Text        := FloatToStr(oSelected.YPos);
    eZ.Text        := FloatToStr(oSelected.ZPos);
    tsxGNodeGetBBox(pCurrSelect, bbBounds);
    eXSize.Text    := FloatToStr(bbBounds.maxbounds.x - bbBounds.minbounds.x);
    eYSize.Text    := FloatToStr(bbBounds.maxbounds.y - bbBounds.minbounds.y);
    eZSize.Text    := FloatToStr(bbBounds.maxbounds.z - bbBounds.minbounds.z);
    //Enable the "Make Array" button
    btnArray.Enabled := True;
  except
    //ignore a selection error here (no need to raise an exception if no objects exist)
    if cbSelection.Items.Count > 0 then
      begin
      cbSelection.ItemIndex := 0;
      ReadSelectionInfo(cbSelection);
      end;
  end;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  frmMain := nil;
end;

procedure TfrmMain.btnArrayClick(Sender: TObject);
//Here is where we actually make the object array
//This can be considered the "meat" of the plugin.
var
  XCopies, YCopies, ZCopies: integer;
  XDist, YDist, ZDist: single;
  XDelta, YDelta, ZDelta: single;
  XRot, YRot, ZRot: single;
  XRotDelta, YRotDelta, ZRotDelta: single;
  XScale, YScale, ZScale: single;
  XScaleDelta, YScaleDelta, ZScaleDelta: single;
  WasTerminated: Boolean;
begin
  //Set the Frame of reference variables
  //Set the other array parameters
  XCopies   := StrToInt( eXCopies.Text );
  YCopies   := StrToInt( eYCopies.Text );
  ZCopies   := StrToInt( eZCopies.Text );
  if XCopies < 0 then XCopies := 0;
  if YCopies < 0 then YCopies := 0;
  if ZCopies < 0 then ZCopies := 0;
  XDist       := StrToFloat( eXDist.Text     );
  YDist       := StrToFloat( eYDist.Text     );
  ZDist       := StrToFloat( eZDist.Text     );
  XDelta      := StrToFloat( eXDelta.Text    );
  YDelta      := StrToFloat( eYDelta.Text    );
  ZDelta      := StrToFloat( eZDelta.Text    );
  XRot        := StrToFloat( eXRot.Text      );
  YRot        := StrToFloat( eYRot.Text      );
  ZRot        := StrToFloat( eZRot.Text      );
  XRotDelta   := StrToFloat( eXRotDelta.Text );
  YRotDelta   := StrToFloat( eYRotDelta.Text );
  ZRotDelta   := StrToFloat( eZRotDelta.Text );
  XScale      := StrToFloat( eXScale.Text );
  YScale      := StrToFloat( eYScale.Text );
  ZScale      := StrToFloat( eZScale.Text );
  XScaleDelta := StrToFloat( eXScaleDelta.Text );
  YScaleDelta := StrToFloat( eYScaleDelta.Text );
  ZScaleDelta := StrToFloat( eZScaleDelta.Text );
  //Do the array function

  //disable callback to prevent trueSpace from recursing the selection callback
  tsxSelectionChangedCB(tSXPluginHost.tsxID, nil);

  //Run the generation function in Progress.Pas
  if GenerateArray(XCopies, YCopies, ZCopies, XDist, YDist, ZDist, XDelta,
        YDelta, ZDelta, XRot, YRot, ZRot, XRotDelta, YRotDelta, ZRotDelta,
        XScale, YScale, ZScale, XScaleDelta, YScaleDelta, ZScaleDelta,
        FSelectedObject, FResultGroup, WasTerminated) then
    begin
    //Enable the Remove button if it wasnt terminated
    btnRemove.Enabled := (not WasTerminated);
    //Name the new object group
    if not WasTerminated then
      with TtsxObjectClass.Create(FResultGroup) do
        begin
        Name := 'An Object Array - built with the ObjArrayDemo plugin';
        Free;
        end;
    end
    else
      begin
      ShowMessage('An error occured during array generation.' + #10#13 +
                  'Object Array Generator will now close.');
      tsxForceLeftClick(tSXPluginHost.tsxID);
      Exit;
      end;

  //Re-enable callback
  tsxSelectionChangedCB(tSXPluginHost.tsxID, SelectionCB);
end;

procedure TfrmMain.ValidateInteger(Sender: TObject);
begin
  try
    StrToInt(TEdit(Sender).Text);
    lEstimated.Caption := IntToStr(StrToInt(eXCopies.Text) *
                                   StrToInt(eYCopies.Text) *
                                   StrToInt(eZCopies.Text));
  except
    ShowMessage('You must enter a valid whole number.');
    TEdit(Sender).SetFocus;
  end;
end;

procedure TfrmMain.ValidateFloat(Sender: TObject);
begin
  try
    StrToFloat(TEdit(Sender).Text);
  except
    ShowMessage('You must enter a valid number.');
    TEdit(Sender).SetFocus;
  end;
end;

procedure TfrmMain.ValidateAngle(Sender: TObject);
  function ReduceAngle(AAngle : single): single;
  var
    Positive: boolean;
  begin
    Positive := AAngle > 0;
    AAngle := abs( AAngle );
    if AAngle > 360 then
      begin
      AAngle := AAngle - 360;
      result := ReduceAngle(AAngle)
      end
      else
        result := AAngle;
    if not Positive then result := 0 - result;
  end;
var
  Angle: single;
begin
  try
    Angle := StrToFloat(TEdit(Sender).Text);
    TEdit(Sender).Text := FloatToStr(ReduceAngle(Angle));
  except
    ShowMessage('You must enter a valid angle.');
    TEdit(Sender).SetFocus;
  end;
end;

procedure TfrmMain.RemoveLast(Sender: TObject);
begin
  //disable callback to prevent trueSpace from recursing the selection callback
  tsxSelectionChangedCB(tSXPluginHost.tsxID, nil);
  //select the new object
  tsxSelectSobj(FSelectedObject, e_tsxSELECT, e_tsxFALSE);
  //Re-enable callback
  tsxSelectionChangedCB(tSXPluginHost.tsxID, SelectionCB);
  //Delete the last created object
  tsxSobjDelete(FResultGroup);
  //Reset the result group pointer
  FResultGroup := nil;
  //disable the "Remove" button
  btnRemove.Enabled := False;
  //Refresh the scene to reflect that the object is removed
  tsxSceneDraw;
end;

procedure TfrmMain.LockControls(Lock: Boolean; srcCtrl, LockCtrl1, LockCtrl2: TEdit);
begin
  LockCtrl1.Enabled := not Lock;
  LockCtrl2.Enabled := not Lock;
  LockCtrl1.ReadOnly := Lock;
  LockCtrl2.ReadOnly := Lock;
  LockCtrl1.TabStop := Lock;
  LockCtrl2.TabStop := Lock;
  if Lock then
    begin
    LockCtrl1.Color := clBtnFace;
    LockCtrl2.Color := clBtnFace;
    UpdateCtrls(srcCtrl, LockCtrl1, LockCtrl2);
    lEstimated.Caption := IntToStr(StrToInt(eXCopies.Text) *
                                   StrToInt(eYCopies.Text) *
                                   StrToInt(eZCopies.Text));
    end
    else
      begin
      LockCtrl1.Color := clWindow;
      LockCtrl2.Color := clWindow;
      end;
end;

procedure TfrmMain.ToggleLock(Sender: TObject);
begin
  case TCheckBox(Sender).Tag of
    1: LockControls(TCheckBox(Sender).Checked, eXCopies     , eYCopies     , eZCopies);
    2: LockControls(TCheckBox(Sender).Checked, eXDist       , eYDist       , eZDist);
    3: LockControls(TCheckBox(Sender).Checked, eXDelta      , eYDelta      , eZDelta);
    4: LockControls(TCheckBox(Sender).Checked, eXRot        , eYRot        , eZRot);
    5: LockControls(TCheckBox(Sender).Checked, eXRotDelta   , eYRotDelta   , eZRotDelta);
    6: LockControls(TCheckBox(Sender).Checked, eXScale      , eYScale      , eZScale);
    7: LockControls(TCheckBox(Sender).Checked, eXScaleDelta , eYScaleDelta , eZScaleDelta);
    end;
end;

procedure TfrmMain.btnRenderClick(Sender: TObject);
begin
  tsxCurrobjPhrender;
end;

procedure TfrmMain.UpdateCtrls(srcCtrl, LockCtrl1, LockCtrl2: TEdit);
begin
  LockCtrl1.Text := srcCtrl.Text;
  LockCtrl2.Text := srcCtrl.Text;
end;

procedure TfrmMain.RefreshDependents(Ctrl : TEdit);
begin
  case Ctrl.Tag of
    11: if cbCopies.Checked     then UpdateCtrls(Ctrl, eYCopies     , eZCopies);
    12: if cbDistance.Checked   then UpdateCtrls(Ctrl, eYDist       , eZDist);
    13: if cbDelta.Checked      then UpdateCtrls(Ctrl, eYDelta      , eZDelta);
    14: if cbRot.Checked        then UpdateCtrls(Ctrl, eYRot        , eZRot);
    15: if cbRotDelta.Checked   then UpdateCtrls(Ctrl, eYRotDelta   , eZRotDelta);
    16: if cbScale.Checked      then UpdateCtrls(Ctrl, eYScale      , eZScale);
    17: if cbScaleDelta.Checked then UpdateCtrls(Ctrl, eYScaleDelta , eZScaleDelta);
    end;
end;

procedure TfrmMain.XValueChange(Sender: TObject);
begin
  if TEdit(Sender).Tag > 10 then
    RefreshDependents(TEdit(Sender));
end;

end.
