unit Progress;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  StdCtrls, ExtCtrls, Forms, tsxTypes, tSXPluginHelper, Buttons, ComCtrls;

type
  TfrmProgress = class(TtsxForm)
    ProgressBar: TProgressBar;
    btnCancel: TBitBtn;
    Label1: TLabel;
    eProgressText: TLabel;
    btnDone: TBitBtn;
    GenerateTrigger: TTimer;
    procedure InitGenerate(Sender: TObject);
    procedure Generate(Sender: TObject);
    procedure CancelGeneration(Sender: TObject);
    procedure SetupTimer(Sender: TObject);
  private
    FSource      : tsxGNODE;
    FResult      : tsxGNODE;
    FXCopies     : integer;
    FYCopies     : integer;
    FZCopies     : integer;
    FXDist       : single;
    FYDist       : single;
    FZDist       : single;
    FXDelta      : single;
    FYDelta      : single;
    FZDelta      : single;
    FXRot        : single;
    FYRot        : single;
    FZRot        : single;
    FXRotDelta   : single;
    FYRotDelta   : single;
    FZRotDelta   : single;
    FXScale      : single;
    FYScale      : single;
    FZScale      : single;
    FXScaleDelta : single;
    FYScaleDelta : single;
    FZScaleDelta : single;
    FTerminated: Boolean; //this allows the Cancel button to function
    function MakeTranslatedCopy(pPosn, pRotn, pScale: CtsxVector3f): tsxGNODE;
    procedure GlueCopy(pCopy: tsxGNODE);
  end;

var
  frmProgress: TfrmProgress;

//This is the interface to the main processing function. It returns True if no
//errors were encountered. If an error occurred (an exception was raised),
//returns False.  A User "cancel" is not considered an error.
//The actual generation occurs in the OnActivate event of the progress dialog
function GenerateArray(XCopies, YCopies, ZCopies: integer;
  XDist, YDist, ZDist, XDelta, YDelta, ZDelta, XRot, YRot, ZRot,
  XRotDelta, YRotDelta, ZRotDelta, XScale, YScale, ZScale,
  XScaleDelta, YScaleDelta, ZScaleDelta: single; Source: tsxGNODE;
  var Destination: tsxGNODE; var Terminated: Boolean): Boolean;

implementation

{$R *.DFM}

function GenerateArray;
begin
  result := True;
  frmProgress := TfrmProgress.CreateParented(tsxGetMainViewHwnd);
  try
    try
      with frmProgress do
        begin
        Caption := '';
        FXCopies     := XCopies;
        FYCopies     := YCopies;
        FZCopies     := ZCopies;
        FXDist       := XDist;
        FYDist       := YDist;
        FZDist       := ZDist;
        FXDelta      := XDelta;
        FYDelta      := YDelta;
        FZDelta      := ZDelta;
        FXRot        := XRot;
        FYRot        := YRot;
        FZRot        := ZRot;
        FXRotDelta   := XRotDelta;
        FYRotDelta   := YRotDelta;
        FZRotDelta   := ZRotDelta;
        FXScale      := XScale;
        FYScale      := YScale;
        FZScale      := ZScale;
        FXScaleDelta := XScaleDelta;
        FYScaleDelta := YScaleDelta;
        FZScaleDelta := ZScaleDelta;
        FSource      := Source;
        FResult      := Destination;
        Terminated   := False;
        if (ShowModal = mrOK) then
          Destination := FResult
          else
            Terminated := True;
        end;
    except
      result := False;
    end;
  finally
    frmProgress.Close;
    frmProgress.Free;
    frmProgress := nil;
  end;
end;

procedure TfrmProgress.InitGenerate(Sender: TObject);
begin
  //initialize the progress bar
  eProgressText.Caption := 'Initializing...';
  ProgressBar.Max := ((FXCopies + 1) * (FYCopies + 1) * (FZCopies + 1)) + 2;
  ProgressBar.Min := 0;
  ProgressBar.Position := 0;
  ProgressBar.Step := 1;
end;

function TfrmProgress.MakeTranslatedCopy(pPosn, pRotn, pScale: CtsxVector3f): tsxGNODE;
var
  pCopy : tsxGNODE;
  pCenter, pAxis : CtsxVector3f;
  fAngle: single;
begin
  //Make a copy of the source object
  tsxGNodeCopy(FSource, pCopy);

  //Make the center of rotation the object axis
  tsxGNodeGetAxesPosition(pCopy, pCenter);

  //Rotate the copy along the X axis
  if pRotn.x > 0 then
    begin
    pAxis.x := 1;
    pAxis.y := 0;
    pAxis.z := 0;
    fAngle := tsxDeg2Rad(pRotn.x);
    tsxGNodeRotate(pCopy, pCenter, pAxis, fAngle, e_tsxWorldFrame);
    end;

  //Rotate the copy along the Y axis
  if pRotn.y > 0 then
    begin
    pAxis.x := 0;
    pAxis.y := 1;
    pAxis.z := 0;
    fAngle := tsxDeg2Rad(pRotn.y);
    tsxGNodeRotate(pCopy, pCenter, pAxis, fAngle, e_tsxWorldFrame);
    end;

  //Rotate the copy along the Z axis
  if pRotn.z > 0 then
    begin
    pAxis.x := 0;
    pAxis.y := 0;
    pAxis.z := 1;
    fAngle := tsxDeg2Rad(pRotn.z);
    tsxGNodeRotate(pCopy, pCenter, pAxis, fAngle, e_tsxWorldFrame);
    end;

  //Move the copy into position
  tsxGNodeTranslate(pCopy, pPosn, e_tsxWorldFrame);

  //Scale the copy
  tsxGNodeScale(pCopy, pScale, e_tsxModelFrame);

  //return the copy as the function result
  result := pCopy;
end;

procedure TfrmProgress.GlueCopy(pCopy: tsxGNODE);
begin
  //select the array group
  tsxSelectSobj(FResult, e_tsxSELECT, e_tsxTRUE);
  //glue the last object to the group
  FResult := tsxGroupAtCurrobj(pCopy);
end;

//Here is the actual routine which generates the array. Its basically just a
//big, nested loop.
procedure TfrmProgress.Generate(Sender: TObject);
var
  iXIndex, iYIndex, iZIndex : integer;
  fCurrXDelta, fCurrYDelta, fCurrZDelta : single;
  fCurrXRotDelta, fCurrYRotDelta, fCurrZRotDelta : single;
  fCurrXScaleDelta, fCurrYScaleDelta, fCurrZScaleDelta : single;
  pPosn : CtsxVector3f;
  pRotn : CtsxVector3f;
  pScale: CtsxVector3f;
begin
  //Turn of the trigger timer
  GenerateTrigger.Enabled := False;

  //Initialize running Delta values
  fCurrXDelta      := 0;
  fCurrYDelta      := 0;
  fCurrZDelta      := 0;
  fCurrXRotDelta   := 0;
  fCurrYRotDelta   := 0;
  fCurrZRotDelta   := 0;
  fCurrXScaleDelta := 0;
  fCurrYScaleDelta := 0;
  fCurrZScaleDelta := 0;

  //initialize the result group to be a copy of the original object
  tsxGNodeCopy(FSource, FResult);

  ProgressBar.StepIt;
  Application.ProcessMessages;

  //Cycle through the number of X Copies
  for iXIndex := 0 to FXCopies do
    begin
    //Cycle through the number of X Copies
    for iYIndex := 0 to FYCopies do
      begin
      //Cycle through the number of X Copies
      for iZIndex := 0 to FZCopies do
        begin
        if not FTerminated then  //check to make sure the user didn't press cancel
          begin
          //Refresh the progress display
          ProgressBar.StepIt;
          eProgressText.Caption := 'Making Copy #' + IntToStr(ProgressBar.Position - 1) +
                                   ' of ' + IntToStr(ProgressBar.Max - 2);
          Application.ProcessMessages;
          //calculate the position and rotation of this array element
          pPosn.x  := (iXIndex * FXDist)  + fCurrXDelta;
          pPosn.y  := (iYIndex * FYDist)  + fCurrYDelta;
          pPosn.z  := (iZIndex * FZDist)  + fCurrZDelta;
          pRotn.x  := (iXIndex * FXRot)   + fCurrXRotDelta;
          pRotn.y  := (iYIndex * FYRot)   + fCurrYRotDelta;
          pRotn.z  := (iZIndex * FZRot)   + fCurrZRotDelta;
          pScale.x := (iXIndex * FXScale) + fCurrXScaleDelta;
          pScale.y := (iYIndex * FYScale) + fCurrYScaleDelta;
          pScale.z := (iZIndex * FZScale) + fCurrZScaleDelta;
          //make a copy, and move/rotate it
          GlueCopy(MakeTranslatedCopy(pPosn, pRotn, pScale));
          //increment the Delta calculation counters
          fCurrZDelta := fCurrZDelta + (iZIndex * FZDelta);
          fCurrZRotDelta := fCurrZRotDelta + (iZIndex * FZRotDelta);
          fCurrZScaleDelta := fCurrZScaleDelta + (iZIndex * FZScaleDelta);
          //Allow window message que to purge, so that we get notified if Cancel was clicked
          //and Windows does not think we are a runaway process
          Application.ProcessMessages;
          end;
        end;
      //calculate translation deltas for this loop iteration
      fCurrZDelta := 0;
      fCurrYDelta := fCurrYDelta + (iYIndex * FYDelta);
      fCurrZRotDelta := 0;
      fCurrYRotDelta := fCurrYRotDelta + (iYIndex * FYRotDelta);
      fCurrZScaleDelta := 0;
      fCurrYScaleDelta := fCurrYScaleDelta + (iYIndex * FYScaleDelta);
      end;
    //calculate translation deltas for this loop iteration
    fCurrYDelta := 0;
    fCurrXDelta := fCurrXDelta + (iXIndex * FXDelta);
    fCurrYRotDelta := 0;
    fCurrXRotDelta := fCurrXRotDelta + (iXIndex * FXRotDelta);
    fCurrYScaleDelta := 0;
    fCurrXScaleDelta := fCurrXScaleDelta + (iXIndex * FXScaleDelta);
    end;

  eProgressText.Caption := 'Cleaning up...';

  if not FTerminated then
    begin
    //select new object
    tsxSelectSobj(FResult, e_tsxSELECT, e_tsxTRUE);
    //Work around an apparent bug where the scene graph isn't updated properly
    //with the new object created in the tsx
    tsxCurrobjCopy;
    tsxSobjDelete(FResult);
    FResult := tsxGetCurrentSelection;
    //redraw
    tsxSceneDraw;

    ProgressBar.StepIt;
    eProgressText.Caption := 'Finished.';

    //Close progress display
    btnCancel.Visible := False;
    btnDone.Visible := True;
    end
    else
      begin
      //Make sure the old object is selected
      tsxSelectSobj(FSource, e_tsxSELECT, e_tsxFALSE);
      //Array Generation was canceled, so we delete the created object
      tsxSobjDelete(FResult);
      //redraw
      tsxSceneDraw;

      ProgressBar.StepIt;
      eProgressText.Caption := 'Generation Canceled.';

      //Close progress display
      Application.ProcessMessages;
      Self.ModalResult := mrCancel;
      end;
end;

procedure TfrmProgress.CancelGeneration(Sender: TObject);
begin
  //Set the Terminated flag, so the loop will break out
  FTerminated := True;
  Self.ModalResult := mrCancel;
end;

procedure TfrmProgress.SetupTimer(Sender: TObject);
begin
  //This just enables the timer, which will only trigger once. It is a kludgy
  //work-around to getting the cancel button working (you cant close a form during
  //form creation or activation)
  GenerateTrigger.Enabled := True;
end;

end.
