unit tsxObjectClass;

interface

uses
  SysUtils, Windows, Messages, Classes, tsxTypes, tSXPluginHelper;

Type
  //This is a wrapper class around a selectable object. It is used
  //to provide easier access to the attributes of a trueSpace object.
  TtsxObjectClass = class(TObject)
  private
    FtsxObject : tsxSOBJ;
    FtsxFRAMEID : tsxFRAMEID;
    function GetName : string;
    procedure SetName(AName : string);
    function GetXPos : single;
    function GetYPos : single;
    function GetZPos : single;
    procedure SetXPos(APos: single);
    procedure SetYPos(APos: single);
    procedure SetZPos(APos: single);
  public
    constructor Create(AObject: tsxSOBJ);
    function ID : tsxSObj;
    function Faces : integer;
    function Vertices : integer;
    property XPos : single read GetXPos write SetXPos;
    property YPos : single read GetYPos write SetYPos;
    property ZPos : single read GetZPos write SetZPos;
    property Name: string read GetName write SetName;
    property FrameID: tsxFRAMEID read FtsxFRAMEID write FtsxFRAMEID;
  end;


implementation

///////////////////////// TtsxObjectClass Methods and Functions ////////////////////////

constructor TtsxObjectClass.Create(AObject: tsxSOBJ);
begin
  FtsxObject := AObject;
  FtsxFRAMEID := e_tsxWorldFrame;
end;

function TtsxObjectClass.GetName : string;
var
  pName : PChar;
  iNameRetSz : integer;
begin
  result := 'Unsupported Object';
  if (tsxIsGNode(FtsxObject) = e_tsxTRUE) then
    begin
    //Get the objects name from trueSpace
    pName := AllocMem(SizeOf(char)*255);
    iNameRetSz := tsxGNodeGetName(FtsxObject, pName, 255);
    pName[iNameRetSz] := #0;
    Result := StrPas(pName);
    FreeMem(pName, SizeOf(char)*255);
    end;
end;

procedure TtsxObjectClass.SetName(AName : string);
var
  pName : PChar;
begin
  pName := AllocMem(SizeOf(char)*255);
  StrPCopy(pName, AName);
  tsxGNodeSetName(FtsxObject, pName);
  FreeMem(pName, SizeOf(char)*255);
end;

function TtsxObjectClass.ID : tsxSObj;
begin
  Result := FtsxObject;
end;

function TtsxObjectClass.Faces : integer;
begin
  if (tsxSobjGetType(FtsxObject) = e_tsxPOLYHEDRON) then
    result := tsxPolyhGetNbrFaces(FtsxObject)
    else result := 0;
end;

function TtsxObjectClass.Vertices : integer;
begin
  if (tsxSobjGetType(FtsxObject) = e_tsxPOLYHEDRON) then
    result := tsxPolyhGetNbrVxs(FtsxObject)
    else result := 0;
end;

function TtsxObjectClass.GetXPos : single;
var
  pAxisPosn : CtsxVector3f;
begin
  tsxGNodeGetAxesPosition(FtsxObject, pAxisPosn);
  result := pAxisPosn.x;
end;

function TtsxObjectClass.GetYPos : single;
var
  pAxisPosn : CtsxVector3f;
begin
  tsxGNodeGetAxesPosition(FtsxObject, pAxisPosn);
  result := pAxisPosn.y;
end;

function TtsxObjectClass.GetZPos : single;
var
  pAxisPosn : CtsxVector3f;
begin
  tsxGNodeGetAxesPosition(FtsxObject, pAxisPosn);
  result := pAxisPosn.z;
end;

procedure TtsxObjectClass.SetXPos(APos : single);
var
  pAxisPosn : CtsxVector3f;
begin
  pAxisPosn.x := APos;
  pAxisPosn.y := 0;
  pAxisPosn.z := 0;
  tsxGNodeTranslate(FtsxObject, pAxisPosn, FtsxFRAMEID);
end;

procedure TtsxObjectClass.SetYPos(APos : single);
var
  pAxisPosn : CtsxVector3f;
begin
  pAxisPosn.x := 0;
  pAxisPosn.y := APos;
  pAxisPosn.z := 0;
  tsxGNodeTranslate(FtsxObject, pAxisPosn, FtsxFRAMEID);
end;

procedure TtsxObjectClass.SetZPos(APos : single);
var
  pAxisPosn : CtsxVector3f;
begin
  pAxisPosn.x := 0;
  pAxisPosn.y := 0;
  pAxisPosn.z := APos;
  tsxGNodeTranslate(FtsxObject, pAxisPosn, FtsxFRAMEID);
end;

end.
