unit BrowseDialog;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  StdCtrls, ExtCtrls, Forms, tsxTypes, tSXPluginHelper, Buttons;

type
  TfrmBrowseDialog = class(TtsxForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    stName: TStaticText;
    stVertexCount: TStaticText;
    stFaceCount: TStaticText;
    stUVCount: TStaticText;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    lbFaces: TListBox;
    lbFaceUV: TListBox;
    lbFaceVertices: TListBox;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    stNormal1: TStaticText;
    stNormal2: TStaticText;
    stNormal3: TStaticText;
    lbVertices: TListBox;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    stVertexX: TStaticText;
    stVertexY: TStaticText;
    stVertexZ: TStaticText;
    lbUV: TListBox;
    Label15: TLabel;
    Label16: TLabel;
    stV: TStaticText;
    stU: TStaticText;
    Label17: TLabel;
    Label18: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure InspectVertex(Sender: TObject);
    procedure InspectFace(Sender: TObject);
    procedure InspectUV(Sender: TObject);
  private
    pObject2View : tsxSOBJ;
    procedure RefreshBrowser;
    procedure SetEnabledControls(Enable : Boolean);
  end;

var
  frmBrowseDialog: TfrmBrowseDialog;

implementation

{$R *.DFM}

procedure TfrmBrowseDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
  frmBrowseDialog := nil;
end;

procedure TfrmBrowseDialog.SetEnabledControls(Enable : Boolean);
begin
  lbFaces.Enabled := Enable;
  lbFaceUV.Enabled := Enable;
  lbFaceVertices.Enabled := Enable;
  lbVertices.Enabled := Enable;
  lbUV.Enabled := Enable;
  if Enable then
    begin
    lbFaces.ItemIndex := 0;
    InspectFace(lbFaces);
    lbVertices.ItemIndex := 0;
    InspectVertex(lbVertices);
    lbUV.ItemIndex := 0;
    InspectUV(lbUV);
    end;
end;

procedure TfrmBrowseDialog.RefreshBrowser;
var
  pObjectName : PChar;
  iNumVertices : integer;
  iNumFaces : integer;
  iNumUV : integer;
  iIndex : integer;
begin
  SetEnabledControls(False);
  //Get the current selection
  pObject2View := tsxGetCurrentSelection();
  //check to see if the current selection is a GNode
  if (tsxIsGNode(pObject2View) = e_tsxTRUE) then
    begin
    //Read the name of the object
    pObjectName := AllocMem(255 * sizeof(char));
    tsxGNodeGetName(pObject2View, pObjectName, 254);
    stName.Caption := StrPas(pObjectName);
    FreeMem(pObjectName);
    //read the number of vertices
    iNumVertices := tsxPolyhGetNbrVxs(pObject2View);
    stVertexCount.Caption := IntToStr(iNumVertices);
    //read the number of faces
    iNumFaces := tsxPolyhGetNbrFaces(pObject2View);
    stFaceCount.Caption := IntToStr(iNumFaces);
    //read the number of UV entries
    iNumUV := tsxPolyhGetNbrUV(pObject2View);
    stUVCount.Caption := IntToStr(iNumUV);
    //clear the list boxes
    lbFaces.Items.Clear;
    lbFaceUV.Items.Clear;
    lbFaceVertices.Items.Clear;
    lbVertices.Items.Clear;
    lbUV.Items.Clear;
    //load the vertex array listbox
    for iIndex := 0 to (iNumVertices - 1) do
      lbVertices.Items.Add(IntToStr(iIndex));
    //load the face listbox
    for iIndex := 0 to (iNumFaces - 1) do
      lbFaces.Items.Add(IntToStr(iIndex));
    //load the UV array listbox
    for iIndex := 0 to (iNumUV - 1) do
      lbUV.Items.Add(IntToStr(iIndex));
    SetEnabledControls(True);
    end;
end;

procedure TfrmBrowseDialog.FormCreate(Sender: TObject);
begin
  RefreshBrowser;
end;

procedure TfrmBrowseDialog.BitBtn1Click(Sender: TObject);
begin
  RefreshBrowser;
end;

procedure TfrmBrowseDialog.BitBtn2Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmBrowseDialog.InspectVertex(Sender: TObject);
var
  pVertexArray : PCtsxVector3fArray;
  iVertex : integer;
begin
  try
    //read the coordinates of the selected vertex
    pVertexArray := tsxPolyhGetVxArray(pObject2View);
    iVertex := lbVertices.ItemIndex;
    stVertexX.Caption := FloatToStr(pVertexArray[iVertex].x);
    stVertexY.Caption := FloatToStr(pVertexArray[iVertex].y);
    stVertexZ.Caption := FloatToStr(pVertexArray[iVertex].z);
  except
    SetEnabledControls(False);
  end;
end;

procedure TfrmBrowseDialog.InspectFace(Sender: TObject);
var
  pFaceArray : PCtsxFaceArray;
  iFace : integer;
  iNumVertices : integer;
  iIndex : integer;
begin
  try
    //read the properties of the selected face
    pFaceArray := tsxPolyhGetFaceptrArray(pObject2View);
    iFace := lbFaces.ItemIndex;
    iNumVertices := pFaceArray[iFace].nbrVxs;
    lbFaceUV.Items.Clear;
    lbFaceVertices.Items.Clear;
    //read the vertex array and UV array indices for the vertices on this face
    for iIndex := 0 to (iNumVertices - 1) do
      begin
      lbFaceVertices.Items.Add(IntToStr(pFaceArray[iFace].pFVxs[iIndex].vix));
      lbFaceUV.Items.Add(IntToStr(pFaceArray[iFace].pFVxs[iIndex].uvix));
      end;
    //read the vertex array indices which are used to compute the face normal
    stNormal1.Caption := FloatToStr(pFaceArray[iFace].normVxs[0]);
    stNormal2.Caption := FloatToStr(pFaceArray[iFace].normVxs[1]);
    stNormal3.Caption := FloatToStr(pFaceArray[iFace].normVxs[2]);
    //refresh the object wireframe, and render the selected face
    tsxCurrobjDraw();
    tsxPolyhPhrenderFace(pObject2View, iFace);
  except
    SetEnabledControls(False);
  end;
end;

procedure TfrmBrowseDialog.InspectUV(Sender: TObject);
var
  pUVArray : PCtsxUVArray;
  iUV : integer;
begin
  try
    //read the coordinates of the selected UV entry
    pUVArray := tsxPolyhGetUVArray(pObject2View);
    iUV := lbUV.ItemIndex;
    stU.Caption := FloatToStr(pUVArray[iUV].u);
    stV.Caption := FloatToStr(pUVArray[iUV].v);
  except
    SetEnabledControls(False);
  end;
end;

end.
