unit tSXPluginHelper;
/////////////////////////////////////////
// tSX Builder                         //
// "Helper" unit to manage some of the //
// particulars of a tSX plugin.        //
//                                     //
// This unit is the heart and soul of  //
// a tSX Builder plugin.               //
/////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, tsxTypes;

// Uncomment the following line to prevent linking the default resources into
//   the compiled plugin. This is not necessary, but will reduce the plugin size
//   by a very small amount. Only do this if you have already successfully included
//   your own custom bitmap and string resources.
//{$DEFINE NO_USE_DEFAULT_RESOURCES}

{$IFNDEF NO_USE_DEFAULT_RESOURCES}

  { Strings ID }
  Const IDS_DEFAULT_TOOLHINT = 00001;  { Sample trueSpace eXtension written in Delphi }

  { BMP ID }
  Const IDB_DEFAULT_EXTENSIONBMP = 00001;

  //include the standard bitmap and string resource defined above
  {$R tSXPluginHelper.RES}

{$ENDIF}

{ Extra DLL attachment constant.
  Needed because of the global plugin host object. }
Const dll_Process_Startup = 99;

type
  TPluginStatus = (e_tsxPLUGIN_ACTIVE, e_tsxPLUGIN_INACTIVE, e_tsxPLUGIN_IDLE);

  TtsxOnGetData = procedure(var tsxPlugReturn : integer);
  TtsxOnLeftClick = procedure(var tsxPlugReturn : integer);
  TtsxOnRightClick = procedure;
  TtsxOnDeactivate = procedure;
  TtsxDllNotify = procedure;

  TtSXPluginHost = class(TObject)
  private
    FtsxID : integer;
    FTitle : string;
    FAuthor : string;
    FBtnResID : short;
    FStrResID : short;
    FPlugState : TPluginStatus;
    FRequiredVersion : TtSVersion;
    FOwnedWindows : TStringList;
    FOnGetData    : TtsxOnGetData;
    FOnLeftClick  : TtsxOnLeftClick;
    FOnRightClick : TtsxOnRightClick;
    FOnDeactivate : TtsxOnDeactivate;
    FOnProcessAttach : TtsxDllNotify;
    FOnProcessDetach : TtsxDllNotify;
    FOnThreadAttach : TtsxDllNotify;
    FOnThreadDetach : TtsxDllNotify;
  public
    property tsxID : integer read FtsxID;
    property ButtonResID: short read FBtnResID write FBtnResID;
    property HelpResID: short read FStrResID write FStrResID;
    constructor Create;
    destructor Destroy; override;
    property OnGetData       : TtsxOnGetData    read FOnGetData       write FOnGetData;
    property OnLeftClick     : TtsxOnLeftClick  read FOnLeftClick     write FOnLeftClick;
    property OnRightClick    : TtsxOnRightClick read FOnRightClick    write FOnRightClick;
    property OnDeactivate    : TtsxOnDeactivate read FOnDeactivate    write FOnDeactivate;
    property OnProcessAttach : TtsxDllNotify    read FOnProcessAttach write FOnProcessAttach;
    property OnProcessDetach : TtsxDllNotify    read FOnProcessDetach write FOnProcessDetach;
    property OnThreadAttach  : TtsxDllNotify    read FOnThreadAttach  write FOnThreadAttach;
    property OnThreadDetach  : TtsxDllNotify    read FOnThreadDetach  write FOnThreadDetach;
    property Title : string read FTitle write FTitle;
    property Author : string read FAuthor write FAuthor;
    property OwnedWindows : TStringList read FOwnedWindows;
    property PluginState : TPluginStatus read FPlugState write FPlugState;
    property RequiredVersion : TtSVersion read FRequiredVersion write FRequiredVersion;
  end;

  TtsxCustomForm = class(TForm);

  TtsxToolForm = class(TtsxCustomForm)
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  end;

  TtsxForm = class(TtsxToolForm)
  private
    FAutoDrag: Boolean;
  protected
    procedure WMNCHitTest(var M: TWMNCHitTest); message wm_NCHitTest;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AutoDrag: Boolean read FAutoDrag write FAutoDrag;
  end;

function tsxGetData(var tsx_Data : tsxData; tsxID : integer): integer; cdecl;
function tsxOnLeftClick : integer; cdecl;
procedure tsxOnRightClick; cdecl;
procedure tsxDeactivate; cdecl;
exports
      tsxGetData name 'tsxGetData',
      tsxOnLeftClick name 'tsxOnLeftClick',
      tsxDeactivate name 'tsxDeactivate',
      tsxOnRightClick name 'tsxOnRightClick';

procedure DisplayParentedForm(AFormType: TFormClass; var AForm);
function IsFormAlive(FormName: String): Boolean;

var
  tSXPluginHost : TtSXPluginHost;

procedure DLLMain(dwReason: DWord);

implementation

Uses FPU;

function IsFormAlive(FormName:String):Boolean;
var
  i : integer;
begin
  Result := False;
  for I := 0 to (Screen.FormCount - 1) do
    if Screen.Forms[i].Name = FormName then
      Result := True;
end;

procedure TtsxForm.WMNCHitTest(var M: TWMNCHitTest);
begin
  inherited;
  if FAutoDrag and (M.Result = htClient) then
      M.Result := htCaption;
end;

constructor TtsxForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoDrag := True;
end;

procedure TtsxToolForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
    begin
    ExStyle := WS_EX_CONTROLPARENT + WS_EX_TOOLWINDOW;
    Style := Style - WS_CHILD;
    end;
end;

procedure DisplayParentedForm(AFormType: TFormClass; var AForm);
var
  Instance: TForm;
begin
  if not Assigned(TForm(AForm)) then
    begin
    Instance := TForm(AFormType.NewInstance);
    TForm(AForm) := Instance;
    try
      Instance.CreateParented(tsxGetMainViewHwnd);
    except
      TForm(AForm) := nil;
      Instance.Free;
      raise;
    end; //try..except
    end;
  //Add the created form to the maintained list, for later cleanup  
  tSXPluginHost.OwnedWindows.AddObject(TForm(AForm).Name, TForm(AForm));
  TForm(AForm).Show;
end;

destructor TtSXPluginHost.Destroy;
var
  iIndex : integer;
begin
  //Clean up the forms which have been instantiated, and are still around
  for iIndex := 0 to (FOwnedWindows.Count - 1) do
    if IsFormAlive(FOwnedWindows.Strings[iIndex]) then
        TForm(FOwnedWindows.Objects[iIndex]).Free;
  FOwnedWindows.Clear;
  FOwnedWindows.Free;
  tSXPluginHost := nil;
  inherited Destroy;
end;

constructor TtSXPluginHost.Create;
begin
  inherited Create;
  //init the info
  FTitle := '';
  FAuthor := '';
  FBtnResID := 0;
  FStrResID := 0;
  FPlugState := e_tsxPLUGIN_INACTIVE;
  FOnGetData    := nil;
  FOnLeftClick  := nil;
  FOnRightClick := nil;
  FOnDeactivate := nil;
  FRequiredVersion := VER_ts32;
  FOwnedWindows := TStringList.Create;
end;

function tsxGetData(var tsx_Data : tsxData; tsxID : integer): integer; cdecl;
var
  SupportedVersion : TtSVersion;
begin
  result := tsxPLUG_DONE;
  try
    //Disable Floating Point Exceptions (Delphi automatically enables them)
    ResetFPU;

    //init the API function pointers
    SupportedVersion := tsxapiInitialize;

    if SupportedVersion < tSXPluginHost.RequiredVersion then
      begin
      result := tsxPLUG_FAILED;
      ShowMessage(Format('This Plugin requires trueSpace version %5.2f, but version %5.2f is installed.',
                         [tSXPluginHost.RequiredVersion / 100, SupportedVersion / 100]));
      tsxapiUninitialize;
      end
    else
      begin
      //Set the tsx_ID
      tSXPluginHost.FtsxID := tsxID;

      //invoke the user-defined event handler
      if Assigned(tSXPluginHost.FOnGetData) then tSXPluginHost.FOnGetData(result);

      //convert the Delphi strings to null-terminated strings
      tSXPluginHost.Title := Copy(tSXPluginHost.Title, 0, 31);
      tSXPluginHost.Author := Copy(tSXPluginHost.Author, 0, 63);

      //Populate the info structure for tS
      with tsx_Data do
        begin
        StrPCopy(m_Title, tSXPluginHost.Title);
        StrPCopy(m_Author, tSXPluginHost.Author);
        m_ResidBtnBmp := tSXPluginHost.FBtnResID;
        m_ResidBtnHelp := tSXPluginHost.FStrResID;
        end;
      end;
  except
    On E:Exception do
      begin
      ShowMessage('Unhandled exception in "' + tSXPluginHost.FTitle + '" tSX plugin.' + #13#10 +
                  'The exception is classified as: ' + #13#10#13#10 +
                  E.ClassName + ' : ' + E.Message);
      result := tsxPLUG_FAILED;
      end;
  end;
end;

function tsxOnLeftClick : integer; cdecl;
begin
  result := tsxPLUG_DONE;
  try
    //Disable Floating Point Exceptions (Delphi automatically enables them)
    ResetFPU;

    if Assigned(tSXPluginHost.FOnLeftClick) then tSXPluginHost.FOnLeftClick(result);

    case result of
      tsxPLUG_DONE   : tSXPluginHost.PluginState := e_tsxPLUGIN_IDLE;
      tsxPLUG_STAYON : tSXPluginHost.PluginState := e_tsxPLUGIN_ACTIVE;
      end;

  except
    On E:Exception do
      begin
      ShowMessage('Unhandled exception in "' + tSXPluginHost.FTitle + '" tSX plugin.' + #13#10 +
                  'The exception is classified as: ' + #13#10#13#10 +
                  E.ClassName + ' : ' + E.Message);
      raise;
      end;
  end;
end;

procedure tsxOnRightClick; cdecl;
begin
  try
    //Disable Floating Point Exceptions (Delphi automatically enables them)
    ResetFPU;

    if Assigned(tSXPluginHost.FOnRightClick) then tSXPluginHost.FOnRightClick;
  except
    On E:Exception do
      begin
      ShowMessage('Unhandled exception in "' + tSXPluginHost.FTitle + '" tSX plugin.' + #13#10 +
                  'The exception is classified as: ' + #13#10#13#10 +
                  E.ClassName + ' : ' + E.Message);
      raise;
      end;
  end;
end;

procedure tsxDeactivate; cdecl;
begin
  try
    //Disable Floating Point Exceptions (Delphi automatically enables them)
    ResetFPU;

    if Assigned(tSXPluginHost) then tSXPluginHost.PluginState := e_tsxPLUGIN_INACTIVE;

    if Assigned(tSXPluginHost) then
      if Assigned(tSXPluginHost.FOnDeactivate) then tSXPluginHost.FOnDeactivate;
  except
    On E:Exception do
      begin
      ShowMessage('Unhandled exception in "' + tSXPluginHost.FTitle + '" tSX plugin.' + #13#10 +
                  'The exception is classified as: ' + #13#10#13#10 +
                  E.ClassName + ' : ' + E.Message);
      raise;
      end;
  end;
end;

procedure DLLMain(dwReason: DWord);
begin
  case dwReason of
    dll_Process_Startup :
      begin
      tSXPluginHost := TtSXPluginHost.Create;
      end;
    dll_Process_Attach :
      try
        if Assigned(tSXPluginHost.FOnProcessAttach) then tSXPluginHost.FOnProcessAttach;
      except
        On E:Exception do
          begin
          ShowMessage('Exception occurred in "' + tSXPluginHost.FTitle + '" tSX plugin while attaching the process.' + #13#10 +
                      'The exception is classified as: ' + #13#10#13#10 +
                      E.ClassName + ' : ' + E.Message);
          raise;
          end;
      end;
    dll_Process_Detach :
      begin
      try
        //The application HWnd is not guaranteed at this point, so any future dialogs need
        //to be raised parentless
        Application.Handle := 0;
        if Assigned(tSXPluginHost.FOnProcessDetach) then tSXPluginHost.FOnProcessDetach;

        //Unload the API functions
        tsxapiUninitialize;
      except
        On E:Exception do
          begin
          ShowMessage('Exception occurred in "' + tSXPluginHost.FTitle + '" tSX plugin while detaching the process.' + #13#10 +
                      'The exception is classified as: ' + #13#10#13#10 +
                      E.ClassName + ' : ' + E.Message);
          raise;
          end;
      end;

      //The plugin is being totally removed from memory, so we can free the global host object
      tSXPluginHost.Free;
      end;
    dll_Thread_Attach :
      try
        if Assigned(tSXPluginHost.FOnThreadAttach) then tSXPluginHost.FOnThreadAttach;
      except
        On E:Exception do
          begin
          ShowMessage('Exception occurred in "' + tSXPluginHost.FTitle + '" tSX plugin while attaching a thread.' + #13#10 +
                      'The exception is classified as: ' + #13#10#13#10 +
                      E.ClassName + ' : ' + E.Message);
          raise;
          end;
      end;
    dll_Thread_Detach :
      try
        if Assigned(tSXPluginHost.FOnThreadDetach) then tSXPluginHost.FOnThreadDetach;
      except
        On E:Exception do
          begin
          ShowMessage('Exception occurred in "' + tSXPluginHost.FTitle + '" tSX plugin while detaching a thread.' + #13#10 +
                      'The exception is classified as: ' + #13#10#13#10 +
                      E.ClassName + ' : ' + E.Message);
          raise;
          end;
      end;
    end;
end;

end.
