
unit tsxDlg;

interface

uses
  ShareMem, SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, ToolIntf, ComCtrls, ExptIntf;

type
  TtSXFrmExpert = class(TForm)
    CancelBtn: TButton;
    PrevButton: TButton;
    NextButton: TButton;
    PageControl: TPageControl;
    NoSetup: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Panel1: TPanel;
    Image1: TImage;
    Label3: TLabel;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure CancelClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    procedure RadioButton1Click(Sender: TObject);
  private
    { Private declarations }
    SourceBuffer: PChar;
    FrmClass: integer;
    procedure RefreshButtons;
    procedure FmtWrite(Stream: TStream; Fmt: PChar; const Args: array of const);
    function DoFormCreation(const FormIdent: string): TForm;
    function CreateSource(const UnitIdent, FormIdent, FormClass: string): TMemoryStream;
    function CreateForm(const FormIdent: string): TMemoryStream;
  public
    { Public declarations }
  end;

procedure tSXFormExpert;
var
  tSXFrmExpert: TtSXFrmExpert;

implementation

uses Proxies, VirtIntf, IStreams, ExConst;

{$R *.DFM}

const
  SourceBufferSize = 1024;

{ TDlgExpert }

procedure TtSXFrmExpert.FormCreate(Sender: TObject);
begin
  RefreshButtons;
  FrmClass := 2;
end;

procedure TtSXFrmExpert.CancelClick(Sender: TObject);
begin
  Close;
end;

procedure TtSXFrmExpert.NextClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TtSXFrmExpert.RefreshButtons;
begin
  PrevButton.Enabled := False;
  NextButton.Caption := LoadStr(sFinish)
end;

{ Create the dialog defined by the user }
function TtSXFrmExpert.DoFormCreation(const FormIdent: string): TForm;
begin
  Result := TForm.Create(nil);
  Proxies.CreateSubClass(Result, 'T' + FormIdent, TForm);
  with Result do
  begin
    if FrmClass > 0 then BorderStyle := bsToolWindow;
    Width := 400;
    Height := 250;
    Left := 210;
    Top := 110;
    Position := poDesigned;
    Name := FormIdent;
    Caption := FormIdent;
  end;
end;

procedure TtSXFrmExpert.FmtWrite(Stream: TStream; Fmt: PChar;
  const Args: array of const);
begin
  if (Stream <> nil) and (SourceBuffer <> nil) then
  begin
    StrLFmt(SourceBuffer, SourceBufferSize, Fmt, Args);
    Stream.Write(SourceBuffer[0], StrLen(SourceBuffer));
  end;
end;

function TtSXFrmExpert.CreateSource(const UnitIdent, FormIdent, FormClass: string): TMemoryStream;
const
  CRLF = #13#10;
begin
  SourceBuffer := StrAlloc(SourceBufferSize);
  try
    Result := TMemoryStream.Create;
    try

      { unit header and uses clause }
      FmtWrite(Result,
        'unit %s;' + CRLF + CRLF +
        'interface' + CRLF + CRLF +
        'uses'#13#10 +
        '  SysUtils, Windows, Messages, Classes, Graphics, Controls,'#13#10 +
        '  StdCtrls, ExtCtrls, Forms', [UnitIdent]);

      { additional units that may be needed }
      FmtWrite(Result, ', tsxTypes, tSXPluginHelper', [nil]);

      FmtWrite(Result, ';' + CRLF + CRLF, [nil]);

      { begin the class declaration }
      FmtWrite(Result,
        PChar('type'#13#10 +
        '  T%s = class(T' + FormClass + ')' + CRLF), [FormIdent]);

      { add variable declarations }
      FmtWrite(Result,
        '  end;' + CRLF + CRLF +
        'var' + CRLF +
        '  %s: T%s;' + CRLF + CRLF +
        'implementation' + CRLF + CRLF +
        '{$R *.DFM}' + CRLF + CRLF, [FormIdent, FormIdent]);

      FmtWrite(Result, 'end.' + CRLF, [nil]);
      Result.Position := 0;

    except
      Result.Free;
      raise;
    end;

  finally
    StrDispose(SourceBuffer);
  end;
end;

function TtSXFrmExpert.CreateForm(const FormIdent: string): TMemoryStream;
var
  DlgForm: TForm;
begin
  DlgForm := DoFormCreation(FormIdent);
  try
    try
      Result := TMemoryStream.Create;
      Result.WriteComponentRes(FormIdent, DlgForm);
      Result.Position := 0;
    except
      Result := nil;
    end;
  finally
    DlgForm.Free;
  end;
end;

procedure tSXFormExpert;
var
  D: TtSXFrmExpert;
  ISourceStream, IFormStream: TIMemoryStream;
  UnitIdent, FormIdent, FormClass: string;
  FileName: TFileName;
begin
  if ToolServices = nil then Exit;
  if ToolServices.GetNewModuleName(UnitIdent, FileName) then
  begin
    D := TtSXFrmExpert.Create(Application);
    try
      if D.ShowModal = mrOK then
      begin
        UnitIdent := AnsiLowerCase(UnitIdent);
        UnitIdent[1] := Upcase(UnitIdent[1]);
        case D.FrmClass of
          0: FormClass := 'Form';
          1: FormClass := 'tsxToolForm';
          2: FormClass := 'tsxForm';
          end;
        
        FormIdent := FormClass + Copy(UnitIdent, 5, 255);

        IFormStream := TIMemoryStream.Create(D.CreateForm(FormIdent));
        try
          IFormStream.AddRef;
          ISourceStream := TIMemoryStream.Create(D.CreateSource(UnitIdent,
            FormIdent, FormClass));
          try
            ISourceStream.AddRef;
            ToolServices.CreateModule(FileName, ISourceStream, IFormStream,
              [cmAddToProject, cmShowSource, cmShowForm, cmUnNamed,
              cmMarkModified]);
          finally
            ISourceStream.OwnStream := True;
            ISourceStream.Free;
          end;

        finally
          IFormStream.OwnStream := True;
          IFormStream.Free;
        end;

      end;
    finally
      D.Free;
    end;
  end;
end;

procedure TtSXFrmExpert.RadioButton1Click(Sender: TObject);
begin
  FrmClass := TRadioButton(Sender).Tag;
end;

end.


