unit tsxFogEdit;
/////////////////////////////////////
// tSX Builder                     //
// TtsxFogEdit component - sample  //
// component which allows the user //
// to use a standard Windows color //
// picker for the scene fog. Also  //
// gives access to the other fog   //
// attributes available.           //
/////////////////////////////////////

interface

uses
  windows, messages, classes, tsxTypes, dialogs, Controls, StdCtrls, ComCtrls, ExtCtrls,
  graphics, SysUtils, tsxNumberSpinner;

Type
  TFogParmType = (fpRed, fpGreen, fpBlue, fpNear, fpFar, fpMax);
  TtsxFogEdit = class(TCustomGroupBox)
  private
    FFogOn : Boolean;
    FFogRed, FFogBlue, FFogGreen : integer;
    FFogNear, FFogFar, FFogMax : integer;
    ColorPreview: TShape;
    cbEnabled: TCheckBox;
    FogRed, FogBlue, FogGreen, FogNear, FogFar, FogMax : TtsxNumberSpinner;
    //internal field creaters
    procedure CreateBevel(Bounds: TRect);
    function CreateSpinEdit(X, Y, AWidth: integer; ACaption: string; APosition: integer): TtsxNumberSpinner;
    //Access functions
    function GetFogColor: TColor;
    //Access methods
    procedure SetFogColor(NewColor: TColor);
    procedure SetFogOn(Value: Boolean);
    procedure SetFogParm(Value : integer; ParmType : TFogParmType);
    procedure SetFogNear(Value : integer);
    procedure SetFogFar(Value : integer);
    procedure SetFogMax(Value : integer);
    procedure SetFogR(Value : integer);
    procedure SetFogG(Value : integer);
    procedure SetFogB(Value : integer);
    //Subcomponent Event Handlers
    procedure FogRedChanged(Sender : TObject);
    procedure FogGreenChanged(Sender : TObject);
    procedure FogBlueChanged(Sender : TObject);
    procedure FogNearChanged(Sender : TObject);
    procedure FogFarChanged(Sender : TObject);
    procedure FogMaxChanged(Sender : TObject);
    procedure FogEnable(Sender: TObject);
    procedure PopupColors(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  protected
    procedure Paint; override;
    procedure FogRefresh; virtual;
    procedure Loaded; override;
  public
    constructor Create(AOwner : TComponent); override;
    property FogColor:   TColor  read GetFogColor write SetFogColor stored False;
    property FogOn:      Boolean read FFogOn      write SetFogOn    stored False;
    property NearValue:  integer read FFogNear    write SetFogNear  stored False;
    property FarValue:   integer read FFogFar     write SetFogFar   stored False;
    property MaxValue:   integer read FFogMax     write SetFogMax   stored False;
    property RedValue:   integer read FFogRed     write SetFogR     stored False;
    property GreenValue: integer read FFogGreen   write SetFogG     stored False;
    property BlueValue:  integer read FFogBlue    write SetFogB     stored False;
  published
    property ShowHint;
  end;

implementation

procedure TtsxFogEdit.SetFogParm(Value : integer; ParmType : TFogParmType);
var
  wFogRed, wFogBlue, wFogGreen : integer;
  iFogNear, iFogFar, iFogMax : integer;
begin
  tsxSceneGetFogParms(@wFogRed, @wFogBlue, @wFogGreen, @iFogNear, @iFogFar, @iFogMax);
  if ParmType = fpRed   then wFogRed   := Value;
  if ParmType = fpGreen then wFogGreen := Value;
  if ParmType = fpBlue  then wFogBlue  := Value;
  if ParmType = fpNear  then iFogNear  := Value;
  if ParmType = fpFar   then iFogFar   := Value;
  if ParmType = fpMax   then iFogMax   := Value;
  tsxSceneSetFogParms(wFogRed, wFogBlue, wFogGreen, iFogNear, iFogFar, iFogMax);
end;

procedure TtsxFogEdit.SetFogNear(Value : integer);
begin
  if FFogNear <> Value then
    begin
    FFogNear := Value;
    SetFogParm(Value, fpNear);
    end;
end;

procedure TtsxFogEdit.SetFogFar(Value : integer);
begin
  if FFogFar <> Value then
    begin
    FFogFar := Value;
    SetFogParm(Value, fpFar);
    end;
end;

procedure TtsxFogEdit.SetFogMax(Value : integer);
begin
  if FFogMax <> Value then
    begin
    FFogMax := Value;
    SetFogParm(Value, fpMax);
    end;
end;

procedure TtsxFogEdit.SetFogR(Value : integer);
begin
  if FFogRed <> Value then
    begin
    FFogRed := Value;
    SetFogParm(Value, fpRed);
    end;
end;

procedure TtsxFogEdit.SetFogG(Value : integer);
begin
  if FFogGreen <> Value then
    begin
    FFogGreen := Value;
    SetFogParm(Value, fpGreen);
    end;
end;

procedure TtsxFogEdit.SetFogB(Value : integer);
begin
  if FFogBlue <> Value then
    begin
    FFogBlue := Value;
    SetFogParm(Value, fpBlue);
    end;
end;

procedure TtsxFogEdit.SetFogOn(Value: Boolean);
begin
  if FFogOn <> Value then
    begin
    FFogOn := Value;
    if not (csDesigning in ComponentState) then
      if FFogOn then tsxSceneSwitchFog(e_tsxTRUE)
        else tsxSceneSwitchFog(e_tsxFALSE);
    Invalidate;
    end;
end;

procedure TtsxFogEdit.PopupColors(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ColorDialog :TColorDialog;
begin
  if (Button = mbRight) then
    begin
    ColorDialog := TColorDialog.Create(nil);
    ColorDialog.Options := [cdFullOpen, cdSolidColor];
    ColorDialog.Color := GetFogColor;
    if ColorDialog.Execute then
      SetFogColor(ColorDialog.Color);
    BringWindowToTop(Self.WindowHandle);
    ColorDialog.Free;
    end;
end;

procedure TtsxFogEdit.FogEnable(Sender: TObject);
begin
  SetFogOn(TCheckBox(Sender).Checked);
end;

procedure TtsxFogEdit.FogRedChanged(Sender: TObject);
begin
  RedValue   := FogRed.Position;
  ColorPreview.Brush.Color := GetFogColor;
end;

procedure TtsxFogEdit.FogBlueChanged(Sender: TObject);
begin
  BlueValue  := FogBlue.Position;
  ColorPreview.Brush.Color := GetFogColor;
end;

procedure TtsxFogEdit.FogGreenChanged(Sender: TObject);
begin
  GreenValue := FogGreen.Position;
  ColorPreview.Brush.Color := GetFogColor;
end;

procedure TtsxFogEdit.FogMaxChanged(Sender: TObject);
begin
  MaxValue   := FogMax.Position;
end;

procedure TtsxFogEdit.FogFarChanged(Sender: TObject);
begin
  FarValue   := FogFar.Position;
end;

procedure TtsxFogEdit.FogNearChanged(Sender: TObject);
begin
  NearValue  := FogNear.Position;
end;

function TtsxFogEdit.GetFogColor: TColor;
begin
  result := MakeLong(MakeWord(RedValue, GreenValue), MakeWord(BlueValue, 0));
end;

procedure TtsxFogEdit.SetFogColor(NewColor: TColor);
begin
  if (NewColor <> GetFogColor) then
    begin
    BlueValue  := GetBValue(ColorToRGB(NewColor));
    GreenValue := GetGValue(ColorToRGB(NewColor));
    RedValue   := GetRValue(ColorToRGB(NewColor));
    end;
end;

procedure TtsxFogEdit.Paint;
var
  tmpFogRed, tmpFogBlue, tmpFogGreen : integer;
  tmpFogNear, tmpFogFar, tmpFogMax : integer;
begin
  inherited Paint;
  //take a "snapshot" of the current fog colors
  tmpFogRed := FFogRed;
  tmpFogBlue := FFogBlue;
  tmpFogGreen := FFogGreen;
  tmpFogNear := FFogNear;
  tmpFogFar := FFogFar;
  tmpFogMax := FFogMax;

  FogRed.Position := tmpFogRed;
  FogBlue.Position := tmpFogBlue;
  FogGreen.Position := tmpFogGreen;
  FogNear.Position := tmpFogNear;
  FogFar.Position := tmpFogFar;
  FogMax.Position := tmpFogMax;
  cbEnabled.Checked := FFogOn;
  ColorPreview.Brush.Color := GetFogColor;
end;

procedure TtsxFogEdit.FogRefresh;
begin
  if not (csDesigning in ComponentState) then
    begin
    FFogOn := (tsxSceneIsFogOn = e_tsxTRUE);
    tsxSceneGetFogParms(@FFogRed, @FFogBlue, @FFogGreen, @FFogNear, @FFogFar, @FFogMax);
    SetFogParm(FFogNear,  fpNear);
    SetFogParm(FFogFar,   fpFar);
    SetFogParm(FFogMax,   fpMax);
    SetFogParm(FFogRed,   fpRed);
    SetFogParm(FFogGreen, fpGreen);
    SetFogParm(FFogBlue,  fpBlue);
    end;
end;

procedure TtsxFogEdit.Loaded;
begin
  inherited Loaded;
  FogRefresh;

  FogRed.OnChange := FogRedChanged;
  FogBlue.OnChange := FogBlueChanged;
  FogGreen.OnChange := FogGreenChanged;
  FogMax.OnChange := FogMaxChanged;
  FogFar.OnChange := FogFarChanged;
  FogNear.OnChange := FogNearChanged;
  cbEnabled.OnClick := FogEnable;
end;

constructor TtsxFogEdit.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Caption := ' Fog Parameters ';
  SetBounds(Left, Top, 273, 114);
  CreateBevel(Rect(8, 20, 81, 61));
  CreateBevel(Rect(92, 20, 173, 61));
  CreateBevel(Rect(8, 83, 257, 24));
  FFogRed   := 128;
  FFogBlue  := 128;
  FFogGreen := 128;
  FFogMax   := 100;
  FFogFar   := 500;
  FFogNear  := 1;
  FogRed   := CreateSpinEdit(100, 24, 89, 'Red',   FFogRed);
  FogBlue  := CreateSpinEdit(100, 60, 89, 'Blue',  FFogBlue);
  FogGreen := CreateSpinEdit(100, 42, 89, 'Green', FFogGreen);
  FogMax   := CreateSpinEdit(179, 86, 79, 'Max',   FFogMax);
  FogFar   := CreateSpinEdit(95,  86, 79, 'Far',   FFogFar);
  FogNear  := CreateSpinEdit(10,  86, 79, 'Near',  FFogNear);
  FogMax.Max := 100;
  FogFar.Max := 32767;
  FogNear.Max := 32767;
  cbEnabled := TCheckBox.Create(Self);
  with cbEnabled do
    begin
    Parent := Self;
    SetBounds(16, 42, 65, 17);
    Caption := 'Enabled';
    Hint := 'Fog Enabled';
    Visible := True;
    end;
  ColorPreview := TShape.Create(Self);
  with ColorPreview do
    begin
    Parent := Self;
    SetBounds(192, 24, 68, 53);
    Visible := True;
    Hint := 'Right-Click Me';
    OnMouseDown := PopupColors;
    end;

  ParentShowHint := False;
  ShowHint := True;
end;

procedure TtsxFogEdit.CreateBevel(Bounds: TRect);
begin
  with TBevel.Create(Self) do
    begin
    Parent := Self;
    Visible := True;
    SetBounds(Bounds.Left, Bounds.Top, Bounds.Right, Bounds.Bottom);
    end;
end;

function TtsxFogEdit.CreateSpinEdit(X, Y, AWidth: integer; ACaption: string; APosition: integer): TtsxNumberSpinner;
begin
  result := TtsxNumberSpinner.Create(Self);
  with result do
    begin
    Parent := Self;
    Left := X;
    Top := Y;
    Width := AWidth;
    Caption := ACaption;
    Hint := 'Fog ' + ACaption + ' Value';
    Max := 255;
    Position := APosition;
    end;
end;

end.
