unit tsxNumberSpinner;
//////////////////////////////////////////
// tSX Builder                          //
// TtsxNumberSpinner - simple trueSpace //
// style number spinner. Does not rely  //
// on the tSX API.                      //
//////////////////////////////////////////

interface

uses
  windows, messages, classes, tsxTypes, dialogs, Controls, StdCtrls, ComCtrls, ExtCtrls,
  graphics, SysUtils;

Type
  TtsxNumberSpinner = class(TCustomControl)
  private
    FEdit : TEdit;
    FMax, FMin, FPosition : integer;
    FCaption : TCaption;
    FSpinOn, FSpinOff : TBitmap;
    FDragging: Boolean;
    StartX: integer;
    FOnChange: TNotifyEvent;
    procedure SetMax(Value: integer);
    procedure SetMin(Value: integer);
    procedure SetPosition(Value: integer);
    procedure SetCaption(ACaption : TCaption);
    procedure KeyPressed(Sender: TObject; var Key: Char);
    procedure WMSize(var Msg: TWMSize); message wm_Size;
  protected
    procedure DrawSpinner; virtual;
    procedure Paint; override;
    procedure Change; dynamic;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X,Y: integer); override;
    procedure MouseMove(Shift: TShiftState; X,Y: integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X,Y: integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Max: integer read FMax write SetMax default 255;
    property Min: integer read FMin write SetMin default 0;
    property Position: integer read FPosition write SetPosition;
    property Caption: TCaption read FCaption write SetCaption;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property Color;
    property Enabled;
    property HelpContext;
    Property Hint;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

{$R tsxNumberSpinner.RES}

//######### TtsxNumberSpinner ############//
constructor TtsxNumberSpinner.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 89;
  Height := 16;
  FMin := 0; FMax := 255; FPosition := 0;
  TabStop := True;
  FDragging := False;
  StartX := 0;
  FCaption := 'Number';
  FSpinOn := TBitmap.Create;
  FSpinOn.Handle := LoadBitmap(HInstance, 'SPINON');
  FSpinOff := TBitmap.Create;
  FSpinOff.Handle := LoadBitmap(HInstance, 'SPINOFF');
  FEdit := TEdit.Create(Self);
  with FEdit do
    begin
    Parent := Self;
    AutoSize := False;
    Width := 33;
    Height := 16;
    Left := Self.Width - 15 - Width;
    Visible := True;
    Text := '0';
    OnKeyPress := KeyPressed;
    end;
end;

destructor TtsxNumberSpinner.Destroy;
begin
  FSpinOn.Free;
  FSpinOff.Free;
  inherited Destroy;
end;

procedure TtsxNumberSpinner.SetMax(Value : integer);
begin
  if Value <> FMax then
    begin
    FMax := Value;
    if FPosition > FMax then Position := FMax;
    Invalidate;
    end;
end;

procedure TtsxNumberSpinner.SetMin(Value : integer);
begin
  if Value <> FMin then
    begin
    FMin := Value;
    if FPosition < FMin then Position := FMin;
    Invalidate;
    end;
end;

procedure TtsxNumberSpinner.SetPosition(Value : integer);
begin
  if Value <> FPosition then
    begin
    if Value < FMin then Value := FMin;
    if Value > FMax then Value := FMax;
    FPosition := Value;
    FEdit.Text := IntToStr(FPosition);
    FEdit.Invalidate;
    if not (csDesigning in ComponentState) then Change;
    end;
end;

procedure TtsxNumberSpinner.SetCaption(ACaption : TCaption);
begin
  if ACaption <> FCaption then
    begin
    FCaption := ACaption;
    Invalidate;
    end;
end;

procedure TtsxNumberSpinner.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TtsxNumberSpinner.KeyPressed(Sender: TObject; var Key: Char);
begin
  if ord(Key) = 13 then
    begin
    Key := #0;
    try
      //Attempt type conversion
      Position := StrToInt(FEdit.Text);
      Invalidate;
    except
      on EConvertError do //Revert back to last value
        FEdit.Text := IntToStr(FPosition);
    end;
    end;
end;

procedure TtsxNumberSpinner.Paint;
begin
  DrawSpinner;
end;

procedure TtsxNumberSpinner.DrawSpinner;
var
  SrcBMP : TBitmap;
begin
  with Canvas do
    begin
    //Fill control with background color
    Brush.Color := Self.Color;
    Brush.Style := bsSolid;
    FillRect(Rect(0, 0, Width, Height));
    //Draw the raised area around the button bmp
    MoveTo(Self.Width, 0);
    Pen.Color := clBtnHighlight;
    Pen.Style := psSolid;
    Pen.Mode := pmCopy;
    LineTo(Self.Width - 15, 0);
    LineTo(Self.Width - 15, Self.Height - 1);
    Pen.Color := clBtnShadow;
    LineTo(Self.Width - 1, Self.Height - 1);
    LineTo(Self.Width - 1, 0);
    //Draw the button bmp
    if FDragging then SrcBMP := FSpinOn
      else SrcBMP := FSpinOff;
    CopyRect(Rect(Self.Width - 14, 1, Self.Width - 2, Self.Height - 1), SrcBMP.Canvas, Rect(1, 0, 12, 13));
    //Position the edit box
    FEdit.Left := Self.Width - 15 - FEdit.Width;
    //Draw caption
    TextRect(Rect(0,1,Self.Width - 15 - FEdit.Width - 2,Height), 2, 1, FCaption);
    //Make text box visible
    FEdit.Text := IntToStr(FPosition);
    FEdit.Invalidate;
    end;
end;

procedure TtsxNumberSpinner.MouseDown(Button: TMouseButton; Shift: TShiftState; X,Y: integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and PtInRect(Rect(Self.Width - 15, 0, Self.Width, Self.Height), Point(X, Y)) then
    begin
    FDragging := True;
    StartX := X;
    Invalidate;
    end;
end;

procedure TtsxNumberSpinner.MouseMove(Shift: TShiftState; X,Y: Integer);
begin
  inherited MouseMove(Shift, X, Y);
  if FDragging then
    begin
    Position := Position + ((X - StartX) div 10);
    end;
end;

procedure TtsxNumberSpinner.MouseUp(Button: TMouseButton; Shift: TShiftState; X,Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if (Button = mbLeft) and FDragging then
    begin
    FDragging := False;
    Invalidate;
    end;
end;

procedure TtsxNumberSpinner.WMSize(var Msg: TWMSize);
begin
  inherited;
  if Height <> 17 then Height := 17;
end;

end.
