unit tsxTypes;
/////////////////////////////////////////////
// tSX Builder                             //
// tSX API import unit                     //
//                                         //
// Delphi conversion of all tSXAPI C++     //
// functions.                              //
/////////////////////////////////////////////

interface

uses Classes, SysUtils, windows;

//////// Custom Exception for missing API functions ////////
type
  EtsxFunctionNotFound = class(Exception);

{ Constants and type for testing for supported truespace versions}
Const
  VER_ts00 = 0;
  VER_ts30 = 300;
  VER_ts31 = 310;
  VER_ts32 = 320;

Type
  TtSVersion = VER_ts00 .. VER_ts32;

//////// c++ - pascal type conversions ////////
type
	Short = SmallInt;
	UnsignedShort = Word;
	UnsignedInt = DWord;
	Float = Single;
  long = integer;

//////// tsxPlug.H ///////////
Type
  tsxData = record
    m_Title : array[0..31] of char;
    m_Author : array[0..63] of char;
    m_ResidBtnBmp : short;
    m_ResidBtnHelp : short;
    end;

Const
  tsxVERSION_ALPHA_BASE = -200;
  tsxVERSION_BETA_BASE  = -100;
  tsxVERSION_MAJOR      = 310;
  tsxVERSION_MINOR      = -4;
  tsxPLUG_DONE   = 0;
  tsxPLUG_FAILED = -1;
  tsxPLUG_STAYON = 1;

//////// tsxTypes.H //////////
Type
  tsxBOOL = (e_tsxFALSE,
            e_tsxTRUE  );

  tsxRET = (e_tsxFAILURE,
           e_tsxSUCCESS  );

  tsxERR = Integer;

  tsxFRAMEID = (e_tsxWorldFrame, e_tsxModelFrame);

  CtsxVector2f = record
    x, y : Single;
    end;
  PCtsxVector2f = ^CtsxVector2f;

  CtsxVector3f = record
    x, y, z : Single;
    end;
  PCtsxVector3f = ^CtsxVector3f;
  CtsxVector3fArray = array[0..1000] of CtsxVector3f;
  PCtsxVector3fArray = ^CtsxVector3fArray;

  CtsxAxes3f = record
    xaxis, yaxis, zaxis : CtsxVector3f ;
    end;
  PCtsxAxes3f = ^CtsxAxes3f;

  CtsxTxmx3f = record
    matrix: Array[0..3-1, 0..4-1] of Single;
    end;
  PCtsxTxmx3f = ^CtsxTxmx3f;

  CtsxBBox3f = record
    minbounds, maxbounds : CtsxVector3f;
    end;
  PCtsxBBox3f= ^CtsxBBox3f;

function tsxVectorsEqual3f(const vec1, vec2: PCtsxVector3f): Boolean;
function tsxVectorsEqual2f(const vec1, vec2: PCtsxVector2f): Boolean;

Type
  CtsxColor = record
    red, green, blue, alpha : Byte;
    end;
  PCtsxColor = ^CtsxColor;

  CtsxUV = record
    u, v : Single;
    end;
  PCtsxUV = ^CtsxUV;
	CtsxUVArray = array[0..1000] of CtsxUV;
	PCtsxUVArray = ^CtsxUVArray;

  // 3.2 API addition
  CtsxRect = record
    x, y :	short;
    MaxX, MaxY :	short;
    end;

  // end 3.2 API Addition

const
  e_tsxUNDEFINED     = 0;
  e_tsxMATERIAL      = 1;
  e_tsxSCENE         = 2;
  e_tsxPLUGIN_FILTER = 3;
  e_tsxPHYS_WIND     = 4;
  e_tsxMATRECT       = 5;
  e_tsxSELECTABLE    = 50;
  e_tsxAXES          = 51;
  e_tsxGNODETYPES    = 100;
  e_tsxCAMERA        = 101;
  e_tsxLIGHT         = 102;
  e_tsxJOINT         = 103;
  e_tsxNAIL          = 104;
  e_tsxLATTICE       = 105;
  e_tsxPATH          = 106;
  e_tsxMNODETYPES    = 200;
  e_tsxPOLYHEDRON    = 201;
  e_tsxGROUP         = 202;
  e_tsxLODGROUP      = 203;
  e_tsxMBALLOBJ      = 204;
  e_tsxSADEF         = 205;
  e_tsxIKGROUP       = 206;
  e_tsxSOBJEND       = 207;

type
  tsxSOBJTYPE = e_tsxUNDEFINED..e_tsxSOBJEND;

  tsxSOBJ = Pointer;
  tsxMATERIAL = Pointer;
  tsxSCENE = Pointer;
  tsxPLUGIN_FILTER = Pointer;
  tsxPHYS_WIND = Pointer;
  tsxMATRECT = Pointer;
  tsxAXES = Pointer;
  tsxGNODE = Pointer;
  tsxCAMERA = Pointer;
  tsxLIGHT = Pointer;
  tsxJOINT = Pointer;
  tsxNAIL = Pointer;
  tsxLATTICE = Pointer;
  tsxPATH = Pointer;
  tsxMNODE = Pointer;
  tsxPOLYHEDRON = Pointer;
  tsxGROUP = Pointer;
  tsxLODGROUP = Pointer;
  tsxMBALLOBJ = Pointer;
  tsxSADEF = Pointer;
  tsxIKGROUP = Pointer;
  tsxKEYFRAME = Pointer;

  PtsxKEYFRAME = ^tsxKEYFRAME;

  tsxBasicCallbackFP = procedure; cdecl;

///////// tsxErr.H //////////
Const
  tsxERR_SUCCESS     = 0;
  tsxERR_NOGOOD	     = -1;  // something generally went wrong
  tsxERR_NO_MEMORY   = -2;  // no memory available
  tsxERR_NULL_REF	   = -3;  // null pointer reference
  tsxERR_BAD_INPUT   = -4;  // bad input data
  tsxERR_FORMAT	     = -5;  // general bad data format
  tsxERR_ABORT	     = -8;  // user abort request
  tsxERR_REPORTED	   = -10; // still an error but was reported below
  tsxERR_UNIMPL	     = -11; // feature unimplemented
  tsxERR_IDENTIFIER  = -12; // bad identifier on data or record
  tsxERR_VERSION 	   = -13; // Correct file type but wrong version
  tsxERR_NOT_FOUND   = -14; // object not found
  tsxERR_OVERFLOW	   = -15; // data overflow for size of result
  tsxERR_RANGE	     = -16; // value out of range
  tsxERR_INIT	       = -18; // subsystem improperly initialized
  // system errors
  tsxERR_SYSERR_BASE = -50;
  tsxERR_STDIO	     = (tsxERR_SYSERR_BASE -0); // error occurred in io
  tsxERR_MSWIN	     = (tsxERR_SYSERR_BASE -1); // error occurred in windows

function tsxSUCCEEDED(err: tsxERR): Boolean;
function tsxFAILED(err: tsxERR): Boolean;

///////// tsxGeom.H /////////
Const
  M_PI = 3.14159265359;
  tsxRAD2DEG_FAC = 57.29577951308;
  tsxDEG2RAD_FAC = 0.01745329251994;

  function tsxRad2Deg(Const rad: Single): Single;
  function tsxDeg2Rad(Const deg: Single): Single; 

Type
  TtsxMagnitude3fProc = function (Const vec: CtsxVector3f): Single; cdecl;
  TtsxMagnitude2fProc = function (Const vec: CtsxVector2f): Single; cdecl;
  TtsxNormalize3fProc = function (var vec: CtsxVector3f): Single; cdecl;
  TtsxNormalize2fProc = function (var vec: CtsxVector2f): Single; cdecl;
  TtsxGetNormalized3fProc = function (var norm: CtsxVector3f; Const vec: CtsxVector3f): Single; cdecl;
  TtsxGetNormalized2fProc = function (var norm: CtsxVector2f; Const vec: CtsxVector2f): Single; cdecl;
  TtsxAddScalar3fProc = function (var vec: CtsxVector3f; Const s: Single): PCtsxVector3f; cdecl;
  TtsxMulScalar3fProc = function (var vec: CtsxVector3f; Const s: Single): PCtsxVector3f; cdecl;
  TtsxAddVec3fProc = function (var u: CtsxVector3f; Const v: CtsxVector3f): PCtsxVector3f; cdecl;
  TtsxSubVec3fProc = function (var u: CtsxVector3f; Const v: CtsxVector3f): PCtsxVector3f; cdecl;
  TtsxDProdVec3fProc = function (Const u, v: CtsxVector3f): Single; cdecl;
  TtsxXProdVec3fProc = function (var uxv: CtsxVector3f; Const u, v: CtsxVector3f): PCtsxVector3f; cdecl;
  TtsxAngleBtwVecs3fProc = function (Const u, v: CtsxVector3f; var axis: CtsxVector3f): Single; cdecl;
  TtsxDistanceToPoint3fProc = function (Const p, q: CtsxVector3f): Single; cdecl;
  TtsxDistanceToLine3fProc = function (Const p, line_point, line_unitvec: CtsxVector3f): Single; cdecl;
  TtsxDistanceToPlane3fProc = function (Const p, plane_norm, plane_point: CtsxVector3f): Single; cdecl;
  TtsxXProdMx3fProc = function (var A: CtsxTxmx3f; Const B, C: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxInvertMx3fProc = function (var Minv: CtsxTxmx3f; Const M: CtsxTxmx3f): tsxRET; cdecl;
  TtsxMakeMxFromXZO3fProc = function (Const xAxis, zAxis, origin: CtsxVector3f;
                              var pTxmx, pInvTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxMakeRotMx3fProc = function (Const cent, axis: CtsxVector3f; angle: Single;
                          var pTxmx, pInvTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxMakePureRotMx3fProc = function (Const axis: CtsxVector3f; angle: Single;
                              var pTxmx, pInvTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxGetMxTranslation3fProc = function (var trvec: CtsxVector3f; Const Txmx: CtsxTxmx3f): PCtsxVector3f; cdecl;
  TtsxSetMxTranslation3fProc = function (var Txmx: CtsxTxmx3f; Const trvec: CtsxVector3f): PCtsxTxmx3f; cdecl;
  TtsxAddMxTranslation3fProc = function (var Txmx: CtsxTxmx3f; Const trvec: CtsxVector3f): PCtsxTxmx3f; cdecl;
  TtsxSubMxTranslation3fProc = function (var Txmx: CtsxTxmx3f; Const trvec: CtsxVector3f): PCtsxTxmx3f; cdecl;
  TtsxUnMatrix3fProc = procedure (Const Txmx: CtsxTxmx3f; var Rot: CtsxTxmx3f; pScaleShear: PSingle); cdecl;
  TtsxTransformVecs3fProc = procedure (numVecs: Integer; var pOutVecs: CtsxVector3f;
                              Const pInVecs: CtsxVector3f; var Txmx: CtsxTxmx3f); cdecl;
  TtsxRotateVecs3fProc = procedure (numVecs: Integer; var pOutVecs: CtsxVector3f;
                           Const pInVecs: CtsxVector3f; var Txmx: CtsxTxmx3f); cdecl;
  TtsxRotateAxes3fProc = procedure (var pOutAxes, pInAxes: CtsxAxes3f; var Txmx: CtsxTxmx3f); cdecl;
  TtsxBBoxInitProc = procedure (var pBBox: CtsxBBox3f); cdecl;
  TtsxBBoxIntersectionProc = function (var pBBintx : CtsxBBox3f; Const pBBox1, pBBox2: CtsxBBox3f): tsxBOOL; cdecl;

///////// tsxSObj.H //////////
  TtsxSobjGetTypeProc = function (pSobj: TSXSOBJ): tsxSOBJTYPE; cdecl;
  TtsxIsSobjProc = function (pSobj: TSXSOBJ): tsxBOOL; cdecl;
  TtsxSobjGetFirstChildProc = function (pSobj: TSXSOBJ): tsxSOBJTYPE; cdecl;
  TtsxSobjGetLastChildProc = function (pSobj: TSXSOBJ): tsxSOBJTYPE; cdecl;
  TtsxSobjGetNextProc = function (pSobj: TSXSOBJ): tsxSOBJTYPE; cdecl;
  TtsxSobjGetPrevProc = function (pSobj: TSXSOBJ): tsxSOBJTYPE; cdecl;
  TtsxSobjDeleteProc = procedure (pSobj: TSXSOBJ); cdecl;

  // 3.2 API addition

  TtsxSobjAddChildProc = function ( pParent, pNewChild : TSXSOBJ ): tsxERR; cdecl;

  // end 3.2 API addition

///////// tsxMatrl.H //////////
Const
  tsxSOLID_DIR_X  = 0;
  tsxSOLID_DIR_Y  = 1;
  tsxSOLID_DIR_Z  = 2;

Type
  CtsxGraniteProps = record
    color1 : CtsxColor; 	//The different stone colors.
    color2 : CtsxColor;
    color3 : CtsxColor;
    color4 : CtsxColor;
    amount1 : single;	//Relative amount of color1 [0.0 - 1.0] (0=none)
    amount2 : single;	//Relative amount of color2 [0.0 - 1.0]
    amount3 : single;	//Relative amount of color3 [0.0 - 1.0]
    amount4 : single;	//Relative amount of color4 [0.0 - 1.0]
    sharpness : single;	//[0.0 (max blurring to one color) - 1.0 (no blur)]
    scale : CtsxVector3f;	//Scaling along solid texture directions. Non-negative.
			// Larger values result in finer texture.
    seed : word; //Random seed for generating pattern [0 - 64k]
    end;
 
  CtsxMarbleProps = record
    stonecol : CtsxColor;	//Base stone color
    veincol : CtsxColor;	//Color of veins
    turbulence : smallint;	//Higher numbers result in larger, bumpier veins.
			// 0 produces straight veins. Typically [0 - 20].
    pad1: word;  // SJM - Undocumented (alignment bytes)
    sharpness : single;	//Blending between vein and stone colors at vein edges.
			// Higher values increase sharpness. [0.0 - 1.0]
    grain : smallint;	//Vein direction (tsxSOLID_DIR_*).
    pad2: word;  // SJM - Undocumented (alignment bytes)
    scale : CtsxVector3f; //Scaling along solid texture directions. Non-negative.
			// Larger values result in more veins.
    seed : word; //Random seed for generating pattern [0 - 64k]
    end;

  CtsxWoodProps = record
    lightcol : CtsxColor; //The light or spring color
    darkcol : CtsxColor;  //The dark or summer color
    ldratio : single;	//Light/Dark ratio.
			// [0.0 (almost all dark) - 1.0 (almost all light)]
    rdensity : single;	//Number of rings per unit distance.
    rwidthvar : single;	//Variation in ring width [0.0 (uniform) - 1.0]
    rshapevar : single;	//Variation in ring shape [0.0 (perfect circle) - 1.0]
    center : CtsxVector3f; //Location of rings center.
    grain : smallint;	//Ring axis direction (tsxSOLID_DIR_*).
    pad: Word; // SJM - Undocumented (alignment bytes)
    scale : CtsxVector3f;	//Scaling along solid texture directions. Non-negative.
    seed : word; //Random seed for generating pattern [0 - 64k]
    end;
   
Const
  // Values for CtsxProcTexture::txtrtype
  tsxTEXTURE_NONE	   = 0;
  tsxTEXTURE_GRANITE = 1;
  tsxTEXTURE_MARBLE  = 2;
  tsxTEXTURE_WOOD    = 3;
  // returned by `tsxMaterialGetTextureType' if texture is not procedural.
  // Not a valid value for field txtrtype.
  tsxTEXTURE_IMAGE   = 4;

Type
  CtsxProcTexture = record
    txtrtype : smallint; //one of tsxTEXTURE_*
    pad: Word; // SJM - Undocumented (alignment bytes)
    case integer of
      tsxTEXTURE_GRANITE : (gprops :CtsxGraniteProps);
      tsxTEXTURE_MARBLE  : (mprops :CtsxMarbleProps);
      tsxTEXTURE_WOOD    : (wprops :CtsxWoodProps);
    end;
  PCtsxProcTexture = ^CtsxProcTexture;

Const
  // Reflection shading types (mutually exclusive)
  tsxSHADER_FLAT   = $01;
  tsxSHADER_PHONG	 = $02;
  tsxSHADER_METAL	 = $04;
  // Surface smoothness (mutually exclusive)
  tsxFACET_FACETED = $01;
  tsxFACET_AUTO    = $02;
  tsxFACET_SMOOTH  = $04;
  
Type
  CtsxSurfaceProps = record
    ka : single;	//ambient coefficent [0.0 -(0.01)- 1.0]
    ks : single;	//specular coefficent [0.0 -(0.01)- 1.0]
    exp: single;	//exponent of specularity (roughness) [0.0 -(0.01)- 1.0]
    end;
  PCtsxSurfaceProps = ^CtsxSurfaceProps;

Const
  // Texture Map properties ..................................
  tsxTXTR_OVERLAY	= $0001;
  tsxTXTR_ON      = $0002;  //otherwise OFF
  tsxTXTR_MOVIE	  = $0004;
  tsxTXTR_MARBLE  = $0400;
  tsxTXTR_WOOD    = $0800;
  tsxTXTR_GRANITE = $1000;
  // Only one of MARBLE, WOOD or GRANITE may be set.

Type
  CtsxTextureProps = record
    offsetu : single;	  //texture U offset [-1.0 -(.001)- 1.0]
    offsetv : single;	  //texture V offset [-1.0 -(.001)- 1.0]
    repeatsu : single;  	  //texture repeats in U [0.01 -(.1)- 100.0]
    repeatsv : single;  	  //texture repeats in V [0.01 -(.1)- 100.0]
    flags : word; //see above
    end;
  PCtsxTextureProps = ^CtsxTextureProps;
      
Const
  // Bump Map properties .....................................
  tsxBUMP_ON    = $0002;  //otherwise OFF
  tsxBUMP_MOVIE	= $0004;

Type
  CtsxBumpProps = record
    offsetu : single;	//bump U offset [-1.0 -(.001)- 1.0]
    offsetv : single;	//bump V offset [-1.0 -(.001)- 1.0]
    repeatsu : single;	//bump repeats in U [0.01 -(.1)- 100.0]
    repeatsv : single;	//bump repeats in V [0.01 -(.1)- 100.0]
    amplitude : single;	//bump amplitude [-10.0 - 10.0]
    flags : word;	//see above
    end;
  PCtsxBumpProps = ^CtsxBumpProps;

Const
  // Environment Map properties ..............................
  tsxENVR_CUBIC	= $0001;  //cubic environment, otherwise its spherical
  tsxENVR_ON    = $0002;  //otherwise OFF
  tsxENVR_MOVIE	= $0004;

  tsxUVPROJ_PLANE        = $01;
  tsxUVPROJ_PLANE_SQUARE = $02;
  tsxUVPROJ_CYLINDER     = $04;
  tsxUVPROJ_SPHERE       = $08;

type
  TtsxMaterialCreateProc = function (var ppMatrl: tsxMATERIAL): tsxERR; cdecl;
  TtsxMaterialCreateCopyProc = function (var ppNewMatrl: tsxMATERIAL; pOldMatrl: tsxMATERIAL): tsxERR; cdecl;
  TtsxMaterialCreateCopyActiveProc = function (var ppMatrl: tsxMATERIAL): tsxERR; cdecl;
  TtsxMaterialDestroyProc = procedure (pMatrl: tsxMATERIAL); cdecl;
  TtsxMaterialSetActiveProc = procedure (pNewActive: tsxMATERIAL); cdecl;
  TtsxMaterialGetActiveProc = function : tsxMATERIAL; cdecl;
  TtsxMaterialCopyActiveProc = procedure (pMatrl: tsxMATERIAL); cdecl;
  TtsxMaterialsAreEqualProc = function (pMatrl1, pMatrl2: tsxMATERIAL): tsxBOOL; cdecl;
  TtsxMaterialGetIdProc = function (pMatrl: tsxMATERIAL): integer; cdecl;
  TtsxMaterialGetShadingTypeProc = function (pMatrl: tsxMATERIAL): integer; cdecl;
  TtsxMaterialGetFacetingTypeProc = function (pMatrl: tsxMATERIAL): integer; cdecl;
  TtsxMaterialGetAutofacetAngleProc = function (pMatrl: tsxMATERIAL): integer; cdecl;
  TtsxMaterialGetColorProc = procedure (pMatrl: tsxMATERIAL; var pColor: CtsxColor); cdecl;
  TtsxMaterialGetSurfacePropsProc = procedure (pMatrl: tsxMATERIAL; var pSprops: CtsxSurfaceProps); cdecl;
  TtsxMaterialGetIorProc = function (pMatrl: tsxMATERIAL): single; cdecl;
  TtsxMaterialGetTextureTypeProc = function (pMatrl: tsxMATERIAL): integer; cdecl;
  TtsxMaterialGetProcTextureProc = procedure (pMatrl: tsxMATERIAL; var pPrTxt: CtsxProcTexture); cdecl;
  TtsxMaterialGetTextureFilenameProc = procedure (pMatrl: tsxMATERIAL; szFilename: PChar;	iFilenameSize: integer); cdecl;
  TtsxMaterialGetTexturePropsProc = procedure (pMatrl: tsxMATERIAL; var pTxtrProps: CtsxTextureProps); cdecl;
  TtsxMaterialGetBumpFilenameProc = procedure (pMatrl: tsxMATERIAL; szFilename: PChar;	iFilenameSize: integer); cdecl;
  TtsxMaterialGetBumpPropsProc = procedure (pMatrl: tsxMATERIAL; var pBumpProps: CtsxBumpProps); cdecl;
  TtsxMaterialGetEnvrFilenameProc = procedure (pMatrl: tsxMATERIAL; szFilename: PChar; iFilenameSize: integer); cdecl;
  TtsxMaterialGetEnvrFlagsProc = function (pMatrl: tsxMATERIAL): integer; cdecl;
  TtsxMaterialSetShadingTypeProc = procedure (pMatrl: tsxMATERIAL; shading: integer); cdecl;
  TtsxMaterialSetFacetingTypeProc = procedure (pMatrl: tsxMATERIAL; faceting: integer); cdecl;
  TtsxMaterialSetAutofacetAngleProc = procedure (pMatrl: tsxMATERIAL; angleDegrees: integer); cdecl;
  TtsxMaterialSetColorProc = procedure (pMatrl: tsxMATERIAL; var pColor: CtsxColor); cdecl;
  TtsxMaterialSetSurfacePropsProc = procedure (pMatrl: tsxMATERIAL; var pSprops: CtsxSurfaceProps); cdecl;
  TtsxMaterialSetIorProc = procedure (pMatrl: tsxMATERIAL; ior: single); cdecl;
  TtsxMaterialSetTextureTypeProc = procedure (pMatrl: tsxMATERIAL; txtype: integer); cdecl;
  TtsxMaterialSetProcTextureProc = procedure (pMatrl: tsxMATERIAL; var pPrTxt: CtsxProcTexture); cdecl;
  TtsxMaterialSetTextureFilenameProc = procedure (pMatrl: tsxMATERIAL; szFilename: PChar); cdecl;
  TtsxMaterialSetTexturePropsProc = procedure (pMatrl: tsxMATERIAL; var pTxtrProps: CtsxTextureProps); cdecl;
  TtsxMaterialSetBumpFilenameProc = procedure (pMatrl: tsxMATERIAL; szFilename: PChar); cdecl;
  TtsxMaterialSetBumpPropsProc = procedure (pMatrl: tsxMATERIAL; var pBumpProps: CtsxBumpProps); cdecl;
  TtsxMaterialSetEnvrFlagsProc = procedure (pMatrl: tsxMATERIAL; flags: integer); cdecl;
//3.1 API additions
  TtsxMaterialSetEnvrFilenameProc = procedure (pMatrl: tsxMATERIAL; szFilename: PChar); cdecl;


////////// tsxScene.H /////////////
  TtsxGetSceneProc = function : tsxScene; cdecl;
  TtsxSceneAddObjectProc = procedure (pGNode: tsxGNode; bNoDraw: tsxBOOL); cdecl;
  TtsxSceneGetFirstNodeProc = function : tsxGNode; cdecl;
  TtsxSceneIsFogOnProc = function : tsxBOOL; cdecl;
  TtsxSceneSwitchFogProc = procedure (fog_on: tsxBOOL); cdecl;
  TtsxSceneGetFogParmsProc = procedure (pFogRed: PInteger;
                                   pFogBlue: PInteger;
                                   pFogGreen: PInteger;
                                   pFogNear: PInteger;
                                   pFogFar: PInteger;
                                   pFogMax: PInteger); cdecl;
  TtsxSceneSetFogParmsProc = procedure (FogRed: Word;
                                   FogBlue: Word;
                                   FogGreen: Word;
                                   FogNear: Integer;
                                   FogFar: Integer;
                                   FogMax: Integer); cdecl;
  TtsxSceneDrawProc = procedure ; cdecl;

////////// tsxGNode.H ///////////
  TtsxIsGNodeProc = function (pSobj: tsxSObj): tsxBOOL; cdecl;
  TtsxGNodeCopyProc = function (pGNode: tsxGNode; var ppGNodeCopy: tsxGNode): tsxERR; cdecl;
  TtsxGNodeGetRootProc = function (pGNode: tsxGNode): tsxGNode; cdecl;
  TtsxGNodeGetParentProc = function (pGNode: tsxGNode): tsxGNode; cdecl;
  TtsxIsGNodeInSceneProc = function (pGNode: tsxGNode): tsxBOOL; cdecl;
  TtsxGNodeGetFirstChildProc = function (pGNode: tsxGNode): tsxGNode; cdecl;
  TtsxGNodeGetNextProc = function (pGNode: tsxGNode): tsxGNode; cdecl;
  TtsxGNodeGetPrevProc = function (pGNode: tsxGNode): tsxGNode; cdecl;
  TtsxIsGNodeSubobjProc = function (pGNode, pCheckGNode: tsxGNode): tsxBOOL; cdecl;
  TtsxGNodeDrawProc = procedure (pGNode: tsxGNode); cdecl;
  TtsxGNodeGetNameProc = function (pGNode: tsxGNode; pName: PChar; iNameSz: Integer): integer; cdecl;
  TtsxGNodeSetNameProc = function (pGNode: tsxGNode; szNewName: PChar): tsxERR; cdecl;
  TtsxGNodeGetAxesPositionProc = function (pGNode: tsxGNode; var posn: CtsxVector3f): PCtsxVector3f; cdecl;
  TtsxGNodeGetAxesOrientationProc = function (pGNode: tsxGNode; var axes: CtsxAxes3f): PCtsxAxes3f; cdecl;
  TtsxGNodeGetBBoxProc = function (pGNode: tsxGNode; var pBBox: CtsxBBox3f): tsxERR; cdecl;
  TtsxGNodeGetPureTransformProc = function (pGNode: tsxGNode; var pTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxGNodeTranslateProc = procedure (pGNode: tsxGNode; const pVec: CtsxVector3f; frameid: tsxFRAMEID); cdecl;
  TtsxGNodeRotateProc = procedure (pGNode: tsxGNode; const pCenter, pAxis: CtsxVector3f;
                           fAngle: Single; frameid: TSXFRAMEID); cdecl;
  TtsxGNodeScaleProc = procedure (pGNode: tsxGNode; const pScaleFacs: CtsxVector3f; frameid: TSXFRAMEID); cdecl;
  TtsxGNodeScaleUniformProc = procedure (pGNode: tsxGNode; fScaleFac: Single; frameid: TSXFRAMEID); cdecl;
  // 3.1 API additions
  TtsxGNodeGetLocationProc = function (pGNode: tsxGNode; var loc : CtsxVector3f ): PCtsxVector3f; cdecl;
  TtsxGNodeGetRotationProc = function (pGNode: tsxGNode; var rot : CtsxVector3f ): PCtsxVector3f; cdecl;
  TtsxGNodeGetSizeProc = function (pGNode: tsxGNode; var size : CtsxVector3f ): PCtsxVector3f; cdecl;
  TtsxGNodeSetLocationProc = procedure (pGNode: tsxGNode; var loc : CtsxVector3f ); cdecl;
  TtsxGNodeSetRotationProc = procedure (pGNode: tsxGNode; var rot : CtsxVector3f ); cdecl;
  TtsxGNodeSetSizeProc = procedure (pGNode: tsxGNode; var size : CtsxVector3f ); cdecl;
  TtsxGNodeGetTransformProc = function (pGNode: tsxGNode; var pTxmx : CtsxTxmx3f ): PCtsxTxmx3f; cdecl;
  // end 3.1 API additions
  // 3.2 API additions
  tsxUNITS = ( e_tsxUNITS_MILLIMETERS,
               e_tsxUNITS_CENTIMETERS,
               e_tsxUNITS_METERS,
               e_tsxUNITS_KILOMETERS,
               e_tsxUNITS_INCHES,
               e_tsxUNITS_FEET,
               e_tsxUNITS_YARDS,
               e_tsxUNITS_MILES,
               e_tsxUNITS_POINTS);
  TtsxGNodeGetUnitsProc = function (pGNode: tsxGNode): tsxUNITS; cdecl;
  TtsxGNodeSetUnitsProc = procedure (pGNode: tsxGNode; units: tsxUNITS); cdecl;
  // end 3.2 API additions

////////// tsxCamra.H ///////////
  TtsxCameraCreateInSceneProc = function : tsxERR;

////////// tsxLight.H ///////////
Type
  tsxLIGHT_TYPE = (e_tsxLT_UNDEF, e_tsxLT_INFINITE, e_tsxLT_LOCAL, e_tsxLT_SPOT);
  // Light intensity falloff
  tsxLIGHT_FALLOFF = (e_tsxLF_UNDEF, e_tsxLF_ZERO, e_tsxLF_LINEAR, e_tsxLF_SQUARED);
  // Shadow type
  tsxLIGHT_SHADOWING = (e_tsxLS_NOSHADOWS, e_tsxLS_RAYTRACE, e_tsxLS_SHADOWMAP);
  // Shadow-map Size
  tsxLIGHT_SMSIZE = (e_tsxSMZ_UNDEF, e_tsxSMZ_LOW, e_tsxSMZ_MEDIUM, e_tsxSMZ_HIGH);
  // Shadow-map sharpness
  tsxLIGHT_SMSHARPNESS = (e_tsxSMS_UNDEF, e_tsxSMS_LOW, e_tsxSMS_MEDIUM, e_tsxSMS_HIGH);

  TtsxLightCreateInSceneProc = function (LtType: tsxLIGHT_TYPE): tsxERR; cdecl;
  TtsxLightGetTypeProc = function (pLight: tsxLIGHT): tsxLIGHT_TYPE; cdecl;
  TtsxLightGetColorProc = procedure (pLight: tsxLIGHT; var pColor: CtsxColor); cdecl;
  TtsxLightSetColorProc = procedure (pLight: tsxLIGHT; var pColor: CtsxColor); cdecl;
  TtsxLightGetIntensityProc = function (pLight: tsxLIGHT): single; cdecl;
  TtsxLightSetIntensityProc = function (pLight: tsxLIGHT; fNewIntens: single): single; cdecl;
  TtsxLightGetFalloffProc = function (pLight: tsxLIGHT): tsxLIGHT_FALLOFF; cdecl;
  TtsxLightSetFalloffProc = function (pLight: tsxLIGHT; NewFo: tsxLIGHT_FALLOFF): tsxLIGHT_FALLOFF; cdecl;
  TtsxLightGetShadTypeProc = function (pLight: tsxLIGHT): tsxLIGHT_SHADOWING; cdecl;
  TtsxLightSetShadTypeProc = function (pLight: tsxLIGHT; newShType: tsxLIGHT_SHADOWING): tsxLIGHT_SHADOWING; cdecl;
  TtsxLightIsShadImgDepProc = function (pLight: tsxLIGHT): tsxBOOL; cdecl;
  TtsxLightSetShadImgDepProc = function (pLight: tsxLIGHT; bDependent: tsxBOOL): tsxBOOL; cdecl;
  TtsxLightGetShmapSizeProc = function (pLight: tsxLIGHT): tsxLIGHT_SMSIZE; cdecl;
  TtsxLightSetShmapSizeProc = function (pLight: tsxLIGHT; NewSz: tsxLIGHT_SMSIZE): tsxLIGHT_SMSIZE; cdecl;
  TtsxLightGetShmapSharpnessProc = function (pLight: tsxLIGHT): tsxLIGHT_SMSHARPNESS; cdecl;
  TtsxLightSetShmapSharpnessProc = function (pLight: tsxLIGHT; NewSharpness: tsxLIGHT_SMSHARPNESS): tsxLIGHT_SMSHARPNESS; cdecl;
  TtsxLightGetSpotAngleProc = function (pLight: tsxLIGHT): single; cdecl;
  TtsxLightSetSpotAngleProc = function (pLight: tsxLIGHT; newAngle: single): single; cdecl;
  TtsxLightGetSpotRatioProc = function (pLight: tsxLIGHT): single; cdecl;
  TtsxLightSetSpotRatioProc = function (pLight: tsxLIGHT; newRatio: single): single; cdecl;

////////// tsxMNode.H ////////////
  TtsxIsMNodeProc = function (pSobj: tsxSObj): TSXBOOL; cdecl;
  TtsxMNodeIsVisibleProc = function (pMNode: tsxMNode): TSXBOOL; cdecl;
  TtsxMNodeSetVisibleProc = procedure (pMNode: tsxMNode); cdecl;
  TtsxMNodeSetInvisibleProc = procedure (pMNode: tsxMNode); cdecl;
  TtsxMNodeAreAxesVisibleProc = function (pMNode: tsxMNode): tsxBOOL; cdecl;
  TtsxCurrobjToggleAxesProc = procedure; cdecl;
  TtsxMNodeSetAxesPositionProc = procedure (pMNode: tsxMNode; const posn: CtsxVector3f); cdecl;
  TtsxMNodeSetAxesOrientationProc = procedure (pMNode: tsxMNode; const axes: CtsxVector3f); cdecl;

/////////// tsxGroup.H ////////////
  TtsxGroupNewWithCurrobjProc = function (pGNode: tsxGNode): tsxGroup; cdecl;
  TtsxGroupAtCurrobjProc = function (pGNode: tsxGNode): tsxGroup; cdecl;
  TtsxGroupRemoveCurrobjProc = function : tsxGNode; cdecl;

/////////// tsxPolyh.H ////////////
Type
  CtsxFaceVx = record
    vix: integer;  // Index into Polyhedron's Vertex Array (see `tsxPolyhGetVxArray').
    uvix: integer; // Index into Polyhedron's UV Array.
    nextfaceix: integer;  // Index (in Polyh) of next face containing this vertex.
    color: CtsxColor;  //IF color.alpha == 0 THEN use material color
    end;
  PCtsxFaceVx = ^CtsxFaceVx;
  CtsxFaceVxArray = array[0..1000] of CtsxFaceVx;
  PCtsxFaceVxArray = ^CtsxFaceVxArray;

  CtsxFace = record
    nbrVxs: smallint;	//Nbr of Vertices
    pFVxs: PCtsxFaceVxArray;	//Array of Face-Vertex data, Vertices in clockwise order.
    r1: Byte;	// reserved
    r2: Byte;	// reserved
    r3: Pointer;		// reserved
    normVxs: array[0..2] of Byte;	//Indices into pFVxs of 3 vertices
    materialIdx: word;	//ID of Face's material
    end;
  PCtsxFace = ^CtsxFace;
  CtsxFaceArray = array[0..9000] of PCtsxFace;
  PCtsxFaceArray = ^CtsxFaceArray;

Const
  tsx_NORM_VXS_ORIGIN = 1;
  tsx_NORM_VXS_RAY1   = 0;
  tsx_NORM_VXS_RAY2   = 2;

type
  TtsxPolyhCreateProc = function (var ppPolyh: tsxPOLYHEDRON): tsxERR; cdecl;
  TtsxPolyhClearProc = procedure (pPolyh: tsxPOLYHEDRON); cdecl;
  TtsxPolyhTessellateProc = function (pPolyh: tsxPOLYHEDRON): tsxERR; cdecl;
  TtsxPolyhMarkGeomChangeProc = procedure (pPolyh: tsxPOLYHEDRON); cdecl;
  TtsxPolyhMarkTopologyChangeProc = procedure (pPolyh: tsxPOLYHEDRON); cdecl;
  TtsxPolyhMarkMaterialChangeProc = procedure (pPolyh: tsxPOLYHEDRON); cdecl;
  TtsxPolyhGetNbrVxsProc = function (pPolyh: tsxPOLYHEDRON): integer; cdecl;
  TtsxPolyhGetVxArrayProc = function (pPolyh: tsxPOLYHEDRON): PCtsxVector3fArray; cdecl;
  TtsxPolyhGetVxTxmxProc = function (pPolyh: tsxPOLYHEDRON; var pTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxPolyhSetVxsProc = procedure (pPolyh: tsxPOLYHEDRON; iNbrVxs: integer; pVxs: PCtsxVector3fArray); cdecl;
  TtsxPolyhSetVxTxmxProc = procedure (pPolyh: tsxPOLYHEDRON; const pTxmx: CtsxTxmx3f); cdecl;
  TtsxFaceAllocProc = function : PCtsxFace; cdecl;
  TtsxFaceDeleteProc = procedure (pFace: PCtsxFace); cdecl;
  TtsxFaceCopyProc = function (var ppFaceCopy: PCtsxFace; pFace: PCtsxFace): tsxERR; cdecl;
  TtsxPolyhGetNbrFacesProc = function (pPolyh: tsxPOLYHEDRON): integer; cdecl;
  TtsxPolyhGetFaceptrArrayProc = function (pPolyh: tsxPOLYHEDRON): PCtsxFaceArray; cdecl;
  TtsxPolyhSetFacesProc = procedure (pPolyh: tsxPOLYHEDRON; iNbrFaces: integer; pFaces: PCtsxFaceArray); cdecl;
  TtsxPolyhGetFaceNormalProc = procedure (pPolyh: tsxPOLYHEDRON; pFace: PCtsxFace; var pNorm: CtsxVector3f); cdecl;
  TtsxPolyhComputeFaceAdjacencyProc = function (pPolyh: tsxPOLYHEDRON): tsxERR; cdecl;
  TtsxPolyhGetNbrUVProc = function (pPolyh: tsxPOLYHEDRON): integer; cdecl;
  TtsxPolyhGetUVArrayProc = function (pPolyh: tsxPOLYHEDRON): PCtsxUVArray; cdecl;
  TtsxPolyhSetUVArrayProc = procedure (pPolyh: tsxPOLYHEDRON; iNbrUV: integer; pUV: PCtsxUVArray); cdecl;
  TtsxPolyhInitializeMaterialsProc = function (pPolyh: tsxPOLYHEDRON): tsxERR; cdecl;
  TtsxPolyhPaintProc = procedure (pPolyh: tsxPOLYHEDRON; pMatrl: tsxMATERIAL); cdecl;
  TtsxPolyhPaintFaceProc = function (pPolyh: tsxPOLYHEDRON; iFaceIndex: integer; pMatrl: tsxMATERIAL; bClearVxColors: tsxBOOL): tsxBOOL; cdecl;
  TtsxPolyhPaintVertexProc = function (pPolyh: tsxPOLYHEDRON; iVxIndex: integer; var pColor: CtsxColor): tsxBOOL; cdecl;
  TtsxPolyhGetMaterialProc = function (pPolyh: tsxPOLYHEDRON; materialIdx: word; pMatrl: tsxMATERIAL): tsxBOOL; cdecl;
  TtsxPolyhPhrenderFaceProc = function (pPolyh: tsxPOLYHEDRON; iFaceIndex: integer): tsxERR; cdecl;
  TtsxCreateSphereProc = function (NbrLongitudes, NbrLattitudes: integer; radius: single): tsxPOLYHEDRON; cdecl;
  TtsxCreateCylinderProc = function (NbrLongitudes, NbrLattitudes: integer; topRadius, botRadius, height: single): tsxPOLYHEDRON; cdecl;
  TtsxCreateConeProc = function (NbrLongitudes, NbrLattitudes: integer; botRadius, height: single): tsxPOLYHEDRON; cdecl;
  TtsxCreateCubeProc = function (nbrSections: integer; x, y, z: single): tsxPOLYHEDRON; cdecl;
  TtsxCreateTorusProc = function (NbrLongitudes, NbrLattitudes: integer; innerRadius: single): tsxPOLYHEDRON; cdecl;
  TtsxCreatePlaneProc = function (nbrXrects, nbrYrects: integer): tsxPOLYHEDRON; cdecl;
//3.1 API additions
  TtsxPolyhMarkVertexChangeProc = procedure ( pPolyh :tsxPOLYHEDRON ; vertex : integer ); cdecl;

///////// tsxSelec.H //////////
Type
  tsxSELECTMODE =(
    e_tsxSELECT,	// Ordinary Selection of a Node.
    e_tsxERASE,		// For selecting an object after erasing curr obj.
    e_tsxGLUE);		// For selecting an object to attach to curr obj.

  TtsxIsSelectableProc = function (pSobj: tsxSOBJ): tsxBOOL; cdecl;
  TtsxIsSelectedProc = function (pSobj: tsxSOBJ): tsxBOOL; cdecl;
  TtsxGetCurrentSelectionProc = function : tsxSOBJ; cdecl;
  TtsxSelectSobjProc = procedure (pSobj: tsxSOBJ; selmode: tsxSELECTMODE; bNoDraw : tsxBOOL); cdecl;
  TtsxSelectDownProc = procedure ; cdecl;
  TtsxSelectUpProc = procedure ; cdecl;
  TtsxSelectNextProc = procedure ; cdecl;
  TtsxSelectPrevProc = procedure ; cdecl;
  TtsxSelectionChangedCBProc = procedure (tsxid: integer; func: tsxBasicCallbackFP); cdecl;
  TtsxCurrobjReplaceProc = procedure (pNewGNode: tsxGNODE; bNoDraw: tsxBOOL); cdecl;
  TtsxCurrobjCopyProc = function : tsxERR; cdecl;
  TtsxCurrobjDrawProc = procedure ; cdecl;
  TtsxCurrobjPhrenderProc = procedure ; cdecl;

////////// tsxAnim.H ///////////
Type
  tsxKFRAME_TYPE = (
      e_tsxKFT_UNDEF,
      e_tsxKFT_LOOK,
      e_tsxKFT_ROTATE,
      e_tsxKFT_MOVE,
      e_tsxKFT_SCALE,
      e_tsxKFT_DEFORM,
      e_tsxKFT_RECT,
      e_tsxKFT_COLOR,
      e_tsxKFT_SURFACE,
      e_tsxKFT_BUMP,
      e_tsxKFT_TEXTURE,
      e_tsxKFT_PROCTEX,
      e_tsxKFT_ENVIRON,
      e_tsxKFT_FACETCOS,
      e_tsxKFT_LCOLOR,
      e_tsxKFT_BACKGROUND,
      e_tsxKFT_GLOBENV,
      e_tsxKFT_FOG,
      e_tsxKFT_RAYTRACE,
      e_tsxKFT_PLUGIN,
      e_tsxKFT_NAIL,
      e_tsxKFT_JOINT,
      e_tsxKFT_KINEMATICS,
      e_tsxKFT_VERTEX,
      e_tsxKFT_INVISIBLE,
      e_tsxKFT_END);

  //these functions are now obsolete in 3.1 and higher API versions,
  //but kept for compatibility with 3.0
  TtsxGNodeCreateScriptProc = function (pGNode: tsxGNODE): tsxERR; cdecl;
  TtsxGNodeCreateScriptTreeProc = function (pGNode: tsxGNODE): tsxERR; cdecl;
  TtsxGNodeDeleteScriptProc = function (pGNode: tsxGNODE): tsxERR; cdecl;
  TtsxGNodeDeleteScriptTreeProc = function (pGNode: tsxGNODE): tsxERR; cdecl;
  TtsxGNodeCopyScriptProc = function (pGNodeTo: tsxGNODE; pGNodeFrom: tsxGNODE): tsxERR; cdecl;
  TtsxGNodeSetFrameProc = function (pGNode: tsxGNODE; keyfType: tsxKFRAME_TYPE): tsxERR; cdecl;
  TtsxGNodeUnsetFrameProc = function (pGNode: tsxGNODE; keyfType: tsxKFRAME_TYPE): tsxERR; cdecl;
  TtsxAnimSetActiveFrameProc = procedure (pGNode: tsxGNODE; frameNbr: single); cdecl;
  //end of obsolete animation functions

//3.1 API additions
  TtsxSobjCreateScriptProc = function (pSobj : tsxSOBJ): tsxERR; cdecl;
  TtsxSobjTreeCreateScriptProc = function (pSobj : tsxSOBJ): tsxERR; cdecl;
  TtsxSobjDeleteScriptProc = function (pSobj : tsxSOBJ): tsxERR; cdecl;
  TtsxSobjTreeDeleteScriptProc = function (pSobj : tsxSOBJ): tsxERR; cdecl;
  TtsxSobjCopyScriptProc = function (pSobjTo, pSobjFrom: tsxSOBJ): tsxERR; cdecl;
  TtsxSobjEmptyScriptProc = function (pSobj : tsxSOBJ): tsxBOOL; cdecl;
  TtsxSobjTreeEmptyScriptProc = function (pSobj : tsxSOBJ): tsxBOOL; cdecl;
  tsxANIM_PLAYMODE = (e_tsxPLAY_SCENE, e_tsxPLAY_OBJECT);
  TtsxAnimSetActiveTimeProc = procedure ( frameNbr : single ); cdecl;
  TtsxAnimGetActiveTimeProc = function: single; cdecl;
  TtsxActiveTimeChangedCBProc = procedure (tsxid: integer; func: tsxBasicCallbackFP); cdecl;
  TtsxAnimSetPlayRangesProc = procedure ( range_start, range_end : single ); cdecl;
  TtsxAnimGetPlayRangesProc = procedure ( var fStart, fEnd : float ); cdecl;
  TtsxAnimSetPlayModeProc = procedure ( mode : tsxANIM_PLAYMODE ); cdecl;
  TtsxAnimGetPlayModeProc = function: tsxANIM_PLAYMODE; cdecl;
  TtsxAnimPlayProc = function: tsxERR; cdecl;
  TtsxAnimGetBaseFramerateProc = function: single; cdecl;
  TtsxSobjSetFrameProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): tsxERR; cdecl;
  TtsxSobjUnsetFrameProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): tsxERR; cdecl;
  TtsxSobjGetFirstTimeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): single; cdecl;
  TtsxSobjGetLastTimeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): single; cdecl;
  TtsxSobjGetNextTimeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; time : single ): single; cdecl;
  TtsxSobjGetPrevTimeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; time : single ): single; cdecl;
  TtsxSobjTreeGetNextTimeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; time : single ): single; cdecl;
  TtsxSobjTreeGetPrevTimeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; time : single ): single; cdecl;
  TtsxSobjGetMinMaxTimeProc = function ( pSobj : tsxSObj; var min, max : single ): tsxERR; cdecl;
  TtsxSobjTreeGetMinMaxTimeProc = function ( pSobj : tsxSObj; var min, max : single ): tsxERR; cdecl;
  TtsxFrameCreateProc = function ( ppKFrame: PtsxKEYFRAME; keyfType : tsxKFRAME_TYPE ): tsxERR; cdecl;
  TtsxFrameDeleteProc = procedure ( pKFrame : tsxKEYFRAME ); cdecl;
  TtsxFrameGetTypeProc = function ( pKFrame : tsxKEYFRAME ): tsxKFRAME_TYPE; cdecl;
  TtsxAnimGetTypeNameProc = function ( keyfType : tsxKFRAME_TYPE ): PChar; cdecl;
  TtsxFrameCopyProc = function ( pKFrame : tsxKEYFRAME; ppKFrameCopy: PtsxKEYFRAME ): tsxERR; cdecl;
  TtsxSobjInsertFrameProc = function ( pSobj : tsxSObj; pKFrame : tsxKEYFRAME ; segment : integer; time : single ): tsxERR; cdecl;
  TtsxFrameRemoveProc = procedure ( pKFrame : tsxKEYFRAME ); cdecl;
  TtsxSobjGetFirstFrameProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; segment : integer ): tsxKEYFRAME; cdecl;
  TtsxSobjGetLastFrameProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; segment : integer ): tsxKEYFRAME; cdecl;
  TtsxSobjGetFrameProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; segment : integer; time : single ): tsxKEYFRAME; cdecl;
  TtsxSobjGetLeftFrameProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; segment : integer; time : single ): tsxKEYFRAME; cdecl;
  TtsxFrameGetNextFrameProc = function ( pKFrame : tsxKEYFRAME ): tsxKEYFRAME; cdecl;
  TtsxFrameGetPrevFrameProc = function ( pKFrame : tsxKEYFRAME ): tsxKEYFRAME; cdecl;
  TtsxFrameGetTimeProc = function ( pKFrame : tsxKEYFRAME ): single; cdecl;
  TtsxFrameGetSplineParametersProc = function ( pKFrame : tsxKEYFRAME; var continuity, tension, bias : single ): tsxERR; cdecl;
  TtsxFrameSetSplineParametersProc = function ( pKFrame : tsxKEYFRAME; continuity, tension, bias : single ): tsxERR; cdecl;
  TtsxFrameGetDataLengthProc = function ( pKFrame : tsxKEYFRAME ): integer; cdecl;
  TtsxFrameGetDataDescriptionProc = function ( pKFrame : tsxKEYFRAME; var ppDescr : PSmallInt ): integer; cdecl;
  TtsxFrameGetDataProc = function ( pKFrame : tsxKEYFRAME; pData : Pointer; length : integer ): integer; cdecl;
  TtsxFrameSetDataProc = function ( pKFrame : tsxKEYFRAME; pData : Pointer; length : integer ): integer; cdecl;
  TtsxSobjGetInterpolatedDataProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; segment : integer; time : single; pData : Pointer; length : integer ): integer; cdecl;
  tsxAnimOriginCallbackFP = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE): tsxERR; cdecl;
  tsxAnimFrameCallbackFP = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; time : single): tsxERR; cdecl;
  tsxAnimCallbackFP = record
  	extAnimSetOriginCB  :tsxAnimOriginCallbackFP;
  	extAnimPutOriginCB  :tsxAnimOriginCallbackFP;
  	extAnimSetFrameCB   :tsxAnimFrameCallbackFP;
  	extAnimUnSetFrameCB :tsxAnimFrameCallbackFP;
  	extAnimPutFrameCB   :tsxAnimFrameCallbackFP;
    end;
  PtsxAnimCallbackFP = ^tsxAnimCallbackFP;
  TtsxAnimCreateAttributeProc = function (
                              tsxid: integer;
                              name: PChar;
                              attrCallback: PtsxAnimCallbackFP;
                              attrDataLength: integer;
                              var kfDataDescription: smallint;
                              kfDataDescriptionSize: integer): tsxKFRAME_TYPE; cdecl;
  TtsxSobjAddAttributeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): tsxERR; cdecl;
  TtsxSobjGetFirstAttributeProc = function ( pSobj : tsxSObj ): tsxKFRAME_TYPE; cdecl;
  TtsxSobjGetNextAttributeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): tsxKFRAME_TYPE; cdecl;
  TtsxSobjHasAttributeProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): tsxBOOL; cdecl;
  TtsxSobjGetFirstSegmentProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE ): integer; cdecl;
  TtsxSobjGetNextSegmentProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; segment : integer ): integer; cdecl;

const
 	pdOpaque         = $0000;
 	pdChar           = $0001;
 	pdShort          = $0002;
 	pdLong           = $0003;
 	pdUnsignedChar   = $0004;
 	pdUnsignedShort  = $0005;
 	pdUnsignedLong   = $0006;
 	pdDouble         = $0008;
 	pdFloat          = $0009;

type
  TtsxSobjGetAttrDataLengthProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE): integer; cdecl;
  TtsxSobjGetAttrDataProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; pData : Pointer; length : integer): integer; cdecl;
  TtsxSobjSetAttrDataProc = function ( pSobj : tsxSObj; keyfType : tsxKFRAME_TYPE; pData : Pointer; length : integer): integer; cdecl;


////////// tsxMisc.H ///////////
  TtsxGetTsVersionNbrsProc = procedure (var pMajor, pMinor : smallint); cdecl;
  TtsxGetMainViewHwndProc = function : HWnd; cdecl;
  TtsxForceLeftClickProc = function (tsxid: Integer): tsxErr; cdecl;
  TtsxMallocProc = function (size: DWord): Pointer; cdecl;
  TtsxCallocProc = function (num, size: DWord) : Pointer; cdecl;
  TtsxReallocProc = function (memblock: Pointer; size: Word) : Pointer; cdecl;
  TtsxFreeProc = procedure (memblock: Pointer); cdecl;

////////// tsxMouse.H ///////////
Type
  tsxMousetool = Pointer;
  PtsxMousetool = ^tsxMousetool;

  tsxMtoolMsg = function(mt: PtsxMousetool; msg, x, y, data: smallint): Boolean; cdecl;
  tsxMtoolClick = function(mt: PtsxMousetool; x, y, flags: smallint): Boolean; cdecl;

Const
  tsxMTM_ACTIVATE	   = 1;
  tsxMTM_DEACTIVATE  = 2;
  tsxMTM_DESELECT	   = 3;
  tsxMTM_SELECT	     = 4;
  tsxMTM_KEY	       = 5;
  tsxMTM_CREATE	     = 6;
  tsxMTM_DESTROY	   = 7;
  tsxMTM_ERASE	     = 8;
  tsxMTM_CHECKCURSOR = 9;
  tsxMTF_RIGHT       = $01;
  tsxMTF_LEFT        = $02;
  tsxMTF_SHIFT       = $04;
  tsxMTF_CTRL	       = $08;
  tsxMTF_OUTSIDE     = $4000;
  tsxMTF_DBLCLK      = $8000;

type
  TtsxMtoolInstallProc = function (tsxid: integer; pMtoolMsgFunc: tsxMtoolMsg; pMtoolEventFunc: tsxMtoolClick): tsxERR; cdecl;
  TtsxMtoolRemoveProc = procedure (tsxid: integer); cdecl;

//////// tsxAView.H //////////
Type
  tsxRENDERMODE = (e_tsxRM_UNDEF, e_tsxRM_WIRE, e_tsxRM_3DR, e_tsxRM_D3D);
  tsxVIEWMODE = (e_tsxVM_PERSPECTIVE, e_tsxVM_TOP, e_tsxVM_FRONT, e_tsxVM_LEFT, e_tsxVM_CAMERA);

  TtsxAViewRefreshProc = procedure ; cdecl;
  TtsxAViewGetRenderModeProc = function : tsxRENDERMODE; cdecl;
  TtsxAViewGetViewModeProc = function : tsxVIEWMODE; cdecl;
  TtsxAViewSetViewModeProc = function (newmode: tsxVIEWMODE): tsxBOOL; cdecl;
  TtsxAViewSetCameraModeProc = function (pGNode: tsxGNODE): tsxERR; cdecl;
  TtsxAViewUnsetCameraModeProc = procedure ; cdecl;
  TtsxAViewGetCameraObjProc = function : tsxSOBJ; cdecl;
  TtsxAViewGetPositionProc = procedure (var posn: CtsxVector3f); cdecl;
  TtsxAViewGetDirectionProc = procedure (var dirn: CtsxVector3f); cdecl;
  TtsxAViewGetBankAngleProc = function : single; cdecl;
  TtsxAViewGetZoomProc = function : single; cdecl;
  TtsxAViewSetViewProc = procedure (var pEyePosn, pLookAt: CtsxVector3f; bLevelIt: tsxBOOL); cdecl;
  TtsxAViewSetBankProc = procedure (bankAngle: single); cdecl;
  TtsxAViewSetZoomProc = function (newzoom: single ): single; cdecl;
  TtsxAViewGetWorld2VportTxmxProc = function (var pTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxAViewGetWorld2EyeTxmxProc = function (var pTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxAViewGetEye2WorldTxmxProc = function (var pTxmx: CtsxTxmx3f): PCtsxTxmx3f; cdecl;
  TtsxAViewGetMouseRayProc = procedure (mousePtX, mousePtY: smallint; var pRayOrigin, pRayDirn: CtsxVector3f); cdecl;
  TtsxAViewPickRootProc = function (x, y: smallint; var ppSobj: tsxSOBJ): tsxBOOL; cdecl;
  TtsxAViewPickVertexProc = function (pMNode: tsxMNODE; x, y: smallint; var ppPolyh: tsxPOLYHEDRON; var pVxIndex: integer): tsxBOOL; cdecl;
  TtsxAViewPickFaceProc = function (pMNode: tsxMNODE; x, y: smallint; var ppPolyh: tsxPOLYHEDRON; var pFxIndex: integer; var pFxDist: single): tsxBOOL; cdecl;
  TtsxAViewGetHwndProc = function : HWND; cdecl;
  // 3.2 API additions
const
  tsxLIL_EOLINES		= -1;
  tsxLIL_NEWSTART		= -2;

type
  tsxPEN = (
	e_tsxPEN_BACKGROUND,	  // background color
	e_tsxPEN_GRID,			  	// grid lines color
	e_tsxPEN_ACTIVE,	  		// active object color
	e_tsxPEN_SCENE,	  			// scene objects
	e_tsxPEN_ACTIVESUB,			// active sub object color
	e_tsxPEN_INACTIVESUB,		// inactive sub object color
	e_tsxPEN_CONTROL,			  // inactive control objects
	e_tsxPEN_ACTIVECONTROL,	// active control objects
	e_tsxPEN_RED,
	e_tsxPEN_GREEN,
	e_tsxPEN_BLUE );

  CtsxLineArray = array[0..1000] of long;
  TtsxAViewDrawLinelistProc = function ( var trans : CtsxTxmx3f;
                                         var vects : CtsxVector3fArray;
                                         numVects : long;
                                         var linelist : CtsxLineArray;
                                         var rect : CtsxRect ): tsxBOOL; cdecl;

  TtsxAViewSetPenProc = procedure ( pen : tsxPEN ); cdecl;
  TtsxAllViewsRefreshProc = procedure; cdecl;
  TtsxAViewGetScreenPointProc = procedure (
              var pWorldPt : CtsxVector3f; //IN: World coordinates
              var pScreenPtX : short; //OUT: screen X coordinate (in pixels)
              var pScreenPtY : short //OUT: screen Y coordinate
              ); cdecl;
  // end 3.2 API additions

//////// tsxExtOb.H //////////
  // 3.2 API additions
const
  e_tsxFILE_BINARY = 0;
  e_tsxFILE_ASCII  = 1;
  e_tsxFILE_READ   = 0;
  e_tsxFILE_WRITE  = 1;
  e_tsxGEOMETRY    = 1;
  e_tsxTOPOLOGY    = 2;
  e_tsxUNSELECTED  = 0;
  e_tsxSELECTED    = 1;
  tsxOMOVE_TRANSLATED	= $0001;  // a translation has been added
  tsxOMOVE_ROTATED    = $0002;	// a rotation has been added
  tsxOMOVE_SCALED     = $0004;  // a scaling has been added
  tsxERR_CLIPPED      = 0;
  tsxERR_VISIBLE      = 1;
  tsxERR_NOHIT        = 0;
  tsxERR_HIT          = 1;

type
  tsxFILE = Pointer;

  tsxFILE_MODE     = e_tsxFILE_BINARY..e_tsxFILE_ASCII;
  tsxFILE_IO       = e_tsxFILE_READ..e_tsxFILE_WRITE;
  tsxOBJECT_CHANGE = e_tsxGEOMETRY..e_tsxTOPOLOGY;
  tsxSELECT        = e_tsxUNSELECTED..e_tsxSELECTED;

  tsxCOLOR_MODE = (
    e_tsxEDIT_CURPEN,     // draw whole node leaving pens in current state
    e_tsxEDIT_ACTIVE,     // draw whole node in active foreground color representation
    e_tsxEDIT_ACTIVESUB,  // draw whole node in active subobject foreground color representation
    e_tsxEDIT_INACTIVE,   // draw whole node in inactive background color representation (as partof larger tree)
    e_tsxEDIT_INACTIVESUB // draw whole node in inactive subobject color (as part of tree) background representation
    );

  tsxObjectDeleteCallbackFP       = function ( pSobj : tsxSOBJ ): tsxERR; cdecl;
  tsxObjectCloneSelfCallbackFP    = function ( pSobj : tsxSOBJ; pClone : tsxSOBJ ): tsxERR; cdecl;
  tsxObjectFileIOCallbackFP       = function ( pSobj : tsxSOBJ; var fd: tsxFILE; io: tsxFILE_IO; mode: tsxFILE_MODE ): tsxERR; cdecl;
  tsxObjectRenderCallbackFP       = function ( pSobj : tsxSOBJ; edit : tsxCOLOR_MODE; var rect: CtsxRect ): tsxERR; cdecl;
  tsxObjectApplyObmoveCallbackFP  = function ( pSobj : tsxSOBJ; var trans: CtsxTxmx3f; flags: UnsignedShort ): tsxERR; cdecl;
  tsxObjectScanBCubeCallbackFP    = function ( pSobj : tsxSOBJ; var bc: CtsxBBox3f; var offset: CtsxTxmx3f ): tsxERR; cdecl;
  tsxObjectRayIntersectCallbackFP = function ( pSobj : tsxSOBJ; var torayspace: CtsxTxmx3f ): tsxERR; cdecl;
  tsxObjectSelectCallbackFP       = function ( pSobj : tsxSOBJ; select_type: tsxSELECT ): tsxERR; cdecl;
  tsxObjectHasChangedCallbackFP   = function ( pSobj : tsxSOBJ; change_type: tsxOBJECT_CHANGE ): tsxERR; cdecl;

const
  tsxVERSION_OBJECT = 2;

type
  tsxObjectCallbackFP = record
    version : integer;
    extObjectDeleteCB       : tsxObjectDeleteCallbackFP;
    extObjectCloneSelfCB    : tsxObjectCloneSelfCallbackFP;
    extObjectFileIOCB       : tsxObjectFileIOCallbackFP;
    extObjectRenderCB       : tsxObjectRenderCallbackFP;
    extObjectApplyObmoveCB  : tsxObjectApplyObmoveCallbackFP;
    extObjectScanBCubeCB    : tsxObjectScanBCubeCallbackFP;
    extObjectRayIntersectCB : tsxObjectRayIntersectCallbackFP;
    extObjectSelectCB       : tsxObjectSelectCallbackFP;
    extObjectHasChangedCB   : tsxObjectHasChangedCallbackFP
    end;

  TtsxObjectDefineProc = function ( tsxid: integer; name: Pchar; var objCallback : tsxObjectCallbackFP; objDataLength: long ): tsxSOBJTYPE; cdecl;
  TtsxObjectCreateProc = function ( objType: tsxSOBJTYPE ): tsxSOBJ; cdecl;
  TtsxObjectGetDataPtrProc = function ( pSobj: tsxSOBJ ): Pointer; cdecl;
  TtsxObjectGetTypeNameProc = function ( objType: tsxSOBJTYPE ): PChar; cdecl;

  tsxDataDeleteCallbackFP    = function ( pData : Pointer ): tsxERR; cdecl;
  tsxDataCloneSelfCallbackFP = function ( pData : Pointer; pClone : Pointer ): tsxERR; cdecl;
  tsxDataFileIOCallbackFP    = function ( pData : Pointer; var fd : tsxFILE; io : tsxFILE_IO; mode: tsxFILE_MODE ): tsxERR; cdecl;

const
  tsxVERSION_DATA = 2;

type
  tsxDataCallbackFP = record
    version : integer;
    extDataDeleteCB    : tsxDataDeleteCallbackFP;
    extDataCloneSelfCB : tsxDataCloneSelfCallbackFP;
    extDataFileIOCB    : tsxDataFileIOCallbackFP
    end;

  tsxDATATYPE = integer;

  TtsxDataDefineProc = function ( tsxid: integer; name: Pchar; var dataCallback: tsxDataCallbackFP; dataLength: long ): tsxDATATYPE; cdecl;

  TtsxSobjCreateDataProc = function ( pSobj: tsxSOBJ; dataType: tsxDATATYPE ): Pointer; cdecl;
  TtsxSobjGetDataProc = function ( pSobj: tsxSOBJ; dataType: tsxDATATYPE ): Pointer; cdecl;
  TtsxSobjDeleteDataProc = procedure ( pSobj: tsxSOBJ; dataType: tsxDATATYPE ); cdecl;

  //File I/O
  TtsxFILEerrorProc = function ( var fd: tsxFILE ): integer; cdecl;
  TtsxFILEgetcProc = function ( var fd: tsxFILE ): integer; cdecl;
  TtsxFILEgetsProc = function ( buf: PChar; size: integer ; var fd: tsxFILE ): PChar; cdecl;
  TtsxFILEprintfProc = function ( var fd: tsxFILE; const fmt: PChar ): integer; cdecl;
  TtsxFILEputcProc = function ( ch: integer; var fd: tsxFILE ): integer; cdecl;
  TtsxFILEputsProc = function ( const buf: PChar ; var fd: tsxFILE ): integer; cdecl;
  TtsxFILEreadProc = function ( buf: Pointer; size: integer; count: integer; var fd: tsxFILE ): integer; cdecl;
  TtsxFILEscanfProc = function ( var fd: tsxFILE; const fmt: PChar ): integer; cdecl;
  TtsxFILEseekProc = function ( var fd: tsxFILE; offset: long; where: integer ): integer; cdecl;
  TtsxFILEtellProc = function ( var fd: tsxFILE ): long; cdecl;
  TtsxFILEwriteProc = function ( const buf: Pointer; size: integer; count: integer; var fd: tsxFILE ): integer; cdecl;
  // end 3.2 API additions

{Function Variables}
var
  tsxGetTsVersionNbrs:            TtsxGetTsVersionNbrsProc;
  tsxGetMainViewHwnd:             TtsxGetMainViewHwndProc;
  tsxForceLeftClick:              TtsxForceLeftClickProc;
  tsxMalloc:                      TtsxMallocProc;
  tsxCalloc:                      TtsxCallocProc;
  tsxRealloc:                     TtsxReallocProc;
  tsxFree:                        TtsxFreeProc;
  tsxGetScene:                    TtsxGetSceneProc;
  tsxSceneAddObject:              TtsxSceneAddObjectProc;
  tsxSceneGetFirstNode:           TtsxSceneGetFirstNodeProc;
  tsxSceneIsFogOn:                TtsxSceneIsFogOnProc;
  tsxSceneSwitchFog:              TtsxSceneSwitchFogProc;
  tsxSceneGetFogParms:            TtsxSceneGetFogParmsProc;
  tsxSceneSetFogParms:            TtsxSceneSetFogParmsProc;
  tsxSceneDraw:                   TtsxSceneDrawProc;
  tsxSobjGetType:                 TtsxSobjGetTypeProc;
  tsxSobjDelete:                  TtsxSobjDeleteProc;
  tsxIsMNode:                     TtsxIsMNodeProc;
  tsxMNodeIsVisible:              TtsxMNodeIsVisibleProc;
  tsxMNodeSetVisible:             TtsxMNodeSetVisibleProc;
  tsxMNodeSetInvisible:           TtsxMNodeSetInvisibleProc;
  tsxMNodeAreAxesVisible:         TtsxMNodeAreAxesVisibleProc;
  tsxCurrobjToggleAxes:           TtsxCurrobjToggleAxesProc;
  tsxMNodeSetAxesPosition:        TtsxMNodeSetAxesPositionProc;
  tsxMNodeSetAxesOrientation:     TtsxMNodeSetAxesOrientationProc;
  tsxIsGNode:                     TtsxIsGNodeProc;
  tsxGNodeCopy:                   TtsxGNodeCopyProc;
  tsxGNodeGetRoot:                TtsxGNodeGetRootProc;
  tsxGNodeGetParent:              TtsxGNodeGetParentProc;
  tsxIsGNodeInScene:              TtsxIsGNodeInSceneProc;
  tsxGNodeGetFirstChild:          TtsxGNodeGetFirstChildProc;
  tsxGNodeGetNext:                TtsxGNodeGetNextProc;
  tsxGNodeGetPrev:                TtsxGNodeGetPrevProc;
  tsxIsGNodeSubobj:               TtsxIsGNodeSubobjProc;
  tsxGNodeDraw:                   TtsxGNodeDrawProc;
  tsxGNodeGetName:                TtsxGNodeGetNameProc;
  tsxGNodeSetName:                TtsxGNodeSetNameProc;
  tsxGNodeGetAxesPosition:        TtsxGNodeGetAxesPositionProc;
  tsxGNodeGetAxesOrientation:     TtsxGNodeGetAxesOrientationProc;
  tsxGNodeGetBBox:                TtsxGNodeGetBBoxProc;
  tsxGNodeGetPureTransform:       TtsxGNodeGetPureTransformProc;
  tsxGNodeTranslate:              TtsxGNodeTranslateProc;
  tsxGNodeRotate:                 TtsxGNodeRotateProc;
  tsxGNodeScale:                  TtsxGNodeScaleProc;
  tsxGNodeScaleUniform:           TtsxGNodeScaleUniformProc;
  tsxMagnitude3f:                 TtsxMagnitude3fProc;
  tsxMagnitude2f:                 TtsxMagnitude2fProc;
  tsxNormalize3f:                 TtsxNormalize3fProc;
  tsxNormalize2f:                 TtsxNormalize2fProc;
  tsxGetNormalized3f:             TtsxGetNormalized3fProc;
  tsxGetNormalized2f:             TtsxGetNormalized2fProc;
  tsxAddScalar3f:                 TtsxAddScalar3fProc;
  tsxMulScalar3f:                 TtsxMulScalar3fProc;
  tsxAddVec3f:                    TtsxAddVec3fProc;
  tsxSubVec3f:                    TtsxSubVec3fProc;
  tsxDProdVec3f:                  TtsxDProdVec3fProc;
  tsxXProdVec3f:                  TtsxXProdVec3fProc;
  tsxAngleBtwVecs3f:              TtsxAngleBtwVecs3fProc;
  tsxDistanceToPoint3f:           TtsxDistanceToPoint3fProc;
  tsxDistanceToLine3f:            TtsxDistanceToLine3fProc;
  tsxDistanceToPlane3f:           TtsxDistanceToPlane3fProc;
  tsxXProdMx3f:                   TtsxXProdMx3fProc;
  tsxInvertMx3f:                  TtsxInvertMx3fProc;
  tsxMakeMxFromXZO3f:             TtsxMakeMxFromXZO3fProc;
  tsxMakeRotMx3f:                 TtsxMakeRotMx3fProc;
  tsxMakePureRotMx3f:             TtsxMakePureRotMx3fProc;
  tsxGetMxTranslation3f:          TtsxGetMxTranslation3fProc;
  tsxSetMxTranslation3f:          TtsxSetMxTranslation3fProc;
  tsxAddMxTranslation3f:          TtsxAddMxTranslation3fProc;
  tsxSubMxTranslation3f:          TtsxSubMxTranslation3fProc;
  tsxUnMatrix3f:                  TtsxUnMatrix3fProc;
  tsxTransformVecs3f:             TtsxTransformVecs3fProc;
  tsxRotateVecs3f:                TtsxRotateVecs3fProc;
  tsxRotateAxes3f:                TtsxRotateAxes3fProc;
  tsxBBoxInit:                    TtsxBBoxInitProc;
  tsxBBoxIntersection:            TtsxBBoxIntersectionProc;
  tsxIsSelectable:                TtsxIsSelectableProc;
  tsxIsSelected:                  TtsxIsSelectedProc;
  tsxGetCurrentSelection:         TtsxGetCurrentSelectionProc;
  tsxSelectSobj:                  TtsxSelectSobjProc;
  tsxSelectDown:                  TtsxSelectDownProc;
  tsxSelectUp:                    TtsxSelectUpProc;
  tsxSelectNext:                  TtsxSelectNextProc;
  tsxSelectPrev:                  TtsxSelectPrevProc;
  tsxSelectionChangedCB:          TtsxSelectionChangedCBProc;
  tsxCurrobjReplace:              TtsxCurrobjReplaceProc;
  tsxCurrobjCopy:                 TtsxCurrobjCopyProc;
  tsxCurrobjDraw:                 TtsxCurrobjDrawProc;
  tsxCurrobjPhrender:             TtsxCurrobjPhrenderProc;
  tsxMaterialCreate:              TtsxMaterialCreateProc;
  tsxMaterialCreateCopy:          TtsxMaterialCreateCopyProc;
  tsxMaterialCreateCopyActive:    TtsxMaterialCreateCopyActiveProc;
  tsxMaterialDestroy:             TtsxMaterialDestroyProc;
  tsxMaterialSetActive:           TtsxMaterialSetActiveProc;
  tsxMaterialGetActive:           TtsxMaterialGetActiveProc;
  tsxMaterialCopyActive:          TtsxMaterialCopyActiveProc;
  tsxMaterialsAreEqual:           TtsxMaterialsAreEqualProc;
  tsxMaterialGetId:               TtsxMaterialGetIdProc;
  tsxMaterialGetShadingType:      TtsxMaterialGetShadingTypeProc;
  tsxMaterialGetFacetingType:     TtsxMaterialGetFacetingTypeProc;
  tsxMaterialGetAutofacetAngle:   TtsxMaterialGetAutofacetAngleProc;
  tsxMaterialGetColor:            TtsxMaterialGetColorProc;
  tsxMaterialGetSurfaceProps:     TtsxMaterialGetSurfacePropsProc;
  tsxMaterialGetIor:              TtsxMaterialGetIorProc;
  tsxMaterialGetTextureType:      TtsxMaterialGetTextureTypeProc;
  tsxMaterialGetProcTexture:      TtsxMaterialGetProcTextureProc;
  tsxMaterialGetTextureFilename:  TtsxMaterialGetTextureFilenameProc;
  tsxMaterialGetTextureProps:     TtsxMaterialGetTexturePropsProc;
  tsxMaterialGetBumpFilename:     TtsxMaterialGetBumpFilenameProc;
  tsxMaterialGetBumpProps:        TtsxMaterialGetBumpPropsProc;
  tsxMaterialGetEnvrFlags:        TtsxMaterialGetEnvrFlagsProc;
  tsxMaterialSetShadingType:      TtsxMaterialSetShadingTypeProc;
  tsxMaterialSetFacetingType:     TtsxMaterialSetFacetingTypeProc;
  tsxMaterialSetAutofacetAngle:   TtsxMaterialSetAutofacetAngleProc;
  tsxMaterialSetColor:            TtsxMaterialSetColorProc;
  tsxMaterialSetSurfaceProps:     TtsxMaterialSetSurfacePropsProc;
  tsxMaterialSetIor:              TtsxMaterialSetIorProc;
  tsxMaterialSetTextureType:      TtsxMaterialSetTextureTypeProc;
  tsxMaterialSetProcTexture:      TtsxMaterialSetProcTextureProc;
  tsxMaterialSetTextureFilename:  TtsxMaterialSetTextureFilenameProc;
  tsxMaterialSetTextureProps:     TtsxMaterialSetTexturePropsProc;
  tsxMaterialSetBumpFilename:     TtsxMaterialSetBumpFilenameProc;
  tsxMaterialSetBumpProps:        TtsxMaterialSetBumpPropsProc;
  tsxMaterialSetEnvrFlags:        TtsxMaterialSetEnvrFlagsProc;
  tsxCameraCreateInScene:         TtsxCameraCreateInSceneProc;
  tsxLightCreateInScene:          TtsxLightCreateInSceneProc;
  tsxLightGetType:                TtsxLightGetTypeProc;
  tsxLightGetColor:               TtsxLightGetColorProc;
  tsxLightSetColor:               TtsxLightSetColorProc;
  tsxLightGetIntensity:           TtsxLightGetIntensityProc;
  tsxLightSetIntensity:           TtsxLightSetIntensityProc;
  tsxLightGetFalloff:             TtsxLightGetFalloffProc;
  tsxLightSetFalloff:             TtsxLightSetFalloffProc;
  tsxLightGetShadType:            TtsxLightGetShadTypeProc;
  tsxLightSetShadType:            TtsxLightSetShadTypeProc;
  tsxLightIsShadImgDep:           TtsxLightIsShadImgDepProc;
  tsxLightSetShadImgDep:          TtsxLightSetShadImgDepProc;
  tsxLightGetShmapSize:           TtsxLightGetShmapSizeProc;
  tsxLightSetShmapSize:           TtsxLightSetShmapSizeProc;
  tsxLightGetShmapSharpness:      TtsxLightGetShmapSharpnessProc;
  tsxLightSetShmapSharpness:      TtsxLightSetShmapSharpnessProc;
  tsxLightGetSpotAngle:           TtsxLightGetSpotAngleProc;
  tsxLightSetSpotAngle:           TtsxLightSetSpotAngleProc;
  tsxLightGetSpotRatio:           TtsxLightGetSpotRatioProc;
  tsxLightSetSpotRatio:           TtsxLightSetSpotRatioProc;
  tsxGroupNewWithCurrobj:         TtsxGroupNewWithCurrobjProc;
  tsxGroupAtCurrobj:              TtsxGroupAtCurrobjProc;
  tsxGroupRemoveCurrobj:          TtsxGroupRemoveCurrobjProc;
  tsxPolyhCreate:                 TtsxPolyhCreateProc;
  tsxPolyhClear:                  TtsxPolyhClearProc;
  tsxPolyhTessellate:             TtsxPolyhTessellateProc;
  tsxPolyhMarkGeomChange:         TtsxPolyhMarkGeomChangeProc;
  tsxPolyhMarkTopologyChange:     TtsxPolyhMarkTopologyChangeProc;
  tsxPolyhMarkMaterialChange:     TtsxPolyhMarkMaterialChangeProc;
  tsxPolyhGetNbrVxs:              TtsxPolyhGetNbrVxsProc;
  tsxPolyhGetVxArray:             TtsxPolyhGetVxArrayProc;
  tsxPolyhGetVxTxmx:              TtsxPolyhGetVxTxmxProc;
  tsxPolyhSetVxs:                 TtsxPolyhSetVxsProc;
  tsxPolyhSetVxTxmx:              TtsxPolyhSetVxTxmxProc;
  tsxFaceAlloc:                   TtsxFaceAllocProc;
  tsxFaceDelete:                  TtsxFaceDeleteProc;
  tsxFaceCopy:                    TtsxFaceCopyProc;
  tsxPolyhGetNbrFaces:            TtsxPolyhGetNbrFacesProc;
  tsxPolyhGetFaceptrArray:        TtsxPolyhGetFaceptrArrayProc;
  tsxPolyhSetFaces:               TtsxPolyhSetFacesProc;
  tsxPolyhGetFaceNormal:          TtsxPolyhGetFaceNormalProc;
  tsxPolyhComputeFaceAdjacency:   TtsxPolyhComputeFaceAdjacencyProc;
  tsxPolyhGetNbrUV:               TtsxPolyhGetNbrUVProc;
  tsxPolyhGetUVArray:             TtsxPolyhGetUVArrayProc;
  tsxPolyhSetUVArray:             TtsxPolyhSetUVArrayProc;
  tsxPolyhInitializeMaterials:    TtsxPolyhInitializeMaterialsProc;
  tsxPolyhPaint:                  TtsxPolyhPaintProc;
  tsxPolyhPaintFace:              TtsxPolyhPaintFaceProc;
  tsxPolyhPaintVertex:            TtsxPolyhPaintVertexProc;
  tsxPolyhGetMaterial:            TtsxPolyhGetMaterialProc;
  tsxPolyhPhrenderFace:           TtsxPolyhPhrenderFaceProc;
  tsxCreateSphere:                TtsxCreateSphereProc;
  tsxCreateCylinder:              TtsxCreateCylinderProc;
  tsxCreateCone:                  TtsxCreateConeProc;
  tsxCreateCube:                  TtsxCreateCubeProc;
  tsxCreateTorus:                 TtsxCreateTorusProc;
  tsxCreatePlane:                 TtsxCreatePlaneProc;
  tsxGNodeCreateScript:           TtsxGNodeCreateScriptProc;
  tsxGNodeCreateScriptTree:       TtsxGNodeCreateScriptTreeProc;
  tsxGNodeDeleteScript:           TtsxGNodeDeleteScriptProc;
  tsxGNodeDeleteScriptTree:       TtsxGNodeDeleteScriptTreeProc;
  tsxGNodeCopyScript:             TtsxGNodeCopyScriptProc;
  tsxGNodeSetFrame:               TtsxGNodeSetFrameProc;
  tsxGNodeUnsetFrame:             TtsxGNodeUnsetFrameProc;
  tsxAnimSetActiveFrame:          TtsxAnimSetActiveFrameProc;
  tsxMtoolInstall:                TtsxMtoolInstallProc;
  tsxMtoolRemove:                 TtsxMtoolRemoveProc;
  tsxAViewRefresh:                TtsxAViewRefreshProc;
  tsxAViewGetRenderMode:          TtsxAViewGetRenderModeProc;
  tsxAViewGetViewMode:            TtsxAViewGetViewModeProc;
  tsxAViewSetViewMode:            TtsxAViewSetViewModeProc;
  tsxAViewSetCameraMode:          TtsxAViewSetCameraModeProc;
  tsxAViewUnsetCameraMode:        TtsxAViewUnsetCameraModeProc;
  tsxAViewGetCameraObj:           TtsxAViewGetCameraObjProc;
  tsxAViewGetPosition:            TtsxAViewGetPositionProc;
  tsxAViewGetDirection:           TtsxAViewGetDirectionProc;
  tsxAViewGetBankAngle:           TtsxAViewGetBankAngleProc;
  tsxAViewGetZoom:                TtsxAViewGetZoomProc;
  tsxAViewSetView:                TtsxAViewSetViewProc;
  tsxAViewSetBank:                TtsxAViewSetBankProc;
  tsxAViewSetZoom:                TtsxAViewSetZoomProc;
  tsxAViewGetWorld2VportTxmx:     TtsxAViewGetWorld2VportTxmxProc;
  tsxAViewGetWorld2EyeTxmx:       TtsxAViewGetWorld2EyeTxmxProc;
  tsxAViewGetEye2WorldTxmx:       TtsxAViewGetEye2WorldTxmxProc;
  tsxAViewGetMouseRay:            TtsxAViewGetMouseRayProc;
  tsxAViewPickRoot:               TtsxAViewPickRootProc;
  tsxAViewPickVertex:             TtsxAViewPickVertexProc;
  tsxAViewPickFace:               TtsxAViewPickFaceProc;
  tsxAViewGetHwnd:                TtsxAViewGetHwndProc;
//New in 3.1 API
  tsxIsSobj:                      TtsxIsSobjProc;
  tsxSobjGetFirstChild:           TtsxSobjGetFirstChildProc;
  tsxSobjGetLastChild:            TtsxSobjGetLastChildProc;
  tsxSobjGetNext:                 TtsxSobjGetNextProc;
  tsxSobjGetPrev:                 TtsxSobjGetPrevProc;
  tsxSobjCreateScript:            TtsxSobjCreateScriptProc;
  tsxSobjTreeCreateScript:        TtsxSobjTreeCreateScriptProc;
  tsxSobjDeleteScript:            TtsxSobjDeleteScriptProc;
  tsxSobjTreeDeleteScript:        TtsxSobjTreeDeleteScriptProc;
  tsxSobjCopyScript:              TtsxSobjCopyScriptProc;
  tsxSobjEmptyScript:             TtsxSobjEmptyScriptProc;
  tsxSobjTreeEmptyScript:         TtsxSobjTreeEmptyScriptProc;
  tsxAnimSetActiveTime:           TtsxAnimSetActiveTimeProc;
  tsxAnimGetActiveTime:           TtsxAnimGetActiveTimeProc;
  tsxActiveTimeChangedCB:         TtsxActiveTimeChangedCBProc;
  tsxAnimSetPlayRanges:           TtsxAnimSetPlayRangesProc;
  tsxAnimSetPlayMode:             TtsxAnimSetPlayModeProc;
  tsxAnimPlay:                    TtsxAnimPlayProc;
  tsxAnimGetBaseFramerate:        TtsxAnimGetBaseFramerateProc;
  tsxSobjSetFrame:                TtsxSobjSetFrameProc;
  tsxSobjUnsetFrame:              TtsxSobjUnsetFrameProc;
  tsxSobjGetFirstTime:            TtsxSobjGetFirstTimeProc;
  tsxSobjGetLastTime:             TtsxSobjGetLastTimeProc;
  tsxSobjGetNextTime:             TtsxSobjGetNextTimeProc;
  tsxSobjGetPrevTime:             TtsxSobjGetPrevTimeProc;
  tsxSobjTreeGetNextTime:         TtsxSobjTreeGetNextTimeProc;
  tsxSobjTreeGetPrevTime:         TtsxSobjTreeGetPrevTimeProc;
  tsxSobjGetMinMaxTime:           TtsxSobjGetMinMaxTimeProc;
  tsxSobjTreeGetMinMaxTime:       TtsxSobjTreeGetMinMaxTimeProc;
  tsxFrameCreate:                 TtsxFrameCreateProc;
  tsxFrameDelete:                 TtsxFrameDeleteProc;
  tsxFrameGetType:                TtsxFrameGetTypeProc;
  tsxAnimGetTypeName:             TtsxAnimGetTypeNameProc;
  tsxFrameCopy:                   TtsxFrameCopyProc;
  tsxSobjInsertFrame:             TtsxSobjInsertFrameProc;
  tsxFrameRemove:                 TtsxFrameRemoveProc;
  tsxSobjGetFirstFrame:           TtsxSobjGetFirstFrameProc;
  tsxSobjGetLastFrame:            TtsxSobjGetLastFrameProc;
  tsxSobjGetFrame:                TtsxSobjGetFrameProc;
  tsxSobjGetLeftFrame:            TtsxSobjGetLeftFrameProc;
  tsxFrameGetNextFrame:           TtsxFrameGetNextFrameProc;
  tsxFrameGetPrevFrame:           TtsxFrameGetPrevFrameProc;
  tsxFrameGetTime:                TtsxFrameGetTimeProc;
  tsxFrameGetSplineParameters:    TtsxFrameGetSplineParametersProc;
  tsxFrameSetSplineParameters:    TtsxFrameSetSplineParametersProc;
  tsxFrameGetDataLength:          TtsxFrameGetDataLengthProc;
  tsxFrameGetDataDescription:     TtsxFrameGetDataDescriptionProc;
  tsxFrameGetData:                TtsxFrameGetDataProc;
  tsxFrameSetData:                TtsxFrameSetDataProc;
  tsxSobjGetInterpolatedData:     TtsxSobjGetInterpolatedDataProc;
  tsxAnimCreateAttribute:         TtsxAnimCreateAttributeProc;
  tsxSobjAddAttribute:            TtsxSobjAddAttributeProc;
  tsxSobjGetFirstAttribute:       TtsxSobjGetFirstAttributeProc;
  tsxSobjGetNextAttribute:        TtsxSobjGetNextAttributeProc;
  tsxSobjHasAttribute:            TtsxSobjHasAttributeProc;
  tsxSobjGetFirstSegment:         TtsxSobjGetFirstSegmentProc;
  tsxSobjGetNextSegment:          TtsxSobjGetNextSegmentProc;
  tsxSobjGetAttrDataLength:       TtsxSobjGetAttrDataLengthProc;
  tsxSobjGetAttrData:             TtsxSobjGetAttrDataProc;
  tsxSobjSetAttrData:             TtsxSobjSetAttrDataProc;
  tsxPolyhMarkVertexChange:       TtsxPolyhMarkVertexChangeProc;
  tsxMaterialSetEnvrFilename:     TtsxMaterialSetEnvrFilenameProc;
  tsxGNodeGetLocation:            TtsxGNodeGetLocationProc;
  tsxGNodeGetRotation:            TtsxGNodeGetRotationProc;
  tsxGNodeGetSize:                TtsxGNodeGetSizeProc;
  tsxGNodeSetLocation:            TtsxGNodeSetLocationProc;
  tsxGNodeSetRotation:            TtsxGNodeSetRotationProc;
  tsxGNodeSetSize:                TtsxGNodeSetSizeProc;
  tsxGNodeGetTransform:           TtsxGNodeGetTransformProc;
  tsxMaterialGetEnvrFilename:     TtsxMaterialGetEnvrFilenameProc;
  tsxAnimGetPlayRanges:           TtsxAnimGetPlayRangesProc;
  tsxAnimGetPlayMode:             TtsxAnimGetPlayModeProc;
//New in 3.2 API
  tsxSobjAddChild:                TtsxSobjAddChildProc;
  tsxGNodeGetUnits:               TtsxGNodeGetUnitsProc;
  tsxGNodeSetUnits:               TtsxGNodeSetUnitsProc;
  tsxAViewSetPen:                 TtsxAViewSetPenProc;
  tsxAViewDrawLinelist:           TtsxAViewDrawLinelistProc;
  tsxAllViewsRefresh:             TtsxAllViewsRefreshProc;
  tsxAViewGetScreenPoint:         TtsxAViewGetScreenPointProc;
  tsxObjectDefine:                TtsxObjectDefineProc;
  tsxObjectCreate:                TtsxObjectCreateProc;
  tsxObjectGetDataPtr:            TtsxObjectGetDataPtrProc;
  tsxObjectGetTypeName:           TtsxObjectGetTypeNameProc;
  tsxDataDefine:                  TtsxDataDefineProc;
  tsxSobjCreateData:              TtsxSobjCreateDataProc;
  tsxSobjGetData:                 TtsxSobjGetDataProc;
  tsxSobjDeleteData:              TtsxSobjDeleteDataProc;
  tsxFILEerror:                   TtsxFILEerrorProc;
  tsxFILEgetc:                    TtsxFILEgetcProc;
  tsxFILEgets:                    TtsxFILEgetsProc;
  tsxFILEprintf:                  TtsxFILEprintfProc;
  tsxFILEputc:                    TtsxFILEputcProc;
  tsxFILEputs:                    TtsxFILEputsProc;
  tsxFILEread:                    TtsxFILEreadProc;
  tsxFILEscanf:                   TtsxFILEscanfProc;
  tsxFILEseek:                    TtsxFILEseekProc;
  tsxFILEtell:                    TtsxFILEtellProc;
  tsxFILEwrite:                   TtsxFILEwriteProc;

{API Initialization}
function tsxapiInitialize: TtSVersion;
procedure tsxapiUninitialize;

implementation

uses dialogs;

const
  tsxapi = 'tsxapi.dll';

var
  tsxapiHandle : HINST;

function GetImportAddress(hModule: HMODULE; lpProcName: LPCSTR): FARPROC;
begin
  result := windows.GetProcAddress(hModule, lpProcName);
  if not Assigned(result) then
    raise EtsxFunctionNotFound.Create('A necessary API function(' + lpProcName + ')' + #13#10 + 'could not be found in ' + tsxapi);
end;

function tsxVectorsEqual3f(const vec1, vec2: PCtsxVector3f): Boolean;
begin
  result := ((vec1.x = vec2.x) AND (vec1.y = vec2.y) AND (vec1.z = vec2.z));
end;

function tsxVectorsEqual2f(const vec1, vec2: PCtsxVector2f): Boolean;
begin
  result := ((vec1.x = vec2.x) AND (vec1.y = vec2.y));
end;

function tsxSUCCEEDED(err: tsxERR): Boolean;
begin
  result := (err >= 0);
end;

function tsxFAILED(err: tsxERR): Boolean;
begin
  result := (err < 0);
end;

function tsxRad2Deg(Const rad: Single): Single;
begin
  result := (rad * tsxRAD2DEG_FAC);
end;

function tsxDeg2Rad(Const deg: Single): Single;
begin
  result := (deg * tsxDEG2RAD_FAC);
end;

procedure InitError (ErrorCode : Integer);
begin
  raise Exception.CreateFmt('Error %d initializing tSX API in Plugin: %s', [ErrorCode,
    SysErrorMessage(ErrorCode)]);
end;

function tsxapiLoad30Procs (Handle : HINST): Boolean;
begin
  // Attempt to load all version 3.0 API functions
  try
    @tsxGetTsVersionNbrs          :=GetImportAddress(Handle, 'tsxGetTsVersionNbrs');
    @tsxGetMainViewHwnd           :=GetImportAddress(Handle, 'tsxGetMainViewHwnd');
    @tsxForceLeftClick            :=GetImportAddress(Handle, 'tsxForceLeftClick');
    @tsxMalloc                    :=GetImportAddress(Handle, 'tsxMalloc');
    @tsxCalloc                    :=GetImportAddress(Handle, 'tsxCalloc');
    @tsxRealloc                   :=GetImportAddress(Handle, 'tsxRealloc');
    @tsxFree                      :=GetImportAddress(Handle, 'tsxFree');
    @tsxGetScene                  :=GetImportAddress(Handle, 'tsxGetScene');
    @tsxSceneAddObject            :=GetImportAddress(Handle, 'tsxSceneAddObject');
    @tsxSceneGetFirstNode         :=GetImportAddress(Handle, 'tsxSceneGetFirstNode');
    @tsxSceneIsFogOn              :=GetImportAddress(Handle, 'tsxSceneIsFogOn');
    @tsxSceneSwitchFog            :=GetImportAddress(Handle, 'tsxSceneSwitchFog');
    @tsxSceneGetFogParms          :=GetImportAddress(Handle, 'tsxSceneGetFogParms');
    @tsxSceneSetFogParms          :=GetImportAddress(Handle, 'tsxSceneSetFogParms');
    @tsxSceneDraw                 :=GetImportAddress(Handle, 'tsxSceneDraw');
    @tsxSobjGetType               :=GetImportAddress(Handle, 'tsxSobjGetType');
    @tsxSobjDelete                :=GetImportAddress(Handle, 'tsxSobjDelete');
    @tsxIsMNode                   :=GetImportAddress(Handle, 'tsxIsMNode');
    @tsxMNodeIsVisible            :=GetImportAddress(Handle, 'tsxMNodeIsVisible');
    @tsxMNodeSetVisible           :=GetImportAddress(Handle, 'tsxMNodeSetVisible');
    @tsxMNodeSetInvisible         :=GetImportAddress(Handle, 'tsxMNodeSetInvisible');
    @tsxMNodeAreAxesVisible       :=GetImportAddress(Handle, 'tsxMNodeAreAxesVisible');
    @tsxCurrobjToggleAxes         :=GetImportAddress(Handle, 'tsxCurrobjToggleAxes');
    @tsxMNodeSetAxesPosition      :=GetImportAddress(Handle, 'tsxMNodeSetAxesPosition');
    @tsxMNodeSetAxesOrientation   :=GetImportAddress(Handle, 'tsxMNodeSetAxesOrientation');
    @tsxIsGNode                   :=GetImportAddress(Handle, 'tsxIsGNode');
    @tsxGNodeCopy                 :=GetImportAddress(Handle, 'tsxGNodeCopy');
    @tsxGNodeGetRoot              :=GetImportAddress(Handle, 'tsxGNodeGetRoot');
    @tsxGNodeGetParent            :=GetImportAddress(Handle, 'tsxGNodeGetParent');
    @tsxIsGNodeInScene            :=GetImportAddress(Handle, 'tsxIsGNodeInScene');
    @tsxGNodeGetFirstChild        :=GetImportAddress(Handle, 'tsxGNodeGetFirstChild');
    @tsxGNodeGetNext              :=GetImportAddress(Handle, 'tsxGNodeGetNext');
    @tsxGNodeGetPrev              :=GetImportAddress(Handle, 'tsxGNodeGetPrev');
    @tsxIsGNodeSubobj             :=GetImportAddress(Handle, 'tsxIsGNodeSubobj');
    @tsxGNodeDraw                 :=GetImportAddress(Handle, 'tsxGNodeDraw');
    @tsxGNodeGetName              :=GetImportAddress(Handle, 'tsxGNodeGetName');
    @tsxGNodeSetName              :=GetImportAddress(Handle, 'tsxGNodeSetName');
    @tsxGNodeGetAxesPosition      :=GetImportAddress(Handle, 'tsxGNodeGetAxesPosition');
    @tsxGNodeGetAxesOrientation   :=GetImportAddress(Handle, 'tsxGNodeGetAxesOrientation');
    @tsxGNodeGetBBox              :=GetImportAddress(Handle, 'tsxGNodeGetBBox');
    @tsxGNodeGetPureTransform     :=GetImportAddress(Handle, 'tsxGNodeGetPureTransform');
    @tsxGNodeTranslate            :=GetImportAddress(Handle, 'tsxGNodeTranslate');
    @tsxGNodeRotate               :=GetImportAddress(Handle, 'tsxGNodeRotate');
    @tsxGNodeScale                :=GetImportAddress(Handle, 'tsxGNodeScale');
    @tsxGNodeScaleUniform         :=GetImportAddress(Handle, 'tsxGNodeScaleUniform');
    @tsxMagnitude3f               :=GetImportAddress(Handle, 'tsxMagnitude3f');
    @tsxMagnitude2f               :=GetImportAddress(Handle, 'tsxMagnitude2f');
    @tsxNormalize3f               :=GetImportAddress(Handle, 'tsxNormalize3f');
    @tsxNormalize2f               :=GetImportAddress(Handle, 'tsxNormalize2f');
    @tsxGetNormalized3f           :=GetImportAddress(Handle, 'tsxGetNormalized3f');
    @tsxGetNormalized2f           :=GetImportAddress(Handle, 'tsxGetNormalized2f');
    @tsxAddScalar3f               :=GetImportAddress(Handle, 'tsxAddScalar3f');
    @tsxMulScalar3f               :=GetImportAddress(Handle, 'tsxMulScalar3f');
    @tsxAddVec3f                  :=GetImportAddress(Handle, 'tsxAddVec3f');
    @tsxSubVec3f                  :=GetImportAddress(Handle, 'tsxSubVec3f');
    @tsxDProdVec3f                :=GetImportAddress(Handle, 'tsxDProdVec3f');
    @tsxXProdVec3f                :=GetImportAddress(Handle, 'tsxXProdVec3f');
    @tsxAngleBtwVecs3f            :=GetImportAddress(Handle, 'tsxAngleBtwVecs3f');
    @tsxDistanceToPoint3f         :=GetImportAddress(Handle, 'tsxDistanceToPoint3f');
    @tsxDistanceToLine3f          :=GetImportAddress(Handle, 'tsxDistanceToLine3f');
    @tsxDistanceToPlane3f         :=GetImportAddress(Handle, 'tsxDistanceToPlane3f');
    @tsxXProdMx3f                 :=GetImportAddress(Handle, 'tsxXProdMx3f');
    @tsxInvertMx3f                :=GetImportAddress(Handle, 'tsxInvertMx3f');
    @tsxMakeMxFromXZO3f           :=GetImportAddress(Handle, 'tsxMakeMxFromXZO3f');
    @tsxMakeRotMx3f               :=GetImportAddress(Handle, 'tsxMakeRotMx3f');
    @tsxMakePureRotMx3f           :=GetImportAddress(Handle, 'tsxMakePureRotMx3f');
    @tsxGetMxTranslation3f        :=GetImportAddress(Handle, 'tsxGetMxTranslation3f');
    @tsxSetMxTranslation3f        :=GetImportAddress(Handle, 'tsxSetMxTranslation3f');
    @tsxAddMxTranslation3f        :=GetImportAddress(Handle, 'tsxAddMxTranslation3f');
    @tsxSubMxTranslation3f        :=GetImportAddress(Handle, 'tsxSubMxTranslation3f');
    @tsxUnMatrix3f                :=GetImportAddress(Handle, 'tsxUnMatrix3f');
    @tsxTransformVecs3f           :=GetImportAddress(Handle, 'tsxTransformVecs3f');
    @tsxRotateVecs3f              :=GetImportAddress(Handle, 'tsxRotateVecs3f');
    @tsxRotateAxes3f              :=GetImportAddress(Handle, 'tsxRotateAxes3f');
    @tsxBBoxInit                  :=GetImportAddress(Handle, 'tsxBBoxInit');
    @tsxBBoxIntersection          :=GetImportAddress(Handle, 'tsxBBoxIntersection');
    @tsxIsSelectable              :=GetImportAddress(Handle, 'tsxIsSelectable');
    @tsxIsSelected                :=GetImportAddress(Handle, 'tsxIsSelected');
    @tsxGetCurrentSelection       :=GetImportAddress(Handle, 'tsxGetCurrentSelection');
    @tsxSelectSobj                :=GetImportAddress(Handle, 'tsxSelectSobj');
    @tsxSelectDown                :=GetImportAddress(Handle, 'tsxSelectDown');
    @tsxSelectUp                  :=GetImportAddress(Handle, 'tsxSelectUp');
    @tsxSelectNext                :=GetImportAddress(Handle, 'tsxSelectNext');
    @tsxSelectPrev                :=GetImportAddress(Handle, 'tsxSelectPrev');
    @tsxSelectionChangedCB        :=GetImportAddress(Handle, 'tsxSelectionChangedCB');
    @tsxCurrobjReplace            :=GetImportAddress(Handle, 'tsxCurrobjReplace');
    @tsxCurrobjCopy               :=GetImportAddress(Handle, 'tsxCurrobjCopy');
    @tsxCurrobjDraw               :=GetImportAddress(Handle, 'tsxCurrobjDraw');
    @tsxCurrobjPhrender           :=GetImportAddress(Handle, 'tsxCurrobjPhrender');
    @tsxMaterialCreate            :=GetImportAddress(Handle, 'tsxMaterialCreate');
    @tsxMaterialCreateCopy        :=GetImportAddress(Handle, 'tsxMaterialCreateCopy');
    @tsxMaterialCreateCopyActive  :=GetImportAddress(Handle, 'tsxMaterialCreateCopyActive');
    @tsxMaterialDestroy           :=GetImportAddress(Handle, 'tsxMaterialDestroy');
    @tsxMaterialSetActive         :=GetImportAddress(Handle, 'tsxMaterialSetActive');
    @tsxMaterialGetActive         :=GetImportAddress(Handle, 'tsxMaterialGetActive');
    @tsxMaterialCopyActive        :=GetImportAddress(Handle, 'tsxMaterialCopyActive');
    @tsxMaterialsAreEqual         :=GetImportAddress(Handle, 'tsxMaterialsAreEqual');
    @tsxMaterialGetId             :=GetImportAddress(Handle, 'tsxMaterialGetId');
    @tsxMaterialGetShadingType    :=GetImportAddress(Handle, 'tsxMaterialGetShadingType');
    @tsxMaterialGetFacetingType   :=GetImportAddress(Handle, 'tsxMaterialGetFacetingType');
    @tsxMaterialGetAutofacetAngle :=GetImportAddress(Handle, 'tsxMaterialGetAutofacetAngle');
    @tsxMaterialGetColor          :=GetImportAddress(Handle, 'tsxMaterialGetColor');
    @tsxMaterialGetSurfaceProps   :=GetImportAddress(Handle, 'tsxMaterialGetSurfaceProps');
    @tsxMaterialGetIor            :=GetImportAddress(Handle, 'tsxMaterialGetIor');
    @tsxMaterialGetTextureType    :=GetImportAddress(Handle, 'tsxMaterialGetTextureType');
    @tsxMaterialGetProcTexture    :=GetImportAddress(Handle, 'tsxMaterialGetProcTexture');
    @tsxMaterialGetTextureFilename:=GetImportAddress(Handle, 'tsxMaterialGetTextureFilename');
    @tsxMaterialGetTextureProps   :=GetImportAddress(Handle, 'tsxMaterialGetTextureProps');
    @tsxMaterialGetBumpFilename   :=GetImportAddress(Handle, 'tsxMaterialGetBumpFilename');
    @tsxMaterialGetBumpProps      :=GetImportAddress(Handle, 'tsxMaterialGetBumpProps');
    @tsxMaterialGetEnvrFlags      :=GetImportAddress(Handle, 'tsxMaterialGetEnvrFlags');
    @tsxMaterialSetShadingType    :=GetImportAddress(Handle, 'tsxMaterialSetShadingType');
    @tsxMaterialSetFacetingType   :=GetImportAddress(Handle, 'tsxMaterialSetFacetingType');
    @tsxMaterialSetAutofacetAngle :=GetImportAddress(Handle, 'tsxMaterialSetAutofacetAngle');
    @tsxMaterialSetColor          :=GetImportAddress(Handle, 'tsxMaterialSetColor');
    @tsxMaterialSetSurfaceProps   :=GetImportAddress(Handle, 'tsxMaterialSetSurfaceProps');
    @tsxMaterialSetIor            :=GetImportAddress(Handle, 'tsxMaterialSetIor');
    @tsxMaterialSetTextureType    :=GetImportAddress(Handle, 'tsxMaterialSetTextureType');
    @tsxMaterialSetProcTexture    :=GetImportAddress(Handle, 'tsxMaterialSetProcTexture');
    @tsxMaterialSetTextureFilename:=GetImportAddress(Handle, 'tsxMaterialSetTextureFilename');
    @tsxMaterialSetTextureProps   :=GetImportAddress(Handle, 'tsxMaterialSetTextureProps');
    @tsxMaterialSetBumpFilename   :=GetImportAddress(Handle, 'tsxMaterialSetBumpFilename');
    @tsxMaterialSetBumpProps      :=GetImportAddress(Handle, 'tsxMaterialSetBumpProps');
    @tsxMaterialSetEnvrFlags      :=GetImportAddress(Handle, 'tsxMaterialSetEnvrFlags');
    @tsxCameraCreateInScene       :=GetImportAddress(Handle, 'tsxCameraCreateInScene');
    @tsxLightCreateInScene        :=GetImportAddress(Handle, 'tsxLightCreateInScene');
    @tsxLightGetType              :=GetImportAddress(Handle, 'tsxLightGetType');
    @tsxLightGetColor             :=GetImportAddress(Handle, 'tsxLightGetColor');
    @tsxLightSetColor             :=GetImportAddress(Handle, 'tsxLightSetColor');
    @tsxLightGetIntensity         :=GetImportAddress(Handle, 'tsxLightGetIntensity');
    @tsxLightSetIntensity         :=GetImportAddress(Handle, 'tsxLightSetIntensity');
    @tsxLightGetFalloff           :=GetImportAddress(Handle, 'tsxLightGetFalloff');
    @tsxLightSetFalloff           :=GetImportAddress(Handle, 'tsxLightSetFalloff');
    @tsxLightGetShadType          :=GetImportAddress(Handle, 'tsxLightGetShadType');
    @tsxLightSetShadType          :=GetImportAddress(Handle, 'tsxLightSetShadType');
    @tsxLightIsShadImgDep         :=GetImportAddress(Handle, 'tsxLightIsShadImgDep');
    @tsxLightSetShadImgDep        :=GetImportAddress(Handle, 'tsxLightSetShadImgDep');
    @tsxLightGetShmapSize         :=GetImportAddress(Handle, 'tsxLightGetShmapSize');
    @tsxLightSetShmapSize         :=GetImportAddress(Handle, 'tsxLightSetShmapSize');
    @tsxLightGetShmapSharpness    :=GetImportAddress(Handle, 'tsxLightGetShmapSharpness');
    @tsxLightSetShmapSharpness    :=GetImportAddress(Handle, 'tsxLightSetShmapSharpness');
    @tsxLightGetSpotAngle         :=GetImportAddress(Handle, 'tsxLightGetSpotAngle');
    @tsxLightSetSpotAngle         :=GetImportAddress(Handle, 'tsxLightSetSpotAngle');
    @tsxLightGetSpotRatio         :=GetImportAddress(Handle, 'tsxLightGetSpotRatio');
    @tsxLightSetSpotRatio         :=GetImportAddress(Handle, 'tsxLightSetSpotRatio');
    @tsxGroupNewWithCurrobj       :=GetImportAddress(Handle, 'tsxGroupNewWithCurrobj');
    @tsxGroupAtCurrobj            :=GetImportAddress(Handle, 'tsxGroupAtCurrobj');
    @tsxGroupRemoveCurrobj        :=GetImportAddress(Handle, 'tsxGroupRemoveCurrobj');
    @tsxPolyhCreate               :=GetImportAddress(Handle, 'tsxPolyhCreate');
    @tsxPolyhClear                :=GetImportAddress(Handle, 'tsxPolyhClear');
    @tsxPolyhTessellate           :=GetImportAddress(Handle, 'tsxPolyhTessellate');
    @tsxPolyhMarkGeomChange       :=GetImportAddress(Handle, 'tsxPolyhMarkGeomChange');
    @tsxPolyhMarkTopologyChange   :=GetImportAddress(Handle, 'tsxPolyhMarkTopologyChange');
    @tsxPolyhMarkMaterialChange   :=GetImportAddress(Handle, 'tsxPolyhMarkMaterialChange');
    @tsxPolyhGetNbrVxs            :=GetImportAddress(Handle, 'tsxPolyhGetNbrVxs');
    @tsxPolyhGetVxArray           :=GetImportAddress(Handle, 'tsxPolyhGetVxArray');
    @tsxPolyhGetVxTxmx            :=GetImportAddress(Handle, 'tsxPolyhGetVxTxmx');
    @tsxPolyhSetVxs               :=GetImportAddress(Handle, 'tsxPolyhSetVxs');
    @tsxPolyhSetVxTxmx            :=GetImportAddress(Handle, 'tsxPolyhSetVxTxmx');
    @tsxFaceAlloc                 :=GetImportAddress(Handle, 'tsxFaceAlloc');
    @tsxFaceDelete                :=GetImportAddress(Handle, 'tsxFaceDelete');
    @tsxFaceCopy                  :=GetImportAddress(Handle, 'tsxFaceCopy');
    @tsxPolyhGetNbrFaces          :=GetImportAddress(Handle, 'tsxPolyhGetNbrFaces');
    @tsxPolyhGetFaceptrArray      :=GetImportAddress(Handle, 'tsxPolyhGetFaceptrArray');
    @tsxPolyhSetFaces             :=GetImportAddress(Handle, 'tsxPolyhSetFaces');
    @tsxPolyhGetFaceNormal        :=GetImportAddress(Handle, 'tsxPolyhGetFaceNormal');
    @tsxPolyhComputeFaceAdjacency :=GetImportAddress(Handle, 'tsxPolyhComputeFaceAdjacency');
    @tsxPolyhGetNbrUV             :=GetImportAddress(Handle, 'tsxPolyhGetNbrUV');
    @tsxPolyhGetUVArray           :=GetImportAddress(Handle, 'tsxPolyhGetUVArray');
    @tsxPolyhSetUVArray           :=GetImportAddress(Handle, 'tsxPolyhSetUVArray');
    @tsxPolyhInitializeMaterials  :=GetImportAddress(Handle, 'tsxPolyhInitializeMaterials');
    @tsxPolyhPaint                :=GetImportAddress(Handle, 'tsxPolyhPaint');
    @tsxPolyhPaintFace            :=GetImportAddress(Handle, 'tsxPolyhPaintFace');
    @tsxPolyhPaintVertex          :=GetImportAddress(Handle, 'tsxPolyhPaintVertex');
    @tsxPolyhGetMaterial          :=GetImportAddress(Handle, 'tsxPolyhGetMaterial');
    @tsxPolyhPhrenderFace         :=GetImportAddress(Handle, 'tsxPolyhPhrenderFace');
    @tsxCreateSphere              :=GetImportAddress(Handle, 'tsxCreateSphere');
    @tsxCreateCylinder            :=GetImportAddress(Handle, 'tsxCreateCylinder');
    @tsxCreateCone                :=GetImportAddress(Handle, 'tsxCreateCone');
    @tsxCreateCube                :=GetImportAddress(Handle, 'tsxCreateCube');
    @tsxCreateTorus               :=GetImportAddress(Handle, 'tsxCreateTorus');
    @tsxCreatePlane               :=GetImportAddress(Handle, 'tsxCreatePlane');
    @tsxGNodeCreateScript         :=GetImportAddress(Handle, 'tsxGNodeCreateScript');
    @tsxGNodeCreateScriptTree     :=GetImportAddress(Handle, 'tsxGNodeCreateScriptTree');
    @tsxGNodeDeleteScript         :=GetImportAddress(Handle, 'tsxGNodeDeleteScript');
    @tsxGNodeDeleteScriptTree     :=GetImportAddress(Handle, 'tsxGNodeDeleteScriptTree');
    @tsxGNodeCopyScript           :=GetImportAddress(Handle, 'tsxGNodeCopyScript');
    @tsxGNodeSetFrame             :=GetImportAddress(Handle, 'tsxGNodeSetFrame');
    @tsxGNodeUnsetFrame           :=GetImportAddress(Handle, 'tsxGNodeUnsetFrame');
    @tsxAnimSetActiveFrame        :=GetImportAddress(Handle, 'tsxAnimSetActiveFrame');
    @tsxMtoolInstall              :=GetImportAddress(Handle, 'tsxMtoolInstall');
    @tsxMtoolRemove               :=GetImportAddress(Handle, 'tsxMtoolRemove');
    @tsxAViewRefresh              :=GetImportAddress(Handle, 'tsxAViewRefresh');
    @tsxAViewGetRenderMode        :=GetImportAddress(Handle, 'tsxAViewGetRenderMode');
    @tsxAViewGetViewMode          :=GetImportAddress(Handle, 'tsxAViewGetViewMode');
    @tsxAViewSetViewMode          :=GetImportAddress(Handle, 'tsxAViewSetViewMode');
    @tsxAViewSetCameraMode        :=GetImportAddress(Handle, 'tsxAViewSetCameraMode');
    @tsxAViewUnsetCameraMode      :=GetImportAddress(Handle, 'tsxAViewUnsetCameraMode');
    @tsxAViewGetCameraObj         :=GetImportAddress(Handle, 'tsxAViewGetCameraObj');
    @tsxAViewGetPosition          :=GetImportAddress(Handle, 'tsxAViewGetPosition');
    @tsxAViewGetDirection         :=GetImportAddress(Handle, 'tsxAViewGetDirection');
    @tsxAViewGetBankAngle         :=GetImportAddress(Handle, 'tsxAViewGetBankAngle');
    @tsxAViewGetZoom              :=GetImportAddress(Handle, 'tsxAViewGetZoom');
    @tsxAViewSetView              :=GetImportAddress(Handle, 'tsxAViewSetView');
    @tsxAViewSetBank              :=GetImportAddress(Handle, 'tsxAViewSetBank');
    @tsxAViewSetZoom              :=GetImportAddress(Handle, 'tsxAViewSetZoom');
    @tsxAViewGetWorld2VportTxmx   :=GetImportAddress(Handle, 'tsxAViewGetWorld2VportTxmx');
    @tsxAViewGetWorld2EyeTxmx     :=GetImportAddress(Handle, 'tsxAViewGetWorld2EyeTxmx');
    @tsxAViewGetEye2WorldTxmx     :=GetImportAddress(Handle, 'tsxAViewGetEye2WorldTxmx');
    @tsxAViewGetMouseRay          :=GetImportAddress(Handle, 'tsxAViewGetMouseRay');
    @tsxAViewPickRoot             :=GetImportAddress(Handle, 'tsxAViewPickRoot');
    @tsxAViewPickVertex           :=GetImportAddress(Handle, 'tsxAViewPickVertex');
    @tsxAViewPickFace             :=GetImportAddress(Handle, 'tsxAViewPickFace');
    @tsxAViewGetHwnd              :=GetImportAddress(Handle, 'tsxAViewGetHwnd');
    result := True;
  except
    result := False;
  end;
end;

function tsxapiLoad31Procs (Handle : HINST): Boolean;
begin
  // Attempt to load all version 3.1 API functions
  try
    @tsxIsSobj                    :=GetImportAddress(Handle, 'tsxIsSobj');
    @tsxSobjGetFirstChild         :=GetImportAddress(Handle, 'tsxSobjGetFirstChild');
    @tsxSobjGetLastChild          :=GetImportAddress(Handle, 'tsxSobjGetLastChild');
    @tsxSobjGetNext               :=GetImportAddress(Handle, 'tsxSobjGetNext');
    @tsxSobjGetPrev               :=GetImportAddress(Handle, 'tsxSobjGetPrev');
    @tsxSobjCreateScript          :=GetImportAddress(Handle, 'tsxSobjCreateScript');
    @tsxSobjTreeCreateScript      :=GetImportAddress(Handle, 'tsxSobjTreeCreateScript');
    @tsxSobjDeleteScript          :=GetImportAddress(Handle, 'tsxSobjDeleteScript');
    @tsxSobjTreeDeleteScript      :=GetImportAddress(Handle, 'tsxSobjTreeDeleteScript');
    @tsxSobjCopyScript            :=GetImportAddress(Handle, 'tsxSobjCopyScript');
    @tsxSobjEmptyScript           :=GetImportAddress(Handle, 'tsxSobjEmptyScript');
    @tsxSobjTreeEmptyScript       :=GetImportAddress(Handle, 'tsxSobjTreeEmptyScript');
    @tsxAnimSetActiveTime         :=GetImportAddress(Handle, 'tsxAnimSetActiveTime');
    @tsxAnimGetActiveTime         :=GetImportAddress(Handle, 'tsxAnimGetActiveTime');
    @tsxActiveTimeChangedCB       :=GetImportAddress(Handle, 'tsxActiveTimeChangedCB');
    @tsxAnimSetPlayRanges         :=GetImportAddress(Handle, 'tsxAnimSetPlayRanges');
    @tsxAnimSetPlayMode           :=GetImportAddress(Handle, 'tsxAnimSetPlayMode');
    @tsxAnimPlay                  :=GetImportAddress(Handle, 'tsxAnimPlay');
    @tsxAnimGetBaseFramerate      :=GetImportAddress(Handle, 'tsxAnimGetBaseFramerate');
    @tsxSobjSetFrame              :=GetImportAddress(Handle, 'tsxSobjSetFrame');
    @tsxSobjUnsetFrame            :=GetImportAddress(Handle, 'tsxSobjUnsetFrame');
    @tsxSobjGetFirstTime          :=GetImportAddress(Handle, 'tsxSobjGetFirstTime');
    @tsxSobjGetLastTime           :=GetImportAddress(Handle, 'tsxSobjGetLastTime');
    @tsxSobjGetNextTime           :=GetImportAddress(Handle, 'tsxSobjGetNextTime');
    @tsxSobjGetPrevTime           :=GetImportAddress(Handle, 'tsxSobjGetPrevTime');
    @tsxSobjTreeGetNextTime       :=GetImportAddress(Handle, 'tsxSobjTreeGetNextTime');
    @tsxSobjTreeGetPrevTime       :=GetImportAddress(Handle, 'tsxSobjTreeGetPrevTime');
    @tsxSobjGetMinMaxTime         :=GetImportAddress(Handle, 'tsxSobjGetMinMaxTime');
    @tsxSobjTreeGetMinMaxTime     :=GetImportAddress(Handle, 'tsxSobjTreeGetMinMaxTime');
    @tsxFrameCreate               :=GetImportAddress(Handle, 'tsxFrameCreate');
    @tsxFrameDelete               :=GetImportAddress(Handle, 'tsxFrameDelete');
    @tsxFrameGetType              :=GetImportAddress(Handle, 'tsxFrameGetType');
    @tsxAnimGetTypeName           :=GetImportAddress(Handle, 'tsxAnimGetTypeName');
    @tsxFrameCopy                 :=GetImportAddress(Handle, 'tsxFrameCopy');
    @tsxSobjInsertFrame           :=GetImportAddress(Handle, 'tsxSobjInsertFrame');
    @tsxFrameRemove               :=GetImportAddress(Handle, 'tsxFrameRemove');
    @tsxSobjGetFirstFrame         :=GetImportAddress(Handle, 'tsxSobjGetFirstFrame');
    @tsxSobjGetLastFrame          :=GetImportAddress(Handle, 'tsxSobjGetLastFrame');
    @tsxSobjGetFrame              :=GetImportAddress(Handle, 'tsxSobjGetFrame');
    @tsxSobjGetLeftFrame          :=GetImportAddress(Handle, 'tsxSobjGetLeftFrame');
    @tsxFrameGetNextFrame         :=GetImportAddress(Handle, 'tsxFrameGetNextFrame');
    @tsxFrameGetPrevFrame         :=GetImportAddress(Handle, 'tsxFrameGetPrevFrame');
    @tsxFrameGetTime              :=GetImportAddress(Handle, 'tsxFrameGetTime');
    @tsxFrameGetSplineParameters  :=GetImportAddress(Handle, 'tsxFrameGetSplineParameters');
    @tsxFrameSetSplineParameters  :=GetImportAddress(Handle, 'tsxFrameSetSplineParameters');
    @tsxFrameGetDataLength        :=GetImportAddress(Handle, 'tsxFrameGetDataLength');
    @tsxFrameGetDataDescription   :=GetImportAddress(Handle, 'tsxFrameGetDataDescription');
    @tsxFrameGetData              :=GetImportAddress(Handle, 'tsxFrameGetData');
    @tsxFrameSetData              :=GetImportAddress(Handle, 'tsxFrameSetData');
    @tsxSobjGetInterpolatedData   :=GetImportAddress(Handle, 'tsxSobjGetInterpolatedData');
    @tsxAnimCreateAttribute       :=GetImportAddress(Handle, 'tsxAnimCreateAttribute');
    @tsxSobjAddAttribute          :=GetImportAddress(Handle, 'tsxSobjAddAttribute');
    @tsxSobjGetFirstAttribute     :=GetImportAddress(Handle, 'tsxSobjGetFirstAttribute');
    @tsxSobjGetNextAttribute      :=GetImportAddress(Handle, 'tsxSobjGetNextAttribute');
    @tsxSobjHasAttribute          :=GetImportAddress(Handle, 'tsxSobjHasAttribute');
    @tsxSobjGetFirstSegment       :=GetImportAddress(Handle, 'tsxSobjGetFirstSegment');
    @tsxSobjGetNextSegment        :=GetImportAddress(Handle, 'tsxSobjGetNextSegment');
    @tsxSobjGetAttrDataLength     :=GetImportAddress(Handle, 'tsxSobjGetAttrDataLength');
    @tsxSobjGetAttrData           :=GetImportAddress(Handle, 'tsxSobjGetAttrData');
    @tsxSobjSetAttrData           :=GetImportAddress(Handle, 'tsxSobjSetAttrData');
    @tsxPolyhMarkVertexChange     :=GetImportAddress(Handle, 'tsxPolyhMarkVertexChange');
    @tsxMaterialSetEnvrFilename   :=GetImportAddress(Handle, 'tsxMaterialSetEnvrFilename');
    @tsxGNodeGetLocation          :=GetImportAddress(Handle, 'tsxGNodeGetLocation');
    @tsxGNodeGetRotation          :=GetImportAddress(Handle, 'tsxGNodeGetRotation');
    @tsxGNodeGetSize              :=GetImportAddress(Handle, 'tsxGNodeGetSize');
    @tsxGNodeSetLocation          :=GetImportAddress(Handle, 'tsxGNodeSetLocation');
    @tsxGNodeSetRotation          :=GetImportAddress(Handle, 'tsxGNodeSetRotation');
    @tsxGNodeSetSize              :=GetImportAddress(Handle, 'tsxGNodeSetSize');
    @tsxGNodeGetTransform         :=GetImportAddress(Handle, 'tsxGNodeGetTransform');
    @tsxMaterialGetEnvrFilename   :=GetImportAddress(Handle, 'tsxMaterialGetEnvrFilename');
    @tsxAnimGetPlayRanges         :=GetImportAddress(Handle, 'tsxAnimGetPlayRanges');
    @tsxAnimGetPlayMode           :=GetImportAddress(Handle, 'tsxAnimGetPlayMode');
    result := True;
  except
    result := False;
  end;
end;

function tsxapiLoad32Procs (Handle : HINST): Boolean;
begin
  // Attempt to load all version 3.2 API functions
  try
    @tsxSobjAddChild              :=GetImportAddress(Handle, 'tsxSobjAddChild');
    @tsxGNodeGetUnits             :=GetImportAddress(Handle, 'tsxGNodeGetUnits');
    @tsxGNodeSetUnits             :=GetImportAddress(Handle, 'tsxGNodeSetUnits');
    @tsxAViewSetPen               :=GetImportAddress(Handle, 'tsxAViewSetPen');
    @tsxAViewDrawLinelist         :=GetImportAddress(Handle, 'tsxAViewDrawLinelist');
    @tsxAllViewsRefresh           :=GetImportAddress(Handle, 'tsxAllViewsRefresh');
    @tsxAViewGetScreenPoint       :=GetImportAddress(Handle, 'tsxAViewGetScreenPoint');
    @tsxObjectDefine              :=GetImportAddress(Handle, 'tsxObjectDefine');
    @tsxObjectCreate              :=GetImportAddress(Handle, 'tsxObjectCreate');
    @tsxObjectGetDataPtr          :=GetImportAddress(Handle, 'tsxObjectGetDataPtr');
    @tsxObjectGetTypeName         :=GetImportAddress(Handle, 'tsxObjectGetTypeName');
//    @tsxDataDefine                :=GetImportAddress(Handle, 'tsxDataDefine');  {ignore the missing function =( }
    @tsxSobjCreateData            :=GetImportAddress(Handle, 'tsxSobjCreateData');
    @tsxSobjGetData               :=GetImportAddress(Handle, 'tsxSobjGetData');
    @tsxSobjDeleteData            :=GetImportAddress(Handle, 'tsxSobjDeleteData');
    @tsxFILEerror                 :=GetImportAddress(Handle, 'tsxFILEerror');
    @tsxFILEgetc                  :=GetImportAddress(Handle, 'tsxFILEgetc');
    @tsxFILEgets                  :=GetImportAddress(Handle, 'tsxFILEgets');
    @tsxFILEprintf                :=GetImportAddress(Handle, 'tsxFILEprintf');
    @tsxFILEputc                  :=GetImportAddress(Handle, 'tsxFILEputc');
    @tsxFILEputs                  :=GetImportAddress(Handle, 'tsxFILEputs');
    @tsxFILEread                  :=GetImportAddress(Handle, 'tsxFILEread');
    @tsxFILEscanf                 :=GetImportAddress(Handle, 'tsxFILEscanf');
    @tsxFILEseek                  :=GetImportAddress(Handle, 'tsxFILEseek');
    @tsxFILEtell                  :=GetImportAddress(Handle, 'tsxFILEtell');
    @tsxFILEwrite                 :=GetImportAddress(Handle, 'tsxFILEwrite');
    result := True;
  except
    result := False;
  end;
end;

function tsxapiLoadProcs (Handle : HINST): TtSVersion;
var
  FMajVer, FMinVer: smallint;
begin
  result := VER_ts00;
  if tsxapiLoad30Procs(Handle) then
    begin
    result := VER_ts30;
    FMajVer := 0;
    tsxGetTsVersionNbrs(FMajVer, FMinVer);
    if (FMajVer >= 310) then
      //Only initialize these functions if trueSpace 3.1 or higher is detected
      if tsxapiLoad31Procs(Handle) then
        begin
        result := VER_ts31;
        if (FMajVer >= 320) then
          //Only initialize these functions if trueSpace 3.2 or higher is detected
          if tsxapiLoad32Procs(Handle) then
            begin
            result := VER_ts32;
            end;
        end;
    end;
end;

function tsxapiInitialize: TtSVersion;
var
  Handle : HINST;
begin
  tsxapiUninitialize;
  Handle:=LoadLibrary(tsxapi);
  if Handle = 0 then
    begin
      InitError(GetLastError);
    end;
  try
    result := tsxapiLoadProcs(Handle);
  except
    FreeLibrary(Handle);
    raise;
  end;
  tsxapiHandle:=Handle;
end;

procedure tsxapiUninitialize;
begin
  if tsxapiHandle <> 0 then
    begin
      FreeLibrary(tsxapiHandle);
      tsxapiHandle:=0;
    end;
end;

end.
