unit tsxVersionInfo;
///////////////////////////////////////
// tSX Builder                       //
// TtSXVersion and TtrueSpaceVersion //
// components. Simple components to  //
// display the trueSpace version and //
// tSX API version used to compile   //
// the plugin.                       //
///////////////////////////////////////

interface

uses
  classes, windows, tsxTypes, Controls, graphics, StdCtrls, ComCtrls, ExtCtrls, SysUtils;

Type

  TCustomtSXVersion = class(TCustomGroupBox)
  private
    eVerMajor, eVerMinor : TEdit;
    FMajVer, FMinVer: smallint;
    FVersionName : string;
    procedure DisplayVersions;
    procedure CreateLabel(ALeft, ATop : integer; AText : string);
  protected
    procedure SetVersions; virtual;
  public
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
  end;

  TtSXVersion = class(TCustomtSXVersion)
  protected
    procedure SetVersions; override;
  end;

  TtrueSpaceVersion = class(TCustomtSXVersion)
  protected
    procedure SetVersions; override;
  end;

implementation

procedure TCustomtSXVersion.SetVersions;
begin
  {This needs to be overridden}
end;

procedure TtSXVersion.SetVersions;
begin
  FMajVer := tsxVERSION_MAJOR;
  FMinVer := tsxVERSION_MINOR;
  FVersionName := 'eXtension API Version';
end;

procedure TtrueSpaceVersion.SetVersions;
begin
  FMajVer := 0;
  FMinVer := 0;
  if not (csDesigning in ComponentState) then
    tsxGetTsVersionNbrs(FMajVer, FMinVer);
  FVersionName := 'trueSpace Version';
end;

procedure TCustomtSXVersion.DisplayVersions;
begin
  eVerMajor.Text := FloatToStr(FMajVer / 100);
  if FMinVer < 0 then
    if (FMinVer - tsxVERSION_BETA_BASE) < 0 then
      eVerMinor.Text := FloatToStr((FMinVer - tsxVERSION_ALPHA_BASE) / 100) +
        ' alpha'
    else
      eVerMinor.Text := FloatToStr((FMinVer - tsxVERSION_BETA_BASE) / 100) +
        ' beta'
  else
    eVerMinor.Text := FloatToStr(FMinVer / 100);
end;

procedure TCustomtSXVersion.CreateLabel(ALeft, ATop : integer; AText : string);
begin
  with TLabel.Create(Self) do
    begin
    Parent := Self;
    Alignment := taRightJustify;
    Visible := True;
    SetBounds(ALeft, ATop, 42, 13);
    Left := ALeft;
    Top := ATop;
    Caption := AText;
    end;
end;

constructor TCustomtSXVersion.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetVersions;
  Caption := FVersionName;;
  Visible := True;
  SetBounds(Left, Top, 145, 80);
  CreateLabel(24, 32, 'Version:');
  CreateLabel(24, 56, 'Release:');
  eVerMajor := TEdit.Create(self);
  with eVerMajor do
    begin
    Parent := Self;
    Color := clBtnFace;
    Visible := True;
    SetBounds(72, 29, 65, 21);
    Left := 72;
    Top := 29;
    ReadOnly := True;
    TabStop := False;
    end;
  eVerMinor := TEdit.Create(self);
  with eVerMinor do
    begin
    Parent := Self;
    Color := clBtnFace;
    Visible := True;
    SetBounds(72, 53, 65, 21);
    ReadOnly := True;
    TabStop := False;
    end;
  DisplayVersions;
end;

end.
